@@ -105,6 +105,9 @@ Options:\n\
     }
     nargv[nargc] = NULL;
 
+    fflush(stdout);
+    fflush(stderr);
+
     if (pipein && !pipeout) {
 	sts = __pmProcessPipe(&h, "r", PM_EXEC_TOSS_NONE, &pin);
 	printf("__pmProcessPipe(..., \"r\", ...) -> %d", sts);
@@ -132,7 +135,7 @@ Options:\n\
 	    printf(": %s\n", pmErrStr(sts));
 	}
 	else {
-	    if (pmDebugOptions.desperate) printf(" fileno(pin)=%d", fileno(pin));
+	    if (pmDebugOptions.desperate) printf(" fileno(pout)=%d", fileno(pout));
 	    putchar('\n');
 	    printf("--- start pipe input ---\n");
 	    while ((c = fgetc(fin)) != EOF) {
