@@ -12,6 +12,10 @@ _pcp_complete()
 
     # Register arguments
     case $cmd in
+    pcp2arrow)
+        all_args="ahLKcCVASTOstrijJ89nZzo"
+        arg_regex="-[ahKcCSstiJ89o]"
+    ;;
     pcp2elasticsearch)
         all_args="ahLKcCeVHGASTmOstRrIijJ4589nNvP0qQbByYgXxp"
         arg_regex="-[ahKceASTOstZiJ489NP0qQbByYgXxp]"
@@ -65,7 +69,7 @@ _pcp_complete()
         arg_regex="-[mst]"
     ;;
     pmie)
-        all_args="AabCcdeFfhjlmnOPqSTtUVvWXxZz"
+        all_args="AabCcdeFfhjlmnOoPqSTtUVvWXxZz"
         arg_regex="-[AachljnOSTtUZ]"
     ;;
     pmie2col)
