@@ -288,6 +288,17 @@ typedef struct pmdaInterface {
 
 } pmdaInterface;
 
+/* comm(unication) flags */
+#define PMDA_FLAG_AUTHORIZE	(1<<2)	/* authentication support */
+#define PMDA_FLAG_CONTAINER	(1<<6)	/* container name support */
+
+/* communication attributes (mirrored from libpcp.h) */
+#define PMDA_ATTR_USERNAME   5  /* username (sasl) */
+#define PMDA_ATTR_USERID	11	/* uid - user identifier (posix) */
+#define PMDA_ATTR_GROUPID	12	/* gid - group identifier (posix) */
+#define PMDA_ATTR_PROCESSID	14	/* pid - process identifier (posix) */
+#define PMDA_ATTR_CONTAINER	15	/* container name */
+
 /*
  * PM_CONTEXT_LOCAL support
  */
@@ -296,6 +307,7 @@ typedef struct __pmDSO {
     char		*name;
     char		*init;
     void		*handle;
+    int			ctx_last_prof;	/* ctx that sent last profile */
     pmdaInterface	dispatch;
 } __pmDSO;
 
@@ -401,7 +413,7 @@ typedef struct pmdaOptions {
  * pmdaOpenLog
  *	Redirects stderr to the logfile.
  *
- * pmdaSetFlags / pmdaExtSetFlags
+ * pmdaSetFlags / pmdaExtSetFlags / pmdaSetCommFlags
  *      Allow behaviour flags to be set to enable features, such as to request
  *      libpcp_pmda internally use direct or hashed PMID metric table mapping.
  *      Can be called multiple times - effects are cumulative - no flag can be
@@ -410,6 +422,10 @@ typedef struct pmdaOptions {
  *      flags involving early setup (such as metric table hashing), otherwise
  *      can be called at any time (such as for namespace change notification).
  *
+ * pmdaSetData / pmdaExtSetData / pmdaExtGetData
+ *	Private data hook get/set for (esp. DSO) PMDAs that is accessible via
+ *	the callbacks through the pmdaExt structure.
+ *
  * pmdaInit
  *      Further initialises the pmdaExt structure with the instance domain and
  *      metric structures. Unique identifiers are applied to each instance 
@@ -469,8 +485,12 @@ PMDA_CALL extern void pmdaUsageMessage(pmdaOptions *);
 PMDA_CALL extern void pmdaDaemon(pmdaInterface *, int, const char *, int , const char *, const char *);
 PMDA_CALL extern void pmdaDSO(pmdaInterface *, int, char *, char *);
 PMDA_CALL extern void pmdaOpenLog(pmdaInterface *);
+PMDA_CALL extern void *pmdaExtGetData(pmdaExt *);
+PMDA_CALL extern void pmdaExtSetData(pmdaExt *, void *);
+PMDA_CALL extern void pmdaSetData(pmdaInterface *, void *);
 PMDA_CALL extern void pmdaExtSetFlags(pmdaExt *, int);
 PMDA_CALL extern void pmdaSetFlags(pmdaInterface *, int);
+PMDA_CALL extern void pmdaSetCommFlags(pmdaInterface *, int);
 PMDA_CALL extern void pmdaInit(pmdaInterface *, pmdaIndom *, int, pmdaMetric *, int);
 PMDA_CALL extern void pmdaConnect(pmdaInterface *);
 
@@ -506,7 +526,7 @@ PMDA_CALL extern void pmdaSetLabelCallBack(pmdaInterface *, pmdaLabelCallBack);
  *	Return the help text for the metric or instance domain.
  *
  * pmdaStore
- *	Store a value into a metric. This is a no-op.
+ *	Store a value into a metric. This sets the context number.
  *
  * pmdaPMID
  *	Return the PMID for a named metric within a dynamic subtree
@@ -642,6 +662,10 @@ PMDA_CALL extern int pmdaTreeChildren(pmdaNameSpace *, const char *, int, char *
 PMDA_CALL extern void pmdaTreeRebuildHash(pmdaNameSpace *, int);
 PMDA_CALL extern int pmdaTreeSize(pmdaNameSpace *);
 
+PMDA_CALL extern int pmdaTreeCreate(pmdaNameSpace **);
+PMDA_CALL extern int pmdaTreeInsert(pmdaNameSpace *, pmID, const char *);
+PMDA_CALL extern void pmdaTreeRelease(pmdaNameSpace *);
+
 /*
  * PMDA instance domain cache support
  *
@@ -669,6 +693,9 @@ PMDA_CALL extern int pmdaTreeSize(pmdaNameSpace *);
  * pmdaCachePurge
  *	cull inactive entries
  *
+ * pmdaCachePurgeCallback
+ *	cull inactive entries and invoke callback for private data
+ *
  * pmdaCacheResize
  *	set the maximum instance identifier
  */
@@ -679,6 +706,7 @@ PMDA_CALL extern int pmdaCacheLookupName(pmInDom, const char *, int *, void **);
 PMDA_CALL extern int pmdaCacheLookupKey(pmInDom, const char *, int, const void *, char **, int *, void **);
 PMDA_CALL extern int pmdaCacheOp(pmInDom, int);
 PMDA_CALL extern int pmdaCachePurge(pmInDom, time_t);
+PMDA_CALL extern int pmdaCachePurgeCallback(pmInDom, time_t, void (*)(void *));
 PMDA_CALL extern int pmdaCacheResize(pmInDom, int);
 
 #define PMDA_CACHE_LOAD			1
@@ -768,6 +796,9 @@ PMDA_CALL extern int pmdaEventResetHighResArray(int);
 PMDA_CALL extern int pmdaEventReleaseHighResArray(int);
 PMDA_CALL extern int pmdaEventAddHighResRecord(int, struct timespec *, int);
 PMDA_CALL extern int pmdaEventAddHighResMissedRecord(int, struct timespec *, int);
+PMDA_CALL extern int pmdaEventAddHighResParam(int, pmID, int, pmAtomValue *);
+PMDA_CALL extern pmHighResEventArray *pmdaEventGetHighResAddr(int);
+/* old names retained for backwards compatibility */
 PMDA_CALL extern int pmdaEventHighResAddParam(int, pmID, int, pmAtomValue *);
 PMDA_CALL extern pmHighResEventArray *pmdaEventHighResGetAddr(int);
 
