@@ -1,13 +1,13 @@
 /*
- * Copyright (c) 2012-2018,2020-2021 Red Hat.
+ * Copyright (c) 2012-2018,2020-2022 Red Hat.
  * Copyright (c) 2007-2008 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002,2004,2006,2008 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
@@ -50,7 +50,8 @@ static int		*contexts_map;
  * Special sentinals for contexts_map[] ...
  */
 #define MAP_FREE	-1		/* contexts[i] can be reused */
-#define MAP_TEARDOWN	-2		/* contexts[i] is being destroyed */
+#define MAP_INIT	-2		/* contexts[i] is being setup */
+#define MAP_TEARDOWN	-3		/* contexts[i] is being destroyed */
 
 #ifdef PM_MULTI_THREAD
 #ifdef HAVE___THREAD
@@ -161,22 +162,22 @@ waitawhile(__pmPMCDCtl *ctl)
 	    n_backoff = 5;
 	    backoff = def_backoff;
 	}
-	PM_UNLOCK(contexts_lock);
 	if (bad) {
+	    PM_UNLOCK(contexts_lock);
 	    pmNotifyErr(LOG_WARNING,
 			 "pmReconnectContext: ignored bad PMCD_RECONNECT_TIMEOUT = '%s'\n",
 			 q);
+	    PM_LOCK(contexts_lock);
 	}
 	if (q != NULL)
 	    free(q);
     }
-    else
-	PM_UNLOCK(contexts_lock);
     if (ctl->pc_timeout == 0)
 	ctl->pc_timeout = 1;
     else if (ctl->pc_timeout < n_backoff)
 	ctl->pc_timeout++;
     ctl->pc_again = time(NULL) + backoff[ctl->pc_timeout-1];
+    PM_UNLOCK(contexts_lock);
 }
 
 /*
@@ -235,9 +236,9 @@ pmGetHostName(int handle, char *buf, int buflen)
     __pmContext *ctxp;
     const char	*name;
     pmID	pmid;
-    pmResult	*resp;
-    int		save_handle;
+    __pmResult	*resp;
     __pmContext	*save_ctxp;
+    int		save_handle;
     int		sts = 0;
 
     PM_INIT_LOCKS();
@@ -277,10 +278,10 @@ pmGetHostName(int handle, char *buf, int buflen)
 		    (resp->vset[0]->valfmt == PM_VAL_DPTR || resp->vset[0]->valfmt == PM_VAL_SPTR)) {
 		    /* pmcd.hostname present */
 		    strncpy(buf, resp->vset[0]->vlist[0].value.pval->vbuf, buflen);
-		    pmFreeResult(resp);
+		    __pmFreeResult(resp);
 		    break;
 		}
-		pmFreeResult(resp);
+		__pmFreeResult(resp);
 		/* FALLTHROUGH */
 	    }
 
@@ -613,6 +614,11 @@ __pmFindOrOpenArchive(__pmContext *ctxp, const char *name, int multi_arch)
     }
     sts = __pmLogOpen(name, ctxp);
     if (sts < 0) {
+	if (pmDebugOptions.log && pmDebugOptions.desperate) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmFindOrOpenArchive(..., %s, ...): __pmLogOpen: %s\n",
+		name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
 #ifdef PM_MULTI_THREAD
 	__pmDestroyMutex(&lcp->lc_lock);
 #endif
@@ -648,7 +654,7 @@ addName(const char *dirname, char *list, size_t *listsize,
     /* Allocate more space */
     if (list == NULL) {
 	if ((list = malloc(dirsize + itemsize + 1)) == NULL) {
-	    pmNoMem("initArchive", itemsize + 1, PM_FATAL_ERR);
+	    pmNoMem("initarchive", itemsize + 1, PM_FATAL_ERR);
 	    /* NOTREACHED */
 	}
 	*listsize = 0;
@@ -656,7 +662,7 @@ addName(const char *dirname, char *list, size_t *listsize,
     else {
 	/* The comma goes where the previous nul was */
 	if ((list_new = realloc(list, dirsize + *listsize + itemsize + 1)) == NULL) {
-	    pmNoMem("initArchive", *listsize + itemsize + 1, PM_FATAL_ERR);
+	    pmNoMem("initarchive", *listsize + itemsize + 1, PM_FATAL_ERR);
 	    /* NOTREACHED */
 	}
 	list = list_new;
@@ -709,7 +715,7 @@ expandArchiveList(const char *names)
 	 * We need nul terminated copy of the name fpr opendir(3).
 	 */
 	if ((dirname = malloc(length + 1)) == NULL) {
-	    pmNoMem("initArchive", length + 1, PM_FATAL_ERR);
+	    pmNoMem("initarchive", length + 1, PM_FATAL_ERR);
 	    /* NOTREACHED */
 	}
 	memcpy(dirname, current, length);
@@ -796,7 +802,7 @@ initarchive(__pmContext	*ctxp, const char *name)
 
     /* Allocate the structure for overal control of the archive(s). */
     if ((ctxp->c_archctl = (__pmArchCtl *)malloc(sizeof(__pmArchCtl))) == NULL) {
-	pmNoMem("initArchive", sizeof(__pmArchCtl), PM_FATAL_ERR);
+	pmNoMem("initarchive", sizeof(__pmArchCtl), PM_FATAL_ERR);
 	/* NOTREACHED */
     }
     acp = ctxp->c_archctl;
@@ -806,6 +812,7 @@ initarchive(__pmContext	*ctxp, const char *name)
     acp->ac_log_list = NULL;
     acp->ac_log = NULL;
     acp->ac_mark_done = 0;
+    acp->ac_flags = ctxp->c_flags;
 
     /*
      * The list of names may contain one or more directories. Examine the
@@ -837,8 +844,14 @@ initarchive(__pmContext	*ctxp, const char *name)
 	 * if necessary
 	 */
 	sts = __pmFindOrOpenArchive(ctxp, current, multi_arch);
-	if (sts < 0)
+	if (sts < 0) {
+	    if (pmDebugOptions.log && pmDebugOptions.desperate) {
+		char	errmsg[PM_MAXERRMSGLEN];
+		fprintf(stderr, "initarchive(..., %s, ...): __pmFindOrOpenArchive: %s\n",
+		    name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	    }
 	    goto error;
+	}
 
 	/*
 	 * Obtain the start time of this archive. The end time could change
@@ -876,27 +889,27 @@ initarchive(__pmContext	*ctxp, const char *name)
 						   (acp->ac_num_logs + 1) *
 						   sizeof(*acp->ac_log_list));
 	    if (list_new == NULL) {
-		pmNoMem("initArchive: list_new",
+		pmNoMem("initarchive: list_new",
 			  (acp->ac_num_logs + 1) * sizeof(*acp->ac_log_list),
 			  PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
 	    acp->ac_log_list = list_new;
 	    if ((mlcp = (__pmMultiLogCtl *)malloc(sizeof(__pmMultiLogCtl))) == NULL) {
-		pmNoMem("initArchive: __pmMultiLogCtl", sizeof(__pmMultiLogCtl), PM_FATAL_ERR);
+		pmNoMem("initarchive: __pmMultiLogCtl", sizeof(__pmMultiLogCtl), PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
 	    if ((mlcp->name = strdup(current)) == NULL) {
-		pmNoMem("initArchive: name", strlen(current) + 1, PM_FATAL_ERR);
+		pmNoMem("initarchive: name", strlen(current) + 1, PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
 	    if ((mlcp->hostname = strdup(lp->hostname)) == NULL) {
-		pmNoMem("initArchive: hostname", strlen(lp->hostname) + 1, PM_FATAL_ERR);
+		pmNoMem("initarchive: hostname", strlen(lp->hostname) + 1, PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
 	    if (lp->timezone != NULL) {
 		if ((mlcp->timezone = strdup(lp->timezone)) == NULL) {
-		    pmNoMem("initArchive: timezone", strlen(lp->timezone) + 1, PM_FATAL_ERR);
+		    pmNoMem("initarchive: timezone", strlen(lp->timezone) + 1, PM_FATAL_ERR);
 		    /* NOTREACHED */
 		}
 	    }
@@ -904,7 +917,7 @@ initarchive(__pmContext	*ctxp, const char *name)
 		mlcp->timezone = NULL;
 	    if (lp->zoneinfo != NULL) {
 		if ((mlcp->zoneinfo = strdup(lp->zoneinfo)) == NULL) {
-		    pmNoMem("initArchive: zoneinfo", strlen(lp->zoneinfo) + 1, PM_FATAL_ERR);
+		    pmNoMem("initarchive: zoneinfo", strlen(lp->zoneinfo) + 1, PM_FATAL_ERR);
 		    /* NOTREACHED */
 		}
 	    }
@@ -941,11 +954,23 @@ initarchive(__pmContext	*ctxp, const char *name)
 	 * implementation, open the first archive and switch to the first volume.
 	 */
 	sts = __pmLogChangeArchive(ctxp, 0);
-	if (sts < 0)
+	if (sts < 0) {
+	    if (pmDebugOptions.log && pmDebugOptions.desperate) {
+		char	errmsg[PM_MAXERRMSGLEN];
+		fprintf(stderr, "initarchive(..., %s, ...): __pmLogChangeArchive: %s\n",
+		    name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	    }
 	    goto error;
+	}
 	sts = __pmLogChangeVol(acp, acp->ac_log->minvol);
-	if (sts < 0)
+	if (sts < 0) {
+	    if (pmDebugOptions.log && pmDebugOptions.desperate) {
+		char	errmsg[PM_MAXERRMSGLEN];
+		fprintf(stderr, "initarchive(..., %s, ...): __pmLogChangeVol(..., %d, ...): %s\n",
+		    name, acp->ac_log->minvol, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	    }
 	    goto error;
+	}
     }
 
     /* start after header + label record + trailer */
@@ -1097,13 +1122,14 @@ INIT_CONTEXT:
     PM_TPD(curr_handle) = new->c_handle = ++last_handle;
     new->c_slot = ctxnum;
     contexts[ctxnum] = &being_initialized;
-    contexts_map[ctxnum] = last_handle;
+    contexts_map[ctxnum] = MAP_INIT;
     PM_UNLOCK(contexts_lock);
     /* c_lock not re-initialized, created once from initcontextlock() above */
     new->c_type = (type & PM_CONTEXT_TYPEMASK);
     new->c_mode = 0;
     new->c_origin.sec = new->c_origin.nsec = 0;
-    new->c_delta = 0;
+    new->c_delta.sec = new->c_delta.nsec = 0;
+    new->c_direction = 0;
     new->c_sent = 0;
     new->c_flags = (type & ~PM_CONTEXT_TYPEMASK);
     if ((new->c_instprof = (pmProfile *)calloc(1, sizeof(pmProfile))) == NULL) {
@@ -1179,7 +1205,7 @@ INIT_CONTEXT:
     }
     else if (new->c_type == PM_CONTEXT_ARCHIVE) {
         /*
-         * Unlock during the archive inital file opens, which can take
+         * Unlock during the archive initial file opens, which can take
          * a noticeable amount of time, esp. for multi-archives.  This
          * is OK because no other thread can validly touch our
          * partly-initialized context.
@@ -1200,6 +1226,7 @@ INIT_CONTEXT:
        battle station ^W context. */
     PM_LOCK(contexts_lock);
     contexts[ctxnum] = new;
+    contexts_map[ctxnum] = new->c_handle;
     PM_UNLOCK(contexts_lock);
 
     /* return the handle to the new (current) context */
@@ -1352,19 +1379,15 @@ pmapi_return:
     return sts;
 }
 
-/*
- * TODO - move all the context copying earlier to a local temporary
- * under the protection of oldcon->c_lock ... then release oldcon->c_lock
- * and after the new one is created lock newcon->c_lock and cherry-pick
- * copy from the local temporary to the new __pmContext
- */
 int
 pmDupContext(void)
 {
-    int			sts, oldtype;
+    int			sts, oldtype, old_c_type;
     int			old, new = -1;
     char		hostspec[4096];
+    char		archivename[MAXPATHLEN];
     __pmContext		*newcon, *oldcon;
+    __pmContext		tmpcon = { 0 };		/* used to build new context */
     __pmMultiLogCtl	*newmlcp, *oldmlcp;
     pmInDomProfile	*q, *p, *p_end;
     int			i;
@@ -1389,59 +1412,50 @@ pmDupContext(void)
     }
 
     oldcon = contexts[ctxnum];
-    PM_UNLOCK(contexts_lock);
-    oldtype = oldcon->c_type | oldcon->c_flags;
-    if (oldcon->c_type == PM_CONTEXT_HOST) {
-	__pmUnparseHostSpec(oldcon->c_pmcd->pc_hosts,
-			oldcon->c_pmcd->pc_nhosts, hostspec, sizeof(hostspec));
-	new = pmNewContext(oldtype, hostspec);
-    }
-    else if (oldcon->c_type == PM_CONTEXT_LOCAL)
-	new = pmNewContext(oldtype, NULL);
-    else if (oldcon->c_type == PM_CONTEXT_ARCHIVE)
-	new = pmNewContext(oldtype, oldcon->c_archctl->ac_log->name);
-    if (new < 0) {
-	/* failed to connect or out of memory */
-	sts = new;
-	goto done;
-    }
-    PM_LOCK(contexts_lock);
-    if ((ctxnum = map_handle(new)) < 0) {
-	sts = PM_ERR_NOCONTEXT;
-	PM_UNLOCK(contexts_lock);
-	goto done;
-    }
-    newcon = contexts[ctxnum];
     PM_LOCK(oldcon->c_lock);
-    PM_LOCK(newcon->c_lock);
     PM_UNLOCK(contexts_lock);
     /*
-     * cherry-pick the fields of __pmContext that need to be copied
+     * Build a skeletal __pmContext in tmpcon with oldcon->c_lock held.
      */
-    newcon->c_mode = oldcon->c_mode;
-    newcon->c_origin = oldcon->c_origin;
-    newcon->c_delta = oldcon->c_delta;
-    newcon->c_flags = oldcon->c_flags;
+    tmpcon.c_mode = oldcon->c_mode;
+    tmpcon.c_origin = oldcon->c_origin;
+    tmpcon.c_delta = oldcon->c_delta;
+    tmpcon.c_direction = oldcon->c_direction;
+    tmpcon.c_flags = oldcon->c_flags;
 
     /* clone the per-domain profiles (if any) */
+    if ((tmpcon.c_instprof = (pmProfile *)calloc(1, sizeof(pmProfile))) == NULL) {
+	sts = -oserror();
+	goto setup_fail;
+    }
+    tmpcon.c_instprof->state = oldcon->c_instprof->state;
     if (oldcon->c_instprof->profile_len > 0) {
-	newcon->c_instprof->profile = (pmInDomProfile *)malloc(
+	tmpcon.c_instprof->profile_len = oldcon->c_instprof->profile_len;
+	tmpcon.c_instprof->profile = (pmInDomProfile *)malloc(
 	    oldcon->c_instprof->profile_len * sizeof(pmInDomProfile));
-	if (newcon->c_instprof->profile == NULL) {
+	if (tmpcon.c_instprof->profile == NULL) {
 	    sts = -oserror();
-	    goto done_locked;
+	    free(tmpcon.c_instprof);
+	    tmpcon.c_instprof = NULL;
+	    goto setup_fail;
 	}
-	memcpy(newcon->c_instprof->profile, oldcon->c_instprof->profile,
+	memcpy(tmpcon.c_instprof->profile, oldcon->c_instprof->profile,
 	    oldcon->c_instprof->profile_len * sizeof(pmInDomProfile));
 	p = oldcon->c_instprof->profile;
 	p_end = p + oldcon->c_instprof->profile_len;
-	q = newcon->c_instprof->profile;
+	q = tmpcon.c_instprof->profile;
 	for (; p < p_end; p++, q++) {
 	    if (p->instances) {
 		q->instances = (int *)malloc(p->instances_len * sizeof(int));
 		if (q->instances == NULL) {
 		    sts = -oserror();
-		    goto done_locked;
+		    /* free any allocated profiles up to the failue */
+		    for (q--; q >= tmpcon.c_instprof->profile; q--)
+			free(q->instances);
+		    free(tmpcon.c_instprof->profile);
+		    free(tmpcon.c_instprof);
+		    tmpcon.c_instprof = NULL;
+		    goto setup_fail;
 		}
 		memcpy(q->instances, p->instances,
 		    p->instances_len * sizeof(int));
@@ -1449,47 +1463,38 @@ pmDupContext(void)
 	}
     }
 
-    /*
-     * The call to pmNewContext (above) should have connected to the pmcd.
-     * Make sure the new profile will be sent before the next fetch.
-     */
-    newcon->c_sent = 0;
-
     /* clone the archive control struct, if any */
-    if (newcon->c_archctl != NULL)
-	__pmArchCtlFree(newcon->c_archctl); /* will allocate a new one below */
     if (oldcon->c_archctl != NULL) {
-	if ((newcon->c_archctl = (__pmArchCtl *)malloc(sizeof(__pmArchCtl))) == NULL) {
+	if ((tmpcon.c_archctl = (__pmArchCtl *)malloc(sizeof(__pmArchCtl))) == NULL) {
 	    sts = -oserror();
-	    goto done_locked;
+	    goto setup_fail;
 	}
-	*newcon->c_archctl = *oldcon->c_archctl;	/* struct assignment */
+	*tmpcon.c_archctl = *oldcon->c_archctl;	/* struct assignment */
 	/*
 	 * Need to make hash list and read cache independent in case oldcon
 	 * is subsequently closed via pmDestroyContext() and don't want
 	 * __pmFreeInterpData() to trash our hash list and read cache.
 	 * Start with an empty hash list and read cache for the dup'd context.
 	 */
-	newcon->c_archctl->ac_pmid_hc.nodes = 0;
-	newcon->c_archctl->ac_pmid_hc.hsize = 0;
-	newcon->c_archctl->ac_cache = NULL;
+	tmpcon.c_archctl->ac_pmid_hc.nodes = 0;
+	tmpcon.c_archctl->ac_pmid_hc.hsize = 0;
+	tmpcon.c_archctl->ac_cache = NULL;
 
 	/*
 	 * Need a new ac_mfp, but pointing at the same volume so ac_offset
 	 * is OK
 	 */
-	newcon->c_archctl->ac_mfp = NULL;
-	vol = newcon->c_archctl->ac_curvol;
-	newcon->c_archctl->ac_curvol = -1;
-	sts = __pmLogChangeVol(newcon->c_archctl, vol);
+	tmpcon.c_archctl->ac_mfp = NULL;
+	vol = oldcon->c_archctl->ac_curvol;
+	tmpcon.c_archctl->ac_curvol = -1;
+	sts = __pmLogChangeVol(tmpcon.c_archctl, vol);
 	if  (sts < 0) {
 	    if (pmDebugOptions.log) {
 		char	errmsg[PM_MAXERRMSGLEN];
-		fprintf(stderr, "pmDupContext: __pmLogChangeVol(newcon, %d) failed: %s\n",
+		fprintf(stderr, "pmDupContext: __pmLogChangeVol(tmpcon, %d) failed: %s\n",
 		    vol, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
 	    }
-	    free(newcon->c_archctl);
-	    goto done_locked;
+	    goto setup_fail;
 	}
 
 	/*
@@ -1499,55 +1504,129 @@ pmDupContext(void)
 	if (oldcon->c_archctl->ac_log_list != NULL) {
 	    size_t size = oldcon->c_archctl->ac_num_logs *
 		sizeof(*oldcon->c_archctl->ac_log_list);
-	    if ((newcon->c_archctl->ac_log_list = malloc(size)) == NULL) {
+	    if ((tmpcon.c_archctl->ac_log_list = malloc(size)) == NULL) {
 		sts = -oserror();
-		free(newcon->c_archctl);
-		goto done_locked;
+		goto setup_fail;
 	    }
 	    /* We need to duplicate each ac_log_list entry. */
-	    for (i = 0; i < newcon->c_archctl->ac_num_logs; i++) {
-		newcon->c_archctl->ac_log_list[i] =
-		    malloc(sizeof(*newcon->c_archctl->ac_log_list[i]));
-		newmlcp = newcon->c_archctl->ac_log_list[i];
+	    for (i = 0; i < tmpcon.c_archctl->ac_num_logs; i++) {
+		tmpcon.c_archctl->ac_log_list[i] =
+		    malloc(sizeof(*tmpcon.c_archctl->ac_log_list[i]));
+		newmlcp = tmpcon.c_archctl->ac_log_list[i];
 		oldmlcp = oldcon->c_archctl->ac_log_list[i];
 		*newmlcp = *oldmlcp;
 		/*
 		 * We need to duplicate the name and the hostname of each
 		 * archive in the list.
 		 */
-		if ((newmlcp->name = strdup (newmlcp->name)) == NULL) {
+		if ((newmlcp->name = strdup(newmlcp->name)) == NULL) {
 		    sts = -oserror();
-		    goto done_locked;
+		    goto setup_fail;
 		}
-		if ((newmlcp->hostname = strdup (newmlcp->hostname)) == NULL) {
+		if ((newmlcp->hostname = strdup(newmlcp->hostname)) == NULL) {
 		    sts = -oserror();
-		    goto done_locked;
+		    goto setup_fail;
 		}
 		if (newmlcp->timezone != NULL) {
-		    if ((newmlcp->timezone = strdup (newmlcp->timezone)) == NULL) {
+		    if ((newmlcp->timezone = strdup(newmlcp->timezone)) == NULL) {
 			sts = -oserror();
-			goto done_locked;
+			goto setup_fail;
 		    }
 		}
 		if (newmlcp->zoneinfo != NULL) {
-		    if ((newmlcp->zoneinfo = strdup (newmlcp->zoneinfo)) == NULL) {
+		    if ((newmlcp->zoneinfo = strdup(newmlcp->zoneinfo)) == NULL) {
 			sts = -oserror();
-			goto done_locked;
+			goto setup_fail;
 		    }
 		}
 	    }
 	    /* We need to bump up the reference count of the ac_log. */
-	    if (newcon->c_archctl->ac_log != NULL)
-		++newcon->c_archctl->ac_log->refcnt;
+	    if (tmpcon.c_archctl->ac_log != NULL)
+		++tmpcon.c_archctl->ac_log->refcnt;
 	}
     }
 
-    sts = new;
+    oldtype = oldcon->c_type | oldcon->c_flags;
+    old_c_type = oldcon->c_type;
+    if (old_c_type == PM_CONTEXT_HOST)
+	__pmUnparseHostSpec(oldcon->c_pmcd->pc_hosts,
+			oldcon->c_pmcd->pc_nhosts, hostspec, sizeof(hostspec));
+    else if (old_c_type == PM_CONTEXT_ARCHIVE) {
+	strncpy(archivename, oldcon->c_archctl->ac_log->name, MAXPATHLEN-1);
+	archivename[MAXPATHLEN-1] = '\0';
+    }
 
-done_locked:
+    /*
+     * we now have local copies of everything we need to preserve/use
+     * from the old __pmContext
+     */
     PM_UNLOCK(oldcon->c_lock);
+
+    if (old_c_type == PM_CONTEXT_HOST)
+	new = pmNewContext(oldtype, hostspec);
+    else if (old_c_type == PM_CONTEXT_LOCAL)
+	new = pmNewContext(oldtype, NULL);
+    else if (old_c_type == PM_CONTEXT_ARCHIVE)
+	new = pmNewContext(oldtype, archivename);
+    if (new < 0) {
+	/* failed to connect or out of memory */
+	sts = new;
+	goto new_fail;
+    }
+    PM_LOCK(contexts_lock);
+    if ((ctxnum = map_handle(new)) < 0) {
+	sts = PM_ERR_NOCONTEXT;
+	PM_UNLOCK(contexts_lock);
+	goto done;
+    }
+    newcon = contexts[ctxnum];
+    PM_LOCK(newcon->c_lock);
+    PM_UNLOCK(contexts_lock);
+    /*
+     * cherry-pick the fields of __pmContext that need to be copied
+     */
+    newcon->c_mode = tmpcon.c_mode;
+    newcon->c_origin = tmpcon.c_origin;
+    newcon->c_delta = tmpcon.c_delta;
+    newcon->c_direction = tmpcon.c_direction;
+    newcon->c_flags = tmpcon.c_flags;
+    // newcon->c_instprof->state = tmpcon.c_instprof->state;
+    newcon->c_instprof->profile_len = tmpcon.c_instprof->profile_len;
+    newcon->c_instprof->profile = tmpcon.c_instprof->profile;
+    free (tmpcon.c_instprof);
+
+    /*
+     * For a HOST context, the call to pmNewContext (above) should
+     * have connected to the pmcd.
+     * Make sure the new profile will be sent before the next fetch.
+     */
+    newcon->c_sent = 0;
+
+    /* clone the archive control struct, if any */
+    if (newcon->c_archctl != NULL)
+	__pmArchCtlFree(newcon->c_archctl); /* will allocate a new one below */
+    newcon->c_archctl = tmpcon.c_archctl;
     PM_UNLOCK(newcon->c_lock);
 
+    sts = new;
+    goto done;
+
+setup_fail:
+    /* failure, get here with c_lock held on oldcon */
+    PM_UNLOCK(oldcon->c_lock);
+    /* FALLTHROUGH */
+
+new_fail:
+    /*
+     * failure, get here with no locks held, but tmpcon may have
+     * malloc'd memory we need to free
+     */
+    if (tmpcon.c_archctl != NULL)
+	free(tmpcon.c_archctl);
+    if (tmpcon.c_instprof != NULL)
+	__pmFreeProfile(tmpcon.c_instprof);
+    /* FALLTHROUGH */
+
 done:
     /* return an error code, or the handle for the new context */
     if (sts < 0 && new >= 0) {
@@ -1563,7 +1642,7 @@ done:
     }
 
 pmapi_return:
-
+    /* no locks held */
     if (pmDebugOptions.pmapi) {
 	fprintf(stderr, ":> returns ");
 	if (sts >= 0)
@@ -1736,11 +1815,15 @@ __pmDumpContext(FILE *f, int context, pmInDom indom)
 {
     int			i, j;
     __pmContext		*con;
+    int			idx;
 
     PM_INIT_LOCKS();
 
+    PM_LOCK(contexts_lock);
+    idx = map_handle_nolock(PM_TPD(curr_handle));
+    PM_UNLOCK(contexts_lock);
     fprintf(f, "Dump Contexts: current -> contexts[%d] handle %d\n",
-	map_handle_nolock(PM_TPD(curr_handle)), PM_TPD(curr_handle));
+	idx, PM_TPD(curr_handle));
     if (PM_TPD(curr_handle) < 0)
 	return;
 
@@ -1812,7 +1895,9 @@ __pmDumpContext(FILE *f, int context, pmInDom indom)
 	    if (con->c_type == PM_CONTEXT_HOST || con->c_type == PM_CONTEXT_ARCHIVE) {
 		fprintf(f, " origin=%" FMT_INT64 ".%09d",
 		    con->c_origin.sec, con->c_origin.nsec);
-		fprintf(f, " delta=%d\n", con->c_delta);
+		fprintf(f, " delta=%s%" FMT_INT64 ".%09d\n",
+		    con->c_direction < 0 ? "-" : "",
+		    con->c_delta.sec, con->c_delta.nsec);
 	    }
 	    __pmDumpProfile(f, indom, con->c_instprof);
 	}
@@ -1846,8 +1931,8 @@ char *
 __pmIsLogCtlLock(void *lock)
 {
     int		i;
-    char	*result = NULL;
     int		reslen = 0;
+    char	*result = NULL;
     __pmContext	*ctxp;
 
     for (i = 0; i < contexts_len+1; i++) {
@@ -1863,26 +1948,28 @@ __pmIsLogCtlLock(void *lock)
 	    /* this should not happen, just being careful */
 	    continue;
 	if ((void *)&ctxp->c_archctl->ac_log->lc_lock == lock) {
-	    char	number[10];
-	    pmsprintf(number, sizeof(number), "%d", ctxp->c_handle);
+	    char	*result_new;
+	    char	number[16];
+	    int		numlen;
+
+	    numlen = pmsprintf(number, sizeof(number), "%d", ctxp->c_handle);
 	    if (reslen == 0) {
-		reslen = strlen(number)+1;
+		reslen = numlen + 1;
 		if ((result = malloc(reslen)) == NULL) {
 		    pmNoMem("__pmIsLogCtlLock: malloc", reslen, PM_FATAL_ERR);
 		    /* NOTREACHED */
 		}
-		strncpy(result, number, strlen(number)+1);
+		strncpy(result, number, numlen + 1);
 	    }
 	    else {
-		char	*result_new;
-		reslen += 1 + strlen(number) + 1;
+		reslen += 1 + numlen + 1;
 		if ((result_new = (char *)realloc(result, reslen)) == NULL) {
 		    pmNoMem("__pmIsLogCtlLock: realloc", reslen, PM_FATAL_ERR);
 		    /* NOTREACHED */
 		}
 		result = result_new;
 		strncat(result, ",", 2);
-		strncat(result, number, strlen(number)+1);
+		strncat(result, number, numlen + 1);
 	    }
 	}
     }
