@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2012-2015 Red Hat.
+ * Copyright (c) 2012-2015,2021 Red Hat.
  * Copyright (c) 1995 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
@@ -18,102 +18,6 @@
 #include "libpcp.h"
 #include "internal.h"
 
-/*
- * PDU for id list (PDU_PMNS_IDS)
- */
-typedef struct {
-    __pmPDUHdr   hdr;
-    int		sts;      /* to encode status of pmns op */
-    int		numids;
-    pmID        idlist[1];
-} idlist_t;
-
-void
-__pmDumpIDList(FILE *f, int numids, const pmID idlist[])
-{
-    int		i;
-    char	strbuf[20];
-
-    fprintf(f, "IDlist dump: numids = %d\n", numids);
-    for (i = 0; i < numids; i++)
-	fprintf(f, "  PMID[%d]: 0x%08x %s\n", i, idlist[i], pmIDStr_r(idlist[i], strbuf, sizeof(strbuf)));
-}
-
-/*
- * Send a PDU_PMNS_IDS across socket.
- */
-int
-__pmSendIDList(int fd, int from, int numids, const pmID idlist[], int sts)
-{
-    idlist_t	*ip;
-    int		need;
-    int		j;
-    int		lsts;
-
-    if (pmDebugOptions.pmns) {
-        fprintf(stderr, "__pmSendIDList\n");
-	__pmDumpIDList(stderr, numids, idlist);
-    }
-
-    need = (int)(sizeof(idlist_t) + (numids-1) * sizeof(idlist[0]));
-
-    if ((ip = (idlist_t *)__pmFindPDUBuf(need)) == NULL)
-	return -oserror();
-    ip->hdr.len = need;
-    ip->hdr.type = PDU_PMNS_IDS;
-    ip->hdr.from = from;
-    ip->sts = htonl(sts);
-    ip->numids = htonl(numids);
-    for (j = 0; j < numids; j++) {
-	ip->idlist[j] = __htonpmID(idlist[j]);
-    }
-
-    lsts = __pmXmitPDU(fd, (__pmPDU *)ip);
-    __pmUnpinPDUBuf(ip);
-    return lsts;
-}
-
-/*
- * Decode a PDU_PMNS_IDS
- * Assumes that we have preallocated idlist prior to this call
- * (i.e. we know how many should definitely be coming over)
- * Returns 0 on success.
- */
-int
-__pmDecodeIDList(__pmPDU *pdubuf, int numids, pmID idlist[], int *sts)
-{
-    idlist_t	*idlist_pdu;
-    char	*pdu_end;
-    int		nids;
-    int		j;
-
-    idlist_pdu = (idlist_t *)pdubuf;
-    pdu_end = (char *)pdubuf + idlist_pdu->hdr.len;
-
-    if (pdu_end - (char *)pdubuf < sizeof(idlist_t) - sizeof(pmID))
-	return PM_ERR_IPC;
-    *sts = ntohl(idlist_pdu->sts);
-    nids = ntohl(idlist_pdu->numids);
-    if (nids <= 0 || nids != numids || nids > idlist_pdu->hdr.len)
-	return PM_ERR_IPC;
-    if (nids >= (INT_MAX - sizeof(idlist_t)) / sizeof(pmID))
-	return PM_ERR_IPC;
-    if (sizeof(idlist_t) + (sizeof(pmID) * (nids-1)) > (size_t)(pdu_end - (char *)pdubuf))
-	return PM_ERR_IPC;
-
-    for (j = 0; j < numids; j++)
-	idlist[j] = __ntohpmID(idlist_pdu->idlist[j]);
-
-    if (pmDebugOptions.pmns) {
-        fprintf(stderr, "__pmDecodeIDList\n");
-	__pmDumpIDList(stderr, numids, idlist);
-    }
-
-    return 0;
-}
-
-/*********************************************************************/
-
 /*
  * PDU for name list (PDU_PMNS_NAMES)
  */
@@ -162,7 +66,7 @@ typedef struct {
  */ 
 
 void
-__pmDumpNameList(FILE *f, int numnames, char *namelist[])
+__pmDumpNameList(FILE *f, int numnames, const char *namelist[])
 {
     int i;
 
@@ -196,7 +100,7 @@ __pmDumpNameAndStatusList(FILE *f, int numnames, char *namelist[], int statuslis
  * Send a PDU_PMNS_NAMES across socket.
  */
 int
-__pmSendNameList(int fd, int from, int numnames, char *namelist[],
+__pmSendNameList(int fd, int from, int numnames, const char *namelist[],
 		 const int statuslist[])
 {
     namelist_t		*nlistp;
@@ -235,7 +139,7 @@ __pmSendNameList(int fd, int from, int numnames, char *namelist[],
     nlistp->numnames = htonl(numnames);
 
     if (statuslist == NULL) {
-        int i, j = 0, namelen;
+        int j = 0, namelen;
         nlistp->numstatus = htonl(0);
 	for(i=0; i<numnames; i++) {
 	    nt = (name_t*)&nlistp->names[j/sizeof(__pmPDU)];
@@ -253,7 +157,7 @@ __pmSendNameList(int fd, int from, int numnames, char *namelist[],
 	}
     }
     else { /* include the status fields */
-        int i, j = 0, namelen;
+        int j = 0, namelen;
         nlistp->numstatus = htonl(numnames);
 	for(i=0; i<numnames; i++) {
 	    nst = (name_status_t*)&nlistp->names[j/sizeof(__pmPDU)];
@@ -305,8 +209,13 @@ __pmDecodeNameList(__pmPDU *pdubuf, int *numnamesp,
     if (statuslist != NULL)
 	*statuslist = NULL;
 
-    if (pdu_end - (char*)namelist_pdu < sizeof(namelist_t) - sizeof(__pmPDU))
+    if (pdu_end - (char*)namelist_pdu < sizeof(namelist_t) - sizeof(__pmPDU)) {
+	if (pmDebugOptions.pmns) {
+	    fprintf(stderr, "__pmDecodeNameList: Bad PDU: bytes %d < min %d\n",
+		(int)(pdu_end - (char*)namelist_pdu), (int)(sizeof(namelist_t) - sizeof(__pmPDU)));
+	}
 	return PM_ERR_IPC;
+    }
 
     numnames = ntohl(namelist_pdu->numnames);
     numstatus = ntohl(namelist_pdu->numstatus);
@@ -314,22 +223,54 @@ __pmDecodeNameList(__pmPDU *pdubuf, int *numnamesp,
 
     if (numnames == 0) {
 	*numnamesp = 0;
+	if (pmDebugOptions.pmns)
+	    fprintf(stderr, "__pmDecodeNameList: no names\n");
 	return 0;
     }
 
-    /* validity checks - none of these conditions should happen */
-    if (numnames < 0 || nstrbytes < 0)
-	return PM_ERR_IPC;
-    /* anti-DOS measure - limiting allowable memory allocations */
-    if (numnames > namelist_pdu->hdr.len || nstrbytes > namelist_pdu->hdr.len)
-	return PM_ERR_IPC;
-    /* numstatus must be one (and only one) of zero or numnames */
-    if (numstatus != 0 && numstatus != numnames)
-	return PM_ERR_IPC;
+    if (numnames < 0) {
+	if (pmDebugOptions.pmns) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmDecodeNameList: numnames %d < 0: %s\n",
+		numnames, pmErrStr_r(numnames, errmsg, sizeof(errmsg)));
+	}
+	/* error code in PDU, nothing more to do ... */
+	goto done;
+    }
 
-    /* need space for name ptrs and the name characters */
-    if (numnames >= (INT_MAX - nstrbytes) / (int)sizeof(char *))
+    /* validity checks - none of these conditions should happen */
+    if (nstrbytes < 0) {
+	if (pmDebugOptions.pmns) {
+	    fprintf(stderr, "__pmDecodeNameList: Bad PDU: numnames %d, and nstrbytes %d < 0\n",
+		numnames, nstrbytes);
+	}
 	return PM_ERR_IPC;
+    }
+    /* anti-DOS measure - limiting allowable memory allocations */
+    if (numnames > namelist_pdu->hdr.len || nstrbytes > namelist_pdu->hdr.len) {
+	if (pmDebugOptions.pmns) {
+	    fprintf(stderr, "__pmDecodeNameList: Bad PDU: numnames %d > len %d or nstrbytes %d > len %d\n",
+		numnames, namelist_pdu->hdr.len, nstrbytes, namelist_pdu->hdr.len);
+	}
+	return PM_ERR_IPC;
+    }
+    /* numstatus must be one (and only one) of zero or numnames */
+    if (numstatus != 0 && numstatus != numnames) {
+	if (pmDebugOptions.pmns) {
+	    fprintf(stderr, "__pmDecodeNameList: Bad PDU: numstatus %d != 0 and != numnames %d\n",
+		numstatus, numnames);
+	}
+	return PM_ERR_IPC;
+    }
+    /* need space for name ptrs and the name characters */
+    if (numnames >= (INT_MAX - nstrbytes) / (int)sizeof(char *)) {
+	if (pmDebugOptions.pmns) {
+	    fprintf(stderr, "__pmDecodeNameList: Bad PDU: numnames %d >= max %d\n",
+		numnames, (INT_MAX - nstrbytes) / (int)sizeof(char *));
+	}
+	return PM_ERR_IPC;
+    }
+
     namesize = numnames * ((int)sizeof(char*)) + nstrbytes;
     if ((names = (char**)malloc(namesize)) == NULL)
 	return -oserror();
@@ -403,7 +344,7 @@ __pmDecodeNameList(__pmPDU *pdubuf, int *numnamesp,
 
     if (pmDebugOptions.pmns) {
 	fprintf(stderr, "__pmDecodeNameList\n");
-	__pmDumpNameList(stderr, numnames, names);
+	__pmDumpNameList(stderr, numnames, (const char **)names);
 	if (status != NULL)
 	    __pmDumpStatusList(stderr, numstatus, status);
     }
@@ -412,6 +353,8 @@ __pmDecodeNameList(__pmPDU *pdubuf, int *numnamesp,
     if (statuslist != NULL)
 	*statuslist = status;
     *numnamesp = numnames;
+
+done:
     return numnames;
 
 corrupt:
@@ -421,7 +364,6 @@ corrupt:
     return PM_ERR_IPC;
 }
 
-
 /*********************************************************************/
 
 /*
