@@ -1,26 +1,24 @@
 /*
  * Copyright (c) 1995 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ * Copyright (c) 2022 Red Hat.
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <stdlib.h>
 #include "pmapi.h"
+#include "libpcp.h"
 
 static int
-comp(const void *a, const void *b)
+diffinsts(const void *a, const void *b)
 {
     pmValue	*ap = (pmValue *)a;
     pmValue	*bp = (pmValue *)b;
@@ -28,14 +26,31 @@ comp(const void *a, const void *b)
     return ap->inst - bp->inst;
 }
 
-void
-pmSortInstances(pmResult *rp)
+static void
+sortinsts(int numpmid, pmValueSet **vset)
 {
     int		i;
 
-    for (i = 0; i < rp->numpmid; i++) {
-	if (rp->vset[i]->numval > 1) {
-	    qsort(rp->vset[i]->vlist, rp->vset[i]->numval, sizeof(pmValue), comp);
-	}
+    for (i = 0; i < numpmid; i++) {
+	if (vset[i]->numval > 1)
+	    qsort(vset[i]->vlist, vset[i]->numval, sizeof(pmValue), diffinsts);
     }
 }
+
+void
+__pmSortInstances(__pmResult *rp)
+{
+    sortinsts(rp->numpmid, &rp->vset[0]);
+}
+
+void
+pmSortInstances(pmResult *rp)
+{
+    sortinsts(rp->numpmid, &rp->vset[0]);
+}
+
+void
+pmSortHighResInstances(pmHighResResult *rp)
+{
+    sortinsts(rp->numpmid, &rp->vset[0]);
+}
