@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2017 Red Hat.
+ * Copyright (c) 2013-2018 Red Hat.
  * Copyright (c) 2008-2010 Aconex.  All Rights Reserved.
  * 
  * This library is free software; you can redistribute it and/or modify it
@@ -30,6 +30,7 @@
  * Thanks to "Earnie" on the mingw-users@lists.sourceforge.net mailing
  * list for this tip.
  */
+#include <w32api.h>
 #define WINVER Windows7
 #define _WIN32_WINNT _WIN32_WINNT_WIN7
 
@@ -197,6 +198,24 @@ pmSetProgname(const char *program)
     __pmSetSignalHandler(SIGTERM, sigterm_callback);
 }
 
+static LPTSTR
+append_option(LPTSTR cmdline, int *size, const char *option)
+{
+    int sz = *size, length = strlen(option);
+
+    /* add 1space or 1null */
+    if ((cmdline = realloc(cmdline, sz + length + 1)) == NULL) {
+	pmNoMem("__pmServerStart", sz + length + 1, PM_FATAL_ERR);
+	/* NOTREACHED */
+    }
+    strcpy(&cmdline[sz], option);
+    cmdline[sz + length] = ' ';
+    sz += length + 1;
+
+    *size = sz;
+    return cmdline;
+}
+
 void
 __pmServerStart(int argc, char **argv, int flags)
 {
@@ -207,7 +226,6 @@ __pmServerStart(int argc, char **argv, int flags)
     int i, sz = 0;
     int	sts;
 
-    (void)flags;
     fflush(stdout);
     fflush(stderr);
 
@@ -229,19 +247,20 @@ __pmServerStart(int argc, char **argv, int flags)
 	    /* append argv[0] (with leading slash) at cmdline[sz] */
 	}
     }
- 
-    for (command = argv[0], i = 0; command && *command; command = argv[++i]) {
-	int length = strlen(command);
-	/* add 1space or 1null */
-	if ((cmdline = realloc(cmdline, sz + length + 1)) == NULL) {
-	    pmNoMem("__pmServerStart", sz + length + 1, PM_FATAL_ERR);
-	    /* NOTREACHED */
-	}
-	strcpy(&cmdline[sz], command);
-	cmdline[sz + length] = ' ';
-	sz += length + 1;
+    for (command = argv[0], i = 0;
+	 i < argc && command && *command;
+	 command = argv[++i]) {
+	cmdline = append_option(cmdline, &sz, command);
+    }
+    if (flags & 0x1) {
+	/*
+	 * force append -f option - no exec() so for traditional PCP
+	 * daemons, this is added to argv to prevent infinite loop.
+	 */
+	cmdline = append_option(cmdline, &sz, "-f");
     }
     cmdline[sz - 1] = '\0';
+
     if (pmDebugOptions.exec)
 	fprintf(stderr, "__pmServerStart: cmdline=%s\n", cmdline);
 
@@ -616,6 +635,22 @@ sleep(unsigned int seconds)
     return 0;
 }
 
+struct tm *
+gmtime_r(const time_t *timep, struct tm *result)
+{
+    if (gmtime_s(result, timep) == 0)
+	return result;
+    return NULL;
+}
+
+char *
+ctime_r(const time_t *timep, char *buf)
+{
+    if (ctime_s(buf, 26, timep) == 0)
+	return buf;
+    return NULL;
+}
+
 void
 setlinebuf(FILE *stream)
 {
