@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2019 Red Hat.
+ * Copyright (c) 2017-2019,2022 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -124,19 +124,43 @@ dictType sdsDictCallBacks = {
 void
 redisMapsInit(void)
 {
-    static const char * const mapnames[] = {
-	"inst.name", "metric.name", "label.name", "context.name"
-    };
-    static int		setup;
+    if (instmap == NULL)
+	instmap = dictCreate(&sdsDictCallBacks,
+				(void *)sdsnew("inst.name"));
+    if (namesmap == NULL)
+	namesmap = dictCreate(&sdsDictCallBacks,
+				(void *)sdsnew("metric.name"));
+    if (labelsmap == NULL)
+	labelsmap = dictCreate(&sdsDictCallBacks,
+				(void *)sdsnew("label.name"));
+    if (contextmap == NULL)
+	contextmap = dictCreate(&sdsDictCallBacks,
+				(void *)sdsnew("context.name"));
+}
 
-    if (setup)
-	return;
-    setup = 1;
-
-    instmap = dictCreate(&sdsDictCallBacks, (void *)sdsnew(mapnames[0]));
-    namesmap = dictCreate(&sdsDictCallBacks, (void *)sdsnew(mapnames[1]));
-    labelsmap = dictCreate(&sdsDictCallBacks, (void *)sdsnew(mapnames[2]));
-    contextmap = dictCreate(&sdsDictCallBacks, (void *)sdsnew(mapnames[3]));
+void
+redisMapsClose(void)
+{
+    if (instmap) {
+	sdsfree(redisMapName(instmap));
+	dictRelease(instmap);
+	instmap = NULL;
+    }
+    if (namesmap) {
+	sdsfree(redisMapName(namesmap));
+	dictRelease(namesmap);
+	namesmap = NULL;
+    }
+    if (labelsmap) {
+	sdsfree(redisMapName(labelsmap));
+	dictRelease(labelsmap);
+	labelsmap = NULL;
+    }
+    if (contextmap) {
+	sdsfree(redisMapName(contextmap));
+	dictRelease(contextmap);
+	contextmap = NULL;
+    }
 }
 
 sds
