@@ -23,10 +23,8 @@ extern dictType sdsKeyDictCallBacks;	/* sds string -> (void *) value */
 extern dictType sdsDictCallBacks;	/* sds key -> sds string value */
 extern dictType sdsOwnDictCallBacks;	/* owned sds key -> sds string value */
 
-extern int tsub(struct timeval *, struct timeval *);
-extern int tadd(struct timeval *, struct timeval *);
-extern const char *timeval_str(struct timeval *, char *, int);
-extern const char *timeval_stream_str(struct timeval *, char *, int);
+extern const char *timespec_str(struct timespec *, char *, int);
+extern const char *timespec_stream_str(struct timespec *, char *, int);
 extern const char *timestamp_str(__pmTimestamp *, char *, int);
 extern const char *timestamp_stream_str(__pmTimestamp *, char *, int);
 
