@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019-2021 Red Hat.
+ * Copyright (c) 2019-2022 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -44,10 +44,8 @@ enum matches { MATCH_EXACT, MATCH_GLOB, MATCH_REGEX };
 enum profile { PROFILE_ADD, PROFILE_DEL };
 
 enum webgroup_metric {
-    CONTEXT_MAP_SIZE,
-    NAMES_MAP_SIZE,
-    LABELS_MAP_SIZE,
-    INST_MAP_SIZE,
+    WEBGROUP_GC_COUNT,
+    WEBGROUP_GC_DROPS,
     NUM_WEBGROUP_METRIC
 };
 
@@ -64,7 +62,6 @@ typedef struct webgroups {
     uv_mutex_t		mutex;
 
     unsigned int	active;
-    int			timerid;
 } webgroups;
 
 static struct webgroups *
@@ -76,7 +73,6 @@ webgroups_lookup(pmWebGroupModule *module)
 	module->privdata = calloc(1, sizeof(struct webgroups));
 	groups = (struct webgroups *)module->privdata;
 	uv_mutex_init(&groups->mutex);
-	groups->timerid = -1;
     }
     return groups;
 }
@@ -286,12 +282,23 @@ webgroup_new_context(pmWebGroupSettings *sp, dict *params,
     return cp;
 }
 
+static void
+webgroup_timers_stop(struct webgroups *groups)
+{
+    if (groups->active) {
+	uv_timer_stop(&groups->timer);
+	uv_close((uv_handle_t *)&groups->timer, NULL);
+	groups->active = 0;
+    }
+}
+
 static void
 webgroup_garbage_collect(struct webgroups *groups)
 {
     dictIterator        *iterator;
     dictEntry           *entry;
     context_t		*cp;
+    unsigned int	count = 0, drops = 0;
 
     if (pmDebugOptions.http || pmDebugOptions.libweb)
 	fprintf(stderr, "%s: started\n", "webgroup_garbage_collect");
@@ -308,33 +315,27 @@ webgroup_garbage_collect(struct webgroups *groups)
 		uv_mutex_unlock(&groups->mutex);
 		webgroup_drop_context(cp, groups);
 		uv_mutex_lock(&groups->mutex);
+		drops++;
 	    }
+	    count++;
 	}
 	dictReleaseIterator(iterator);
+
+	/* if dropping the last remaining context, do cleanup */
+	if (groups->active && drops == count) {
+	    if (pmDebugOptions.http || pmDebugOptions.libweb)
+		fprintf(stderr, "%s: freezing\n", "webgroup_garbage_collect");
+	    webgroup_timers_stop(groups);
+	}
 	uv_mutex_unlock(&groups->mutex);
     }
 
+    mmv_set(groups->map, groups->metrics[WEBGROUP_GC_DROPS], &drops);
+    mmv_set(groups->map, groups->metrics[WEBGROUP_GC_COUNT], &count);
+
     if (pmDebugOptions.http || pmDebugOptions.libweb)
-	fprintf(stderr, "%s: finished\n", "webgroup_garbage_collect");
-}
-
-static void
-refresh_maps_metrics(void *data)
-{
-    struct webgroups	*groups = (struct webgroups *)data;
-
-    if (groups->metrics) {
-	unsigned int	value;
-
-	value = dictSize(contextmap);
-	mmv_set(groups->map, groups->metrics[CONTEXT_MAP_SIZE], &value);
-	value = dictSize(namesmap);
-	mmv_set(groups->map, groups->metrics[NAMES_MAP_SIZE], &value);
-	value = dictSize(labelsmap);
-	mmv_set(groups->map, groups->metrics[LABELS_MAP_SIZE], &value);
-	value = dictSize(instmap);
-	mmv_set(groups->map, groups->metrics[INST_MAP_SIZE], &value);
-    }
+	fprintf(stderr, "%s: finished [%u drops from %u entries]\n",
+			"webgroup_garbage_collect", drops, count);
 }
 
 static void
@@ -405,8 +406,6 @@ webgroup_lookup_context(pmWebGroupSettings *sp, sds *id, dict *params,
 	groups->timer.data = (void *)groups;
 	uv_timer_start(&groups->timer, webgroup_worker,
 			default_worker, default_worker);
-	/* timer for map stats refresh */
-	groups->timerid = pmWebTimerRegister(refresh_maps_metrics, groups);
     }
 
     if (*id == NULL) {
@@ -487,6 +486,7 @@ pmWebGroupDestroy(pmWebGroupSettings *settings, sds id, void *arg)
 	if (pmDebugOptions.libweb)
 	    fprintf(stderr, "%s: destroy context %p gp=%p\n", "pmWebGroupDestroy", cp, gp);
 
+	webgroup_deref_context(cp);
 	webgroup_drop_context(cp, gp);
     }
     sdsfree(msg);
@@ -673,15 +673,15 @@ webgroup_fetch(pmWebGroupSettings *settings, context_t *cp,
     pmWebResult		webresult;
     pmWebValueSet	webvalueset;
     pmWebValue		webvalue;
-    pmResult		*result;
+    pmHighResResult	*result;
     char		err[PM_MAXERRMSGLEN];
     sds			v = sdsempty(), series = NULL;
     sds			id = cp->origin;
     int			i, j, k, sts, inst, type, status = 0;
 
-    if ((sts = pmFetch(numpmid, pmidlist, &result)) >= 0) {
+    if ((sts = pmFetchHighRes(numpmid, pmidlist, &result)) >= 0) {
 	webresult.seconds = result->timestamp.tv_sec;
-	webresult.nanoseconds = result->timestamp.tv_usec * 1000;
+	webresult.nanoseconds = result->timestamp.tv_nsec;
 
 	settings->callbacks.on_fetch(id, &webresult, arg);
 
@@ -751,7 +751,7 @@ webgroup_fetch(pmWebGroupSettings *settings, context_t *cp,
 		}
 	    }
 	}
-	pmFreeResult(result);
+	pmFreeHighResResult(result);
     } else if (sts == PM_ERR_IPC) {
 	cp->setup = 0;
     }
@@ -1720,7 +1720,7 @@ webgroup_scrape(pmWebGroupSettings *settings, context_t *cp,
     struct value	*value;
     pmWebLabelSet	labels;
     pmWebScrape		scrape;
-    pmResult		*result;
+    pmHighResResult	*result;
     sds			sems, types, units;
     sds			v = sdsempty(), series = NULL;
     int			i, j, k, sts, type;
@@ -1732,9 +1732,9 @@ webgroup_scrape(pmWebGroupSettings *settings, context_t *cp,
     labels.buffer = sdsnewlen(NULL, PM_MAXLABELJSONLEN);
     sdsclear(labels.buffer);
 
-    if ((sts = pmFetch(numpmid, pmidlist, &result)) >= 0) {
+    if ((sts = pmFetchHighRes(numpmid, pmidlist, &result)) >= 0) {
 	scrape.seconds = result->timestamp.tv_sec;
-	scrape.nanoseconds = result->timestamp.tv_usec * 1000;
+	scrape.nanoseconds = result->timestamp.tv_nsec;
 
 	/* extract all values from the result for later stages */
 	for (i = 0; i < numpmid; i++)
@@ -1824,7 +1824,7 @@ webgroup_scrape(pmWebGroupSettings *settings, context_t *cp,
 		}
 	    }
 	}
-	pmFreeResult(result);
+	pmFreeHighResResult(result);
     } else {
 	char		err[PM_MAXERRMSGLEN];
 
@@ -2075,7 +2075,7 @@ webgroup_store(struct context *context, struct metric *metric,
     struct indom	*indom;
     pmAtomValue		atom = {0};
     pmValueSet		*valueset = NULL;
-    pmResult		*result = NULL;
+    pmHighResResult	*result = NULL;
     size_t		bytes;
     long		cursor = 0;
     int			i, id, sts, count;
@@ -2097,7 +2097,7 @@ webgroup_store(struct context *context, struct metric *metric,
 	count = 1;
 
     bytes = sizeof(pmValueSet) + sizeof(pmValue) * (count - 1);
-    if ((result = (pmResult *)calloc(1, sizeof(pmResult))) == NULL ||
+    if ((result = (pmHighResResult *)calloc(1, sizeof(*result))) == NULL ||
 	(valueset = (pmValueSet *)calloc(1, bytes)) == NULL) {
 	if (atom.cp && metric->desc.type == PM_TYPE_STRING)
 	    free(atom.cp);
@@ -2148,9 +2148,9 @@ webgroup_store(struct context *context, struct metric *metric,
 	valueset->valfmt = sts;
 	valueset->numval = count;
 	valueset->pmid = metric->desc.pmid;
-	sts = pmStore(result);
+	sts = pmStoreHighRes(result);
     }
-    pmFreeResult(result);
+    pmFreeHighResResult(result);
     return sts;
 }
 
@@ -2233,7 +2233,7 @@ int
 pmWebGroupSetup(pmWebGroupModule *module)
 {
     struct webgroups	*groups = webgroups_lookup(module);
-    struct timeval	tv;
+    struct timespec	ts;
     unsigned int	pid;
 
     if (groups == NULL)
@@ -2268,9 +2268,9 @@ pmWebGroupSetup(pmWebGroupModule *module)
     AUTH_PASSWORD = sdsnew("auth.password");
 
     /* setup the random number generator for context IDs */
-    gettimeofday(&tv, NULL);
+    pmtimespecNow(&ts);
     pid = (unsigned int)getpid();
-    srandom(pid ^ (unsigned int)tv.tv_sec ^ (unsigned int)tv.tv_usec);
+    srandom(pid ^ (unsigned int)ts.tv_sec ^ (unsigned int)ts.tv_nsec);
 
     /* setup a dictionary mapping context number to data */
     groups->contexts = dictCreate(&intKeyDictCallBacks, NULL);
@@ -2339,36 +2339,21 @@ pmWebGroupSetupMetrics(pmWebGroupModule *module)
     if (groups == NULL || groups->registry == NULL)
 	return; /* no metric registry has been set up */
 
-    /*
-     * Reverse mapping dict metrics
-     */
-    mmv_stats_add_metric(groups->registry, "contextmap.size", 1,
+    mmv_stats_add_metric(groups->registry, "gc.context.scans", 1,
 	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
-	"context map dictionary size",
-	"Number of entries in the context map dictionary");
+	"contexts scanned in last garbage collection",
+	"Contexts scanned during most recent webgroup garbage collection");
 
-    mmv_stats_add_metric(groups->registry, "namesmap.size", 2,
+    mmv_stats_add_metric(groups->registry, "gc.context.drops", 2,
 	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
-	"metric names map dictionary size",
-	"Number of entries in the metric names map dictionary");
-
-    mmv_stats_add_metric(groups->registry, "labelsmap.size", 3,
-	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
-	"labels map dictionary size",
-	"Number of entries in the labels map dictionary");
-
-    mmv_stats_add_metric(groups->registry, "instmap.size", 4,
-	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
-	"instance name map dictionary size",
-	"Number of entries in the instance name map dictionary");
+	"contexts dropped in last garbage collection",
+	"Contexts dropped during most recent webgroup garbage collection");
 
     groups->map = map = mmv_stats_start(groups->registry);
 
     ap = groups->metrics;
-    ap[CONTEXT_MAP_SIZE] = mmv_lookup_value_desc(map, "contextmap.size", NULL);
-    ap[NAMES_MAP_SIZE] = mmv_lookup_value_desc(map, "namesmap.size", NULL);
-    ap[LABELS_MAP_SIZE] = mmv_lookup_value_desc(map, "labelsmap.size", NULL);
-    ap[INST_MAP_SIZE] = mmv_lookup_value_desc(map, "instmap.size", NULL);
+    ap[WEBGROUP_GC_DROPS] = mmv_lookup_value_desc(map, "gc.context.scans", NULL);
+    ap[WEBGROUP_GC_COUNT] = mmv_lookup_value_desc(map, "gc.context.drops", NULL);
 }
 
 
@@ -2394,19 +2379,15 @@ pmWebGroupClose(pmWebGroupModule *module)
 
     if (groups) {
 	/* walk the contexts, stop timers and free resources */
-	if (groups->active) {
-	    groups->active = 0;
-	    uv_timer_stop(&groups->timer);
-	    pmWebTimerRelease(groups->timerid);
-	    groups->timerid = -1;
-	}
 	iterator = dictGetIterator(groups->contexts);
 	while ((entry = dictNext(iterator)) != NULL)
 	    webgroup_drop_context((context_t *)dictGetVal(entry), NULL);
 	dictReleaseIterator(iterator);
 	dictRelease(groups->contexts);
+	webgroup_timers_stop(groups);
 	memset(groups, 0, sizeof(struct webgroups));
 	free(groups);
+	module->privdata = NULL;
     }
 
     sdsfree(PARAM_HOSTNAME);
