@@ -23,39 +23,40 @@
 #include "photoproc.h"
 #include "parseable.h"
 
-void 	print_CPU();
-void 	print_cpu();
-void 	print_CPL();
-void 	print_GPU();
-void 	print_MEM();
-void 	print_SWP();
-void 	print_PAG();
-void 	print_PSI();
-void 	print_LVM();
-void 	print_MDD();
-void 	print_DSK();
-void 	print_NFM();
-void 	print_NFC();
-void 	print_NFS();
-void 	print_NET();
-void 	print_IFB();
-void 	print_NUM();
-void 	print_NUC();
-void 	print_LLC();
+void   print_CPU(char *, struct sstat *, struct tstat *, int);
+void   print_cpu(char *, struct sstat *, struct tstat *, int);
+void   print_CPL(char *, struct sstat *, struct tstat *, int);
+void   print_GPU(char *, struct sstat *, struct tstat *, int);
+void   print_MEM(char *, struct sstat *, struct tstat *, int);
+void   print_SWP(char *, struct sstat *, struct tstat *, int);
+void   print_PAG(char *, struct sstat *, struct tstat *, int);
+void   print_PSI(char *, struct sstat *, struct tstat *, int);
+void   print_LVM(char *, struct sstat *, struct tstat *, int);
+void   print_MDD(char *, struct sstat *, struct tstat *, int);
+void   print_DSK(char *, struct sstat *, struct tstat *, int);
+void   print_NFM(char *, struct sstat *, struct tstat *, int);
+void   print_NFC(char *, struct sstat *, struct tstat *, int);
+void   print_NFS(char *, struct sstat *, struct tstat *, int);
+void   print_NET(char *, struct sstat *, struct tstat *, int);
+void   print_IFB(char *, struct sstat *, struct tstat *, int);
+void   print_NUM(char *, struct sstat *, struct tstat *, int);
+void   print_NUC(char *, struct sstat *, struct tstat *, int);
+void   print_LLC(char *, struct sstat *, struct tstat *, int);
 
-void 	print_PRG();
-void 	print_PRC();
-void 	print_PRM();
-void 	print_PRD();
-void 	print_PRN();
-void 	print_PRE();
+void   print_PRG(char *, struct sstat *, struct tstat *, int);
+void   print_PRC(char *, struct sstat *, struct tstat *, int);
+void   print_PRM(char *, struct sstat *, struct tstat *, int);
+void   print_PRD(char *, struct sstat *, struct tstat *, int);
+void   print_PRN(char *, struct sstat *, struct tstat *, int);
+void   print_PRE(char *, struct sstat *, struct tstat *, int);
 
+static void calc_freqscale(count_t, count_t, count_t, count_t *, int *);
 static char *spaceformat(char *, char *);
 static int  cgroupv2max(int, int);
 
 /*
 ** table with possible labels and the corresponding
-** print-function for parseable output
+** print-function for parsable output
 */
 struct labeldef {
 	char	*label;
@@ -168,7 +169,7 @@ parsedef(char *pd)
 }
 
 /*
-** produce parseable output for an interval
+** produce parsable output for an interval
 */
 char
 parseout(double timed, double delta,
@@ -223,7 +224,7 @@ parseout(double timed, double delta,
 /*
 ** print functions for system-level statistics
 */
-void
+static void
 calc_freqscale(count_t maxfreq, count_t cnt, count_t ticks, 
                count_t *freq, int *freqperc)
 {
@@ -373,7 +374,8 @@ void
 print_MEM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
 	printf(	"%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
-   		"%lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
+   		"%lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
+		"%lld %lld %lld\n",
 			hp,
 			pagesize,
 			ss->mem.physmem,
@@ -387,15 +389,20 @@ print_MEM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
         		ss->mem.shmem,
         		ss->mem.shmrss,
         		ss->mem.shmswp,
-        		ss->mem.hugepagesz,
-        		ss->mem.tothugepage,
-        		ss->mem.freehugepage,
+        		ss->mem.shugepagesz,
+        		ss->mem.stothugepage,
+        		ss->mem.sfreehugepage,
         		ss->mem.zfsarcsize != -1 ? ss->mem.zfsarcsize : 0,
         		ss->mem.ksmsharing != -1 ? ss->mem.ksmsharing : 0,
         		ss->mem.ksmshared  != -1 ? ss->mem.ksmshared  : 0,
 			ss->mem.tcpsock,
 			ss->mem.udpsock,
-   			ss->mem.pagetables);
+   			ss->mem.pagetables,
+        		ss->mem.lhugepagesz,
+        		ss->mem.ltothugepage,
+        		ss->mem.lfreehugepage,
+			ss->mem.availablemem,
+			ss->mem.anonhugepage);
 }
 
 void
@@ -410,14 +417,15 @@ print_SWP(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->mem.committed,
 			ss->mem.commitlim,
         		ss->mem.swapcached,
-        		ss->mem.zswstored != -1 ? ss->mem.zswstored : 0,
-        		ss->mem.zswtotpool != -1 ? ss->mem.zswtotpool : 0);
+        		ss->mem.zswapped,
+        		ss->mem.zswap);
 }
 
 void
 print_PAG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
-	printf("%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
+	printf("%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld "
+	       "%lld %lld\n",
 			hp,
 			pagesize,
 			ss->mem.pgscans,
@@ -430,7 +438,9 @@ print_PAG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->mem.pgmigrate,
 			ss->mem.numamigrate,
 			ss->mem.pgins,
-			ss->mem.pgouts);
+			ss->mem.pgouts,
+			ss->mem.zswins,
+			ss->mem.zswouts);
 }
 
 void
@@ -584,7 +594,7 @@ print_NET(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
 	register int 	i;
 
-	printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
+	printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
 			hp,
 			"upper",
         		ss->net.tcp.InSegs,
@@ -610,7 +620,8 @@ print_NET(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->net.tcp.CurrEstab,
 			ss->net.tcp.RetransSegs,
 			ss->net.tcp.InErrs,
-			ss->net.tcp.OutRsts);
+			ss->net.tcp.OutRsts,
+			ss->net.tcp.InCsumErrors);
 
 	for (i=0; ss->intf.intf[i].name[0]; i++)
 	{
@@ -653,7 +664,7 @@ print_NUM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 
 	for (i=0; i < ss->memnuma.nrnuma; i++)
 	{
-		printf(	"%s %d %u %.0f %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
+		printf(	"%s %d %u %.0f %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
 			hp, ss->memnuma.numa[i].numanr,
 			pagesize,
 			ss->memnuma.numa[i].frag * 100.0,
@@ -666,7 +677,8 @@ print_NUM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->memnuma.numa[i].slabmem,
 			ss->memnuma.numa[i].slabreclaim,
 			ss->memnuma.numa[i].shmem,
-			ss->memnuma.numa[i].tothp);
+			ss->memnuma.numa[i].tothp,
+			ss->memnuma.numa[i].freehp);
 	}
 }
 
@@ -699,7 +711,7 @@ print_LLC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 
 	for (i=0; i < ss->llc.nrllcs; i++)
 	{
-		printf(	"%s LLC%03d %3.1f%% %lld %lld\n",
+	        printf( "%s LLC%02d %3.1f%% %lld %lld\n",
 			hp,
 			ss->llc.perllc[i].id,
 			ss->llc.perllc[i].occupancy * 100,
@@ -726,7 +738,7 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			exitcode = (ps->gen.excode >>   8) & 0xff;
 
 		printf("%s %d %s %c %d %d %d %d %d %ld %s %d %d %d %d "
- 		       "%d %d %d %d %d %d %ld %c %d %d %s %c %s\n",
+ 		       "%d %d %d %d %d %d %ld %c %d %d %s %c %s %ld %d\n",
 			hp,
 			ps->gen.pid,
 			spaceformat(ps->gen.name, namout),
@@ -736,7 +748,7 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ps->gen.tgid,
 			ps->gen.nthr,
 			exitcode,
-			ps->gen.btime,
+			(long)ps->gen.btime,
 			spaceformat(ps->gen.cmdline, cmdout),
 			ps->gen.ppid,
 			ps->gen.nthrrun,
@@ -748,13 +760,16 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ps->gen.sgid,
 			ps->gen.fsuid,
 			ps->gen.fsgid,
-			ps->gen.elaps,
+			(long)ps->gen.elaps,
 			ps->gen.isproc ? 'y':'n',
 			ps->gen.vpid,
 			ps->gen.ctid,
-			ps->gen.container[0] ? ps->gen.container:"-",
+			ps->gen.utsname[0] ? ps->gen.utsname:"-",
         		ps->gen.excode & ~(INT_MAX) ? 'N' : '-',
-			spaceformat(ps->gen.cgpath, pathout));
+			spaceformat(ps->gen.cgpath, pathout),
+			ps->gen.state == 'E' ?
+			    ps->gen.btime + ps->gen.elaps/hertz : 0,
+			ps->gen.nthridle);
 	}
 }
 
@@ -767,7 +782,7 @@ print_PRC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	for (i=0; i < nact; i++, ps++)
 	{
 		printf("%s %d %s %c %u %lld %lld %d %d %d %d %d %d %d %c "
-		       "%llu %s %llu %d %d\n",
+		       "%llu %s %llu %d %d %llu %llu\n",
 			hp,
 			ps->gen.pid,
 			spaceformat(ps->gen.name, namout),
@@ -787,7 +802,9 @@ print_PRC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			spaceformat(ps->cpu.wchan, wchanout),
 			ps->cpu.blkdelay,
 			cgroupv2max(ps->gen.isproc, ps->cpu.cgcpumax),
-			cgroupv2max(ps->gen.isproc, ps->cpu.cgcpumaxr));
+			cgroupv2max(ps->gen.isproc, ps->cpu.cgcpumaxr),
+			ps->cpu.nvcsw,
+			ps->cpu.nivcsw);
 	}
 }
 
