@@ -44,7 +44,6 @@ static struct pselection procsel = {"", {USERSTUB, }, {0,},
 static struct sselection syssel;
 
 static void	showhelp(int);
-static int	paused;     	/* boolean: currently in pause-mode     */
 static int	fixedhead;	/* boolean: fixate header-lines         */
 static int	sysnosort;	/* boolean: suppress sort of resources  */
 static int	threadsort;	/* boolean: sort threads per process    */
@@ -71,6 +70,8 @@ static short	coloralmost = COLOR_CYAN;
 static short	colorcrit   = COLOR_RED;
 static short	colorthread = COLOR_YELLOW;
 
+int		paused;    	/* boolean: currently in pause-mode     */
+
 static int	cumusers(struct tstat **, struct tstat *, int);
 static int	cumprogs(struct tstat **, struct tstat *, int);
 static int	cumconts(struct tstat **, struct tstat *, int);
@@ -80,7 +81,11 @@ static int	procsuppress(struct tstat *, struct pselection *);
 static void	limitedlines(void);
 static long	getnumval(char *, long, int);
 static void	generic_init(void);
+static char	text_samp(double, double, struct devtstat *,
+			struct sstat *, int, unsigned int, int);
 
+static char	text_samp(double, double, struct devtstat *,
+			struct sstat *, int, unsigned int, int);
 
 static int	(*procsort[])(const void *, const void *) = {
 			[MSORTCPU&0x1f]=compcpu, 
@@ -98,16 +103,159 @@ extern proc_printpair ownprocs[];
 int	startoffset;
 
 /*
-** print the deviation-counters on process- and system-level
+** main function to handle sample generically
+** to show flat text, full screen text or bar graphs
 */
 char
 generic_samp(double sampletime, double nsecs,
            struct devtstat *devtstat, struct sstat *sstat,
            int nexit, unsigned int noverflow, int flag)
 {
-	static int	callnr = 0;
-	char		*p;
+	static char	firstcall = 1;
+	char		retval, sorted = 0;
 
+	if (firstcall)
+	{
+		generic_init();
+		firstcall = 0;
+	}
+
+	while (1)
+	{
+		if (displaymode == 'T')		// text mode?
+		{
+			// show sample and wait for input or timer expiration
+			//
+			switch (retval = text_samp(sampletime, nsecs,
+				devtstat, sstat, nexit, noverflow, flag))
+			{
+			   case MBARGRAPH:
+				displaymode = 'D';
+				break;
+
+  			   default:
+				return retval;
+			}
+
+			sorted = 1;	// resources have been sorted by text mode
+		}
+
+		if (displaymode == 'D')		// draw mode?
+		{
+			// show sample and wait for input or timer expiration
+			//
+			switch (retval = draw_samp(sampletime, nsecs, sstat, flag, sorted))
+			{
+			   case MBARGRAPH:	// just switch to text mode
+				displaymode = 'T';
+				break;
+
+			   case MPROCGEN:	// switch to text mode: generic
+				showtype  = MPROCGEN;
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTCPU;
+
+				displaymode = 'T';
+				break;
+
+ 			   case MPROCMEM:	// switch to text mode: memory
+				showtype  = MPROCMEM;
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTMEM;
+
+				displaymode = 'T';
+				break;
+
+			   case MPROCDSK:	// switch to text mode: disk
+				if (supportflags & IOSTAT)
+				{
+					showtype  = MPROCDSK;
+
+					if (showorder != MSORTAUTO)
+						showorder = MSORTDSK;
+				}
+				else
+				{
+					showtype  = MPROCGEN;
+					showorder = MSORTCPU;
+				}
+
+				displaymode = 'T';
+				break;
+
+			   case MPROCNET:	// switch to text mode: network
+				if (supportflags & NETATOP || supportflags & NETATOPBPF)
+				{
+					showtype  = MPROCNET;
+
+					if (showorder != MSORTAUTO)
+						showorder = MSORTNET;
+				}
+				else
+				{
+					showtype  = MPROCGEN;
+					showorder = MSORTCPU;
+				}
+
+				displaymode = 'T';
+				break;
+
+			   case MPROCGPU:	// switch to text mode: GPU
+				if (supportflags & GPUSTAT)
+				{
+					showtype  = MPROCGPU;
+
+					if (showorder != MSORTAUTO)
+						showorder = MSORTGPU;
+				}
+				else
+				{
+					showtype  = MPROCGEN;
+					showorder = MSORTCPU;
+				}
+
+				displaymode = 'T';
+				break;
+
+			   case MPROCSCH:	// switch to text mode: scheduling
+				showtype  = MPROCSCH;
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTCPU;
+
+				displaymode = 'T';
+				break;
+
+			   case MPROCVAR:	// switch to text mode: various
+				showtype  = MPROCVAR;
+
+				displaymode = 'T';
+				break;
+
+			   case MPROCARG:	// switch to text mode: arguments
+				showtype  = MPROCARG;
+
+				displaymode = 'T';
+				break;
+
+  			   default:
+				return retval;
+			}
+		}
+	}
+}
+
+/*
+** print the deviation-counters on process- and system-level
+** in text mode
+*/
+static char
+text_samp(double sampletime, double nsecs,
+           struct devtstat *devtstat, struct sstat *sstat, 
+           int nexit, unsigned int noverflow, int flag)
+{
 	register int	i, curline, statline, nproc;
 	int		firstproc = 0, plistsz, alistsz, killpid, killsig;
 	int		lastchar;
@@ -146,7 +294,7 @@ generic_samp(double sampletime, double nsecs,
 	int		nucum      = 0;
 	char		ulastorder = 0;
 
-	struct tstat	*tccumlist = 0;		// per container accumulation
+	struct tstat	*tccumlist = 0;		// per container/pod accumulation
 	struct tstat	**ccumlist = 0;
 	int		nccum      = 0;
 	char		clastorder = 0;
@@ -167,7 +315,7 @@ generic_samp(double sampletime, double nsecs,
 	/*
 	** sellist contains the pointers to the structs in tstat
 	** that are currently selected on basis of a particular
-	** username (regexp), program name (regexp), container name
+	** username (regexp), program name (regexp), container/pod name
 	** or suppressed exited procs
 	**
 	** this list will be allocated 'lazy'
@@ -178,12 +326,6 @@ generic_samp(double sampletime, double nsecs,
 
 	char		threadallowed = 0;
 
-
-	if (callnr == 0)	/* first call? */
-		generic_init();
-
-	callnr++;
-
 	startoffset = 0;
 
 	/*
@@ -301,7 +443,7 @@ generic_samp(double sampletime, double nsecs,
 			suppressexit 		      ? MSUPEXITS  : '-',
 			procsel.userid[0] != USERSTUB ? MSELUSER   : '-',
 			procsel.prognamesz	      ? MSELPROC   : '-',
-			procsel.container[0]	      ? MSELCONT   : '-',
+			procsel.utsname[0]	      ? MSELCONT   : '-',
 			procsel.pid[0] != 0	      ? MSELPID    : '-',
 			procsel.argnamesz	      ? MSELARG    : '-',
 			procsel.states[0]	      ? MSELSTATE  : '-',
@@ -403,7 +545,7 @@ generic_samp(double sampletime, double nsecs,
 			{
 				clrtoeol();
 				if (usecolors)
-					attron(COLOR_PAIR(COLORINFO));
+					attron(COLOR_PAIR(FGCOLORINFO));
 			}
 
 			printg(statmsg);
@@ -411,7 +553,7 @@ generic_samp(double sampletime, double nsecs,
 			if (screen)
 			{
 				if (usecolors)
-					attroff(COLOR_PAIR(COLORINFO));
+					attroff(COLOR_PAIR(FGCOLORINFO));
 			}
 
 			statmsg = NULL;
@@ -429,11 +571,7 @@ generic_samp(double sampletime, double nsecs,
 				}
 				else
 				{
- 					uid_t ruid, euid, suid;
-
-					getresuid(&ruid, &euid, &suid);
-
-					if (suid == 0)
+					if (rootprivs())
 					{
 						viewmsg   = "Unrestricted view (privileged)";
 					}
@@ -446,7 +584,7 @@ generic_samp(double sampletime, double nsecs,
 				if (screen)
 				{
 					if (usecolors)
-						attron(COLOR_PAIR(COLORINFO));
+						attron(COLOR_PAIR(FGCOLORINFO));
 
 					attron(A_BLINK);
 				}
@@ -456,7 +594,7 @@ generic_samp(double sampletime, double nsecs,
 				if (screen)
 				{
 					if (usecolors)
-						attroff(COLOR_PAIR(COLORINFO));
+						attroff(COLOR_PAIR(FGCOLORINFO));
 					attroff(A_BLINK);
 
 					printg("%*s", (int)(
@@ -473,7 +611,7 @@ generic_samp(double sampletime, double nsecs,
 				if (screen)
 				{
 					if (usecolors)
-						attron(COLOR_PAIR(COLORALMOST));
+						attron(COLOR_PAIR(FGCOLORALMOST));
 
 					attron(A_BLINK);
 				}
@@ -483,7 +621,7 @@ generic_samp(double sampletime, double nsecs,
 				if (screen)
 				{
 					if (usecolors)
-						attroff(COLOR_PAIR(COLORALMOST));
+						attroff(COLOR_PAIR(FGCOLORALMOST));
 					attroff(A_BLINK);
 				}
 
@@ -646,7 +784,7 @@ generic_samp(double sampletime, double nsecs,
 
 			if ( procsel.userid[0] == USERSTUB &&
 			    !procsel.prognamesz            &&
-			    !procsel.container[0]          &&
+			    !procsel.utsname[0]            &&
 			    !procsel.states[0]             &&
 			    !procsel.argnamesz             &&
 			    !procsel.pid[0]                &&
@@ -895,6 +1033,24 @@ generic_samp(double sampletime, double nsecs,
 				refresh();
 				cleanstop(0);
 
+			   /*
+			   ** switch to bar graph mode
+			   */
+			   case MBARGRAPH:
+				erase();
+				refresh();
+
+				if (tpcumlist) free(tpcumlist);
+				if (pcumlist)  free(pcumlist);
+				if (tucumlist) free(tucumlist);
+				if (ucumlist)  free(ucumlist);
+				if (tccumlist) free(tccumlist);
+				if (ccumlist)  free(ccumlist);
+				if (tsklist)   free(tsklist);
+				if (sellist)   free(sellist);
+
+				return lastchar;
+
 			   /*
 			   ** manual trigger for next sample
 			   */
@@ -966,7 +1122,7 @@ generic_samp(double sampletime, double nsecs,
                                 move(statline, 0);
                                 clrtoeol();
                                 printw("Enter new time "
-				       "(format [YYYYMMDD]hh:mm:ss): ");
+				       "(format [YYYYMMDD]hhmm[ss]): ");
 
                                 branchtime[0] = '\0';
                                 scanw("%31s\n", branchtime);
@@ -990,7 +1146,7 @@ generic_samp(double sampletime, double nsecs,
 				curtime = timeval;
 				pmtimevalDec(&curtime, &interval);
 				if (time_less_than(&curtime, &start))
-				    curtime = start;
+					curtime = start;
 
 				if (tpcumlist) free(tpcumlist);
 				if (pcumlist)  free(pcumlist);
@@ -1046,9 +1202,9 @@ generic_samp(double sampletime, double nsecs,
 			   ** sort in network-activity order
 			   */
 			   case MSORTNET:
-				if ( !(supportflags & NETATOP) )
+				if ( !(supportflags & NETATOP || supportflags & NETATOPBPF))
 				{
-					statmsg = "BCC PMDA not active or "
+					statmsg = "BPF or BCC PMDA not active or "
 					          "'netproc' module not enabled; "
 					          "request ignored!";
 					break;
@@ -1118,9 +1274,9 @@ generic_samp(double sampletime, double nsecs,
 			   ** network-specific figures per process
 			   */
 			   case MPROCNET:
-				if ( !(supportflags & NETATOP) )
+				if ( !(supportflags & NETATOP || supportflags & NETATOPBPF) )
 				{
-					statmsg = "BCC PMDA not active or "
+					statmsg = "BPF or BCC PMDA not active or "
 					          "'netproc' module not enabled; "
 					          "request ignored!";
 					break;
@@ -1237,10 +1393,10 @@ generic_samp(double sampletime, double nsecs,
 				break;
 
 			   /*
-			   ** accumulated resource consumption per container
+			   ** accumulated resource consumption per container/pod
 			   */
 			   case MCUMCONT:
-				statmsg = "Consumption per container; use 'a' to "
+				statmsg = "Consumption per container/pod; use 'a' to "
 				          "toggle between all/active processes";
 
 				showtype  = MCUMCONT;
@@ -1480,7 +1636,7 @@ generic_samp(double sampletime, double nsecs,
 				break;
 
 			   /*
-			   ** focus on specific container id
+			   ** focus on specific container/pod id
 			   */
 			   case MSELCONT:
 				setalarm2(0, 0);	/* stop the clock */
@@ -1488,49 +1644,25 @@ generic_samp(double sampletime, double nsecs,
 
 				move(statline, 0);
 				clrtoeol();
-				printw("Containerid 12 postitions "
- 				       "(enter=all, "
+				printw("Container or pod name (enter=all, "
 				       "'host'=host processes): ");
 
-				procsel.container[0]  = '\0';
-				scanw("%15s", procsel.container);
-				procsel.container[12] = '\0';
+				procsel.utsname[0]  = '\0';
+				scanw("%15s", procsel.utsname);
+				procsel.utsname[UTSLEN] = '\0';
 
-				switch (strlen(procsel.container))
+				switch (strlen(procsel.utsname))
 				{
                                    case 0:
 					break;	// enter key pressed
 
 				   case 4:	// host?
-					if (strcmp(procsel.container, "host"))
+					if (strcmp(procsel.utsname, "host") == 0)
 					{
-						statmsg="Invalid containerid!";
-						beep();
-						procsel.container[0] = '\0';
-					}
-					else
-					{
-						procsel.container[0] = 'H';
-						procsel.container[1] = '\0';
+						procsel.utsname[0] = 'H';
+						procsel.utsname[1] = '\0';
 					}
 					break;
-
-				   case 12:	// container id
-					(void)strtol(procsel.container, &p, 16);
-
-					if (*p)
-					{
-						statmsg ="Containerid not hex!";
-						beep();
-						procsel.container[0] = '\0';
-					}
-					break;
-
-				   default:
-					statmsg = "Invalid containerid!";
-					beep();
-
-					procsel.container[0] = '\0';
 				}
 
 				noecho();
@@ -1956,7 +2088,12 @@ generic_samp(double sampletime, double nsecs,
 				else
 				{
 					calcpss    = 1;
-					statmsg    = "PSIZE gathering enabled";
+
+					if (rootprivs())
+						statmsg    = "PSIZE gathering enabled";
+					else
+						statmsg    = "PSIZE gathering only "
+						             "for own processes";
 				}
 				break;
 
@@ -2314,7 +2451,7 @@ cumprogs(struct tstat **curprocs, struct tstat *curprogs, int numprocs)
 }
 
 /*
-** accumulate all processes per container in new list
+** accumulate all processes per container/pod in new list
 */
 static int
 cumconts(struct tstat **curprocs, struct tstat *curconts, int numprocs)
@@ -2322,12 +2459,12 @@ cumconts(struct tstat **curprocs, struct tstat *curconts, int numprocs)
 	register int	i, numconts;
 
 	/*
-	** sort list of active processes in order of container (increasing)
+	** sort list of active processes in order of container/pod (increasing)
 	*/
 	qsort(curprocs, numprocs, sizeof(struct tstat *), compcon);
 
 	/*
-	** accumulate all processes per container in the new list
+	** accumulate all processes per container/pod in the new list
 	*/
 	for (numconts=i=0; i < numprocs; i++, curprocs++)
 	{
@@ -2340,16 +2477,16 @@ cumconts(struct tstat **curprocs, struct tstat *curconts, int numprocs)
 		if ((*curprocs)->gen.state == 'E' && suppressexit)
 			continue;
  
-		if ( strcmp(curconts->gen.container,
-                         (*curprocs)->gen.container) != 0)
+		if ( strcmp(curconts->gen.utsname,
+                         (*curprocs)->gen.utsname) != 0)
 		{
 			if (curconts->gen.pid)
 			{
 				numconts++;
 				curconts++;
 			}
-			strcpy(curconts->gen.container,
-			    (*curprocs)->gen.container);
+			strcpy(curconts->gen.utsname,
+			    (*curprocs)->gen.utsname);
 		}
 
 		accumulate(*curprocs, curconts);
@@ -2377,6 +2514,10 @@ accumulate(struct tstat *curproc, struct tstat *curstat)
 	curstat->gen.nthr   += curproc->gen.nthr;
 	curstat->cpu.utime  += curproc->cpu.utime;
 	curstat->cpu.stime  += curproc->cpu.stime;
+	curstat->cpu.nvcsw  += curproc->cpu.nvcsw;
+	curstat->cpu.nivcsw += curproc->cpu.nivcsw;
+	curstat->cpu.rundelay += curproc->cpu.rundelay;
+	curstat->cpu.blkdelay += curproc->cpu.blkdelay;
 
 	if (curproc->dsk.wsz > curproc->dsk.cwsz)
                	nett_wsz = curproc->dsk.wsz -curproc->dsk.cwsz;
@@ -2401,13 +2542,8 @@ accumulate(struct tstat *curproc, struct tstat *curstat)
 
 	if (curproc->gen.state != 'E')
 	{
-		if (curstat->mem.pmem != -1)
-		{
-			if  (curproc->mem.pmem != -1)  // no errors?
-				curstat->mem.pmem += curproc->mem.pmem;
-			else
-				curstat->mem.pmem  = -1;
-		}
+		if  (curproc->mem.pmem != -1)  // no errors?
+			curstat->mem.pmem += curproc->mem.pmem;
 
 		curstat->mem.vmem   += curproc->mem.vmem;
 		curstat->mem.rmem   += curproc->mem.rmem;
@@ -2523,19 +2659,19 @@ procsuppress(struct tstat *curstat, struct pselection *sel)
 	}
 
 	/*
-	** check if only processes related to a particular container
-	** should be shown (container 'H' stands for native host processes)
+	** check if only processes related to a particular container/pod
+	** should be shown (container/pod 'H' stands for native host processes)
 	*/
-	if (sel->container[0])
+	if (sel->utsname[0])
 	{
-		if (sel->container[0] == 'H')	// only host processes
+		if (sel->utsname[0] == 'H')	// only host processes
 		{
-			if (curstat->gen.container[0])
+			if (curstat->gen.utsname[0])
 				return 1;
 		}
 		else
 		{
-			if (memcmp(sel->container, curstat->gen.container, 12))
+			if (memcmp(sel->utsname, curstat->gen.utsname, 12))
 				return 1;
 		}
 	}
@@ -2728,9 +2864,9 @@ generic_init(void)
 			break;
 
 		   case MPROCNET:
-			if ( !(supportflags & NETATOP) )
+			if ( !(supportflags & NETATOP || supportflags & NETATOPBPF) )
 			{
-				fprintf(stderr, "BCC PMDA not active or "
+				fprintf(stderr, "BPF or BCC PMDA not active or "
 					        "'netproc' module not enabled; "
 					        "request ignored!\n");
 				sleep(3);
@@ -2814,9 +2950,21 @@ generic_init(void)
 
 		   case MCALCPSS:
 			if (calcpss)
+			{
 				calcpss = 0;
+			}
 			else
+			{
 				calcpss = 1;
+
+				if (!rootprivs())
+				{
+					fprintf(stderr,
+				 	        "PSIZE gathering only for own "
+					        "processes\n");
+					sleep(3);
+				}
+			}
 			break;
 
 		   case MGETWCHAN:
@@ -2865,7 +3013,7 @@ generic_init(void)
 
        	/*
        	** install catch-routine to finish in a controlled way
-	** and activate cbreak-mode
+	** and activate cbreak mode
        	*/
        	if (screen)
 	{
@@ -2890,13 +3038,17 @@ generic_init(void)
 		noecho();
 		keypad(stdscr, TRUE);
 
-		if (COLS  < 30)
+		/*
+		** verify minimal dimensions
+		*/
+		if (COLS  < MINCOLUMNS || LINES  < MINLINES)
 		{
-			endwin();	// finish curses interface
+			endwin();	// finish ncurses interface
 
-			fprintf(stderr, "Not enough columns available\n"
-			                "(need at least %d columns)\n", 30);
-			fprintf(stderr, "Please resize window....\n");
+			fprintf(stderr,
+				"Terminal size should be at least "
+				"%d columns by %d lines\n",
+				MINCOLUMNS, MINLINES);
 
 			cleanstop(1);
 		}
@@ -2906,10 +3058,51 @@ generic_init(void)
 			use_default_colors();
 			start_color();
 
-			init_pair(COLORINFO,   colorinfo,   -1);
-			init_pair(COLORALMOST, coloralmost, -1);
-			init_pair(COLORCRIT,   colorcrit,   -1);
-			init_pair(COLORTHR,    colorthread, -1);
+			// color definitions
+			//
+			init_color(COLOR_MYORANGE, 675, 500,  50);
+			init_color(COLOR_MYGREEN,    0, 600, 100);
+			init_color(COLOR_MYGREY,   240, 240, 240);
+
+			init_color(COLOR_MYBROWN1, 420, 160, 160);
+			init_color(COLOR_MYBROWN2, 735, 280, 280);
+
+			init_color(COLOR_MYBLUE0,   50,  50, 300);
+			init_color(COLOR_MYBLUE1,   50, 300, 500);
+			init_color(COLOR_MYBLUE2,   50, 500, 800);
+			init_color(COLOR_MYBLUE3,  100, 350, 600);
+			init_color(COLOR_MYBLUE4,  150, 500, 700);
+			init_color(COLOR_MYBLUE5,  200, 650, 800);
+
+			init_color(COLOR_MYGREEN0,  90, 300,   0);
+			init_color(COLOR_MYGREEN1,  90, 400, 100);
+			init_color(COLOR_MYGREEN2,  90, 600, 100);
+
+			// color pair definitions (foreground/background)
+			//
+			init_pair(FGCOLORINFO,     colorinfo,   -1);
+			init_pair(FGCOLORALMOST,   coloralmost, -1);
+			init_pair(FGCOLORCRIT,     colorcrit,   -1);
+			init_pair(FGCOLORTHR,      colorthread, -1);
+                	init_pair(FGCOLORBORDER,   COLOR_CYAN,  -1);
+
+	                init_pair(WHITE_GREEN,     COLOR_WHITE, COLOR_MYGREEN);
+			init_pair(WHITE_ORANGE,    COLOR_WHITE, COLOR_MYORANGE);
+			init_pair(WHITE_RED,       COLOR_WHITE, COLOR_RED);
+			init_pair(WHITE_GREY,      COLOR_WHITE, COLOR_MYGREY);
+			init_pair(WHITE_BLUE,      COLOR_WHITE, COLOR_BLUE);
+			init_pair(WHITE_MAGENTA,   COLOR_WHITE, COLOR_MAGENTA);
+
+			init_pair(WHITE_BROWN1,    COLOR_WHITE, COLOR_MYBROWN1);
+			init_pair(WHITE_BROWN2,    COLOR_WHITE, COLOR_MYBROWN2);
+
+			init_pair(WHITE_BLUE0,     COLOR_WHITE, COLOR_MYBLUE0);
+			init_pair(WHITE_BLUE1,     COLOR_WHITE, COLOR_MYBLUE1);
+			init_pair(WHITE_BLUE2,     COLOR_WHITE, COLOR_MYBLUE2);
+			init_pair(WHITE_BLUE3,     COLOR_WHITE, COLOR_MYBLUE3);
+			init_pair(WHITE_BLUE4,     COLOR_WHITE, COLOR_MYBLUE4);
+			init_pair(WHITE_BLUE5,     COLOR_WHITE, COLOR_MYBLUE5);
+
 		}
 		else
 		{
@@ -2921,6 +3114,9 @@ generic_init(void)
 	signal(SIGTERM,  cleanstop);
 }
 
+
+
+
 /*
 ** show help information in interactive mode
 */
@@ -2928,7 +3124,11 @@ static struct helptext {
 	char *helpline;
 	char helparg;
 } helptext[] = {
-	{"Figures shown for active processes:\n", 		' '},
+	{"Display mode:\n", ' '},
+	{"\t'%c'  - show bar graphs for system utilization (toggle)\n",
+								MBARGRAPH},
+	{"\n",							' '},
+	{"Information in text mode for active processes:\n", 	' '},
 	{"\t'%c'  - generic info (default)\n",			MPROCGEN},
 	{"\t'%c'  - memory details\n",				MPROCMEM},
 	{"\t'%c'  - disk details\n",				MPROCDSK},
@@ -2954,14 +3154,14 @@ static struct helptext {
 	{"\t'%c'  - total resource consumption per user\n", 	MCUMUSER},
 	{"\t'%c'  - total resource consumption per program (i.e. same "
 	 "process name)\n",					MCUMPROC},
-	{"\t'%c'  - total resource consumption per container\n",MCUMCONT},
+	{"\t'%c'  - total resource consumption per container/pod\n",MCUMCONT},
 	{"\n",							' '},
 	{"Process selections (keys shown in header line):\n",	' '},
 	{"\t'%c'  - focus on specific user name           "
 	                              "(regular expression)\n", MSELUSER},
 	{"\t'%c'  - focus on specific program name        "
 	                              "(regular expression)\n", MSELPROC},
-	{"\t'%c'  - focus on specific container id (CID)\n",    MSELCONT},
+	{"\t'%c'  - focus on specific container/pod name\n",    MSELCONT},
 	{"\t'%c'  - focus on specific command line string "
 	                              "(regular expression)\n", MSELARG},
 	{"\t'%c'  - focus on specific process id (PID)\n",      MSELPID},
@@ -2974,12 +3174,12 @@ static struct helptext {
 	{"Screen-handling:\n",					      ' '},
 	{"\t^L   - redraw the screen                       \n",	      ' '},
 	{"\tPgDn - show next page in the process list (or ^F)\n",     ' '},
-	{"\tArDn - arrow-down for next line in process list\n",       ' '},
 	{"\tPgUp - show previous page in the process list (or ^B)\n", ' '},
+	{"\tArDn - arrow-down for next line in process list\n",       ' '},
 	{"\tArUp   arrow-up for previous line in process list\n",     ' '},
 	{"\n",							' '},
 	{"\tArRt - arrow-right for next character in full command line\n", ' '},
-	{"\tArLt - arrow-left  for previous character in full command line\n",
+	{"\tArLt - arrow-left for previous character in full command line\n",
 									' '},
 	{"\n",							' '},
 	{"Presentation (keys shown in header line):\n",  	' '},
@@ -3055,35 +3255,55 @@ showhelp(int helpline)
 		*/
 		if (i >= winlines-2 && shown >= tobeshown)
 		{
-			wmove    (helpwin, winlines-1, 0);
-			wclrtoeol(helpwin);
-			wprintw  (helpwin, "Press 'q' to leave help, " 
-					"space for next page or "
-					"other key for next line... ");
+			int inputkey;
 
-			switch (wgetch(helpwin))
+			wmove(helpwin, winlines-1, 0);
+			wclrtoeol(helpwin);
+		      	wprintw(helpwin, "Press q (leave help), " 
+					"space (next page), "
+					"Enter (next line) or select key...");
+
+			keypad(helpwin, 1);	// recognize keypad keys
+
+			switch (inputkey = wgetch(helpwin))
 			{
+			   case KEY_NPAGE:
+			   case KEY_PPAGE:
+			   case KEY_UP:
+			   case KEY_DOWN:
+			   case KEY_LEFT:
+			   case KEY_RIGHT:
+				break;		// ignore keypad keys
+
 			   case 'q':
 				delwin(helpwin);
 				return;
+
 			   case ' ':
 				shown = 0;
 				tobeshown = winlines-1;
 				break;
-			   default:
+
+			   case '\n':
 				shown = 0;
 				tobeshown = 1;
+				break;
+
+			   default:
+                		ungetch(inputkey);
+				delwin(helpwin);
+				return;
 			}
 
-			wmove  (helpwin, winlines-1, 0);
+			wmove(helpwin, winlines-1, 0);
 		}
 	}
 
-	wmove    (helpwin, winlines-1, 0);
+	wmove(helpwin, winlines-1, 0);
 	wclrtoeol(helpwin);
-	wprintw  (helpwin, "End of help - press 'q' to leave help... ");
-        while ( wgetch(helpwin) != 'q' );
-	delwin   (helpwin);
+	wprintw(helpwin, "End of help - press 'q' to leave help... ");
+        while (wgetch(helpwin) != 'q');
+	delwin(helpwin);
 }
 
 /*
@@ -3162,7 +3382,7 @@ generic_usage(void)
 	printf("\t  -%c  show cumulated process-info per program "
 	                "(i.e. same name)\n",
 			MCUMPROC);
-	printf("\t  -%c  show cumulated process-info per container\n\n",
+	printf("\t  -%c  show cumulated process-info per container/pod\n\n",
 			MCUMCONT);
 	printf("\t  -%c  sort processes in order of cpu consumption "
 	                "(default)\n",
@@ -3262,9 +3482,6 @@ do_procname(char *name, char *val)
 	}
 }
 
-extern int get_posval(char *name, char *val);
-
-
 void
 do_maxcpu(char *name, char *val)
 {
@@ -3402,6 +3619,14 @@ do_flags(char *name, char *val)
 		   case '-':
 			break;
 
+		   case MBARGRAPH:
+			displaymode = 'D';
+			break;
+
+		   case MBARMONO:
+			barmono = 1;
+			break;
+
 		   case MSORTCPU:
 			showorder = MSORTCPU;
 			break;
