@@ -36,6 +36,140 @@
 #include "showlinux.h"
 
 static void	format_bandw(char *, size_t, count_t);
+static void	gettotwidth(proc_printpair *, int *, int *, int *);
+static int 	*getspacings(proc_printpair *);
+
+char *procprt_TID_ae(struct tstat *, int, double);
+char *procprt_PID_a(struct tstat *, int, double);
+char *procprt_PID_e(struct tstat *, int, double);
+char *procprt_PPID_a(struct tstat *, int, double);
+char *procprt_PPID_e(struct tstat *, int, double);
+char *procprt_VPID_a(struct tstat *, int, double);
+char *procprt_VPID_e(struct tstat *, int, double);
+char *procprt_CTID_a(struct tstat *, int, double);
+char *procprt_CTID_e(struct tstat *, int, double);
+char *procprt_CID_a(struct tstat *, int, double);
+char *procprt_CID_e(struct tstat *, int, double);
+char *procprt_SYSCPU_ae(struct tstat *, int, double);
+char *procprt_USRCPU_ae(struct tstat *, int, double);
+char *procprt_VGROW_a(struct tstat *, int, double);
+char *procprt_VGROW_e(struct tstat *, int, double);
+char *procprt_RGROW_a(struct tstat *, int, double);
+char *procprt_RGROW_e(struct tstat *, int, double);
+char *procprt_MINFLT_ae(struct tstat *, int, double);
+char *procprt_MAJFLT_ae(struct tstat *, int, double);
+char *procprt_VSTEXT_a(struct tstat *, int, double);
+char *procprt_VSTEXT_e(struct tstat *, int, double);
+char *procprt_VSIZE_a(struct tstat *, int, double);
+char *procprt_VSIZE_e(struct tstat *, int, double);
+char *procprt_RSIZE_a(struct tstat *, int, double);
+char *procprt_RSIZE_e(struct tstat *, int, double);
+char *procprt_PSIZE_a(struct tstat *, int, double);
+char *procprt_PSIZE_e(struct tstat *, int, double);
+char *procprt_VSLIBS_a(struct tstat *, int, double);
+char *procprt_VSLIBS_e(struct tstat *, int, double);
+char *procprt_VDATA_a(struct tstat *, int, double);
+char *procprt_VDATA_e(struct tstat *, int, double);
+char *procprt_VSTACK_a(struct tstat *, int, double);
+char *procprt_VSTACK_e(struct tstat *, int, double);
+char *procprt_SWAPSZ_a(struct tstat *, int, double);
+char *procprt_SWAPSZ_e(struct tstat *, int, double);
+char *procprt_LOCKSZ_a(struct tstat *, int, double);
+char *procprt_LOCKSZ_e(struct tstat *, int, double);
+char *procprt_CMD_a(struct tstat *, int, double);
+char *procprt_CMD_e(struct tstat *, int, double);
+char *procprt_RUID_ae(struct tstat *, int, double);
+char *procprt_EUID_a(struct tstat *, int, double);
+char *procprt_EUID_e(struct tstat *, int, double);
+char *procprt_SUID_a(struct tstat *, int, double);
+char *procprt_SUID_e(struct tstat *, int, double);
+char *procprt_FSUID_a(struct tstat *, int, double);
+char *procprt_FSUID_e(struct tstat *, int, double);
+char *procprt_RGID_ae(struct tstat *, int, double);
+char *procprt_EGID_a(struct tstat *, int, double);
+char *procprt_EGID_e(struct tstat *, int, double);
+char *procprt_SGID_a(struct tstat *, int, double);
+char *procprt_SGID_e(struct tstat *, int, double);
+char *procprt_FSGID_a(struct tstat *, int, double);
+char *procprt_FSGID_e(struct tstat *, int, double);
+char *procprt_STDATE_ae(struct tstat *, int, double);
+char *procprt_STTIME_ae(struct tstat *, int, double);
+char *procprt_ENDATE_a(struct tstat *, int, double);
+char *procprt_ENDATE_e(struct tstat *, int, double);
+char *procprt_ENTIME_a(struct tstat *, int, double);
+char *procprt_ENTIME_e(struct tstat *, int, double);
+char *procprt_THR_a(struct tstat *, int, double);
+char *procprt_THR_e(struct tstat *, int, double);
+char *procprt_TRUN_a(struct tstat *, int, double);
+char *procprt_TRUN_e(struct tstat *, int, double);
+char *procprt_TSLPI_a(struct tstat *, int, double);
+char *procprt_TSLPI_e(struct tstat *, int, double);
+char *procprt_TSLPU_a(struct tstat *, int, double);
+char *procprt_TSLPU_e(struct tstat *, int, double);
+char *procprt_TIDLE_a(struct tstat *, int, double);
+char *procprt_TIDLE_e(struct tstat *, int, double);
+char *procprt_POLI_a(struct tstat *, int, double);
+char *procprt_POLI_e(struct tstat *, int, double);
+char *procprt_NICE_a(struct tstat *, int, double);
+char *procprt_NICE_e(struct tstat *, int, double);
+char *procprt_PRI_a(struct tstat *, int, double);
+char *procprt_PRI_e(struct tstat *, int, double);
+char *procprt_RTPR_a(struct tstat *, int, double);
+char *procprt_RTPR_e(struct tstat *, int, double);
+char *procprt_CURCPU_a(struct tstat *, int, double);
+char *procprt_CURCPU_e(struct tstat *, int, double);
+char *procprt_ST_a(struct tstat *, int, double);
+char *procprt_ST_e(struct tstat *, int, double);
+char *procprt_EXC_a(struct tstat *, int, double);
+char *procprt_EXC_e(struct tstat *, int, double);
+char *procprt_S_a(struct tstat *, int, double);
+char *procprt_S_e(struct tstat *, int, double);
+char *procprt_COMMAND_LINE_ae(struct tstat *, int, double);
+char *procprt_NPROCS_ae(struct tstat *, int, double);
+char *procprt_RDDSK_a(struct tstat *, int, double);
+char *procprt_RDDSK_e(struct tstat *, int, double);
+char *procprt_WRDSK_a(struct tstat *, int, double);
+char *procprt_WRDSK_e(struct tstat *, int, double);
+char *procprt_CWRDSK_a(struct tstat *, int, double);
+char *procprt_WCANCEL_a(struct tstat *, int, double);
+char *procprt_WCANCEL_e(struct tstat *, int, double);
+char *procprt_BANDWI_a(struct tstat *, int, double);
+char *procprt_BANDWI_e(struct tstat *, int, double);
+char *procprt_BANDWO_a(struct tstat *, int, double);
+char *procprt_BANDWO_e(struct tstat *, int, double);
+char *procprt_GPULIST_ae(struct tstat *, int, double);
+char *procprt_GPUMEMNOW_ae(struct tstat *, int, double);
+char *procprt_GPUMEMAVG_ae(struct tstat *, int, double);
+char *procprt_GPUGPUBUSY_ae(struct tstat *, int, double);
+char *procprt_GPUMEMBUSY_ae(struct tstat *, int, double);
+char *procprt_WCHAN_a(struct tstat *, int, double);
+char *procprt_WCHAN_e(struct tstat *, int, double);
+char *procprt_RUNDELAY_a(struct tstat *, int, double);
+char *procprt_RUNDELAY_e(struct tstat *, int, double);
+char *procprt_BLKDELAY_a(struct tstat *, int, double);
+char *procprt_BLKDELAY_e(struct tstat *, int, double);
+char *procprt_NVCSW_a(struct tstat *, int, double);
+char *procprt_NVCSW_e(struct tstat *, int, double);
+char *procprt_NIVCSW_a(struct tstat *, int, double);
+char *procprt_NIVCSW_e(struct tstat *, int, double);
+char *procprt_CGROUP_PATH_a(struct tstat *, int, double);
+char *procprt_CGROUP_PATH_e(struct tstat *, int, double);
+char *procprt_CGRCPUWGT_a(struct tstat *, int, double);
+char *procprt_CGRCPUWGT_e(struct tstat *, int, double);
+char *procprt_CGRCPUMAX_a(struct tstat *, int, double);
+char *procprt_CGRCPUMAX_e(struct tstat *, int, double);
+char *procprt_CGRCPUMAXR_a(struct tstat *, int, double);
+char *procprt_CGRCPUMAXR_e(struct tstat *, int, double);
+char *procprt_CGRMEMMAX_a(struct tstat *, int, double);
+char *procprt_CGRMEMMAX_e(struct tstat *, int, double);
+char *procprt_CGRMEMMAXR_a(struct tstat *, int, double);
+char *procprt_CGRMEMMAXR_e(struct tstat *, int, double);
+char *procprt_CGRSWPMAX_a(struct tstat *, int, double);
+char *procprt_CGRSWPMAX_e(struct tstat *, int, double);
+char *procprt_CGRSWPMAXR_a(struct tstat *, int, double);
+char *procprt_CGRSWPMAXR_e(struct tstat *, int, double);
+char *procprt_SORTITEM_ae(struct tstat *, int, double);
+
 
 static char     *columnhead[] = {
 	[MSORTCPU]= "CPU", [MSORTMEM]= "MEM",
@@ -67,7 +201,7 @@ static proc_printpair newelems[MAXITEMS];
  * sumwidth: (ref) returns the total width of the printitems in the array
  * varwidth: (ref) returns the number of variable width items in the array
  */
-void
+static void
 gettotwidth(proc_printpair* elemptr, int *nitems, int *sumwidth, int* varwidth) 
 {
         int i;
@@ -99,7 +233,7 @@ gettotwidth(proc_printpair* elemptr, int *nitems, int *sumwidth, int* varwidth)
  *
  * Note: this function is only to be called when screen is true.
  */
-int *
+static int *
 getspacings(proc_printpair* elemptr) 
 {
         static int spacings[MAXITEMS];
@@ -313,7 +447,7 @@ showprocline(proc_printpair* elemptr, struct tstat *curstat,
 	{
 		if (usecolors && !curstat->gen.isproc)
 		{
-			attron(COLOR_PAIR(COLORTHR));
+			attron(COLOR_PAIR(FGCOLORTHR));
 		}
 		else
 		{
@@ -359,7 +493,7 @@ showprocline(proc_printpair* elemptr, struct tstat *curstat,
 	{
 		if (usecolors && !curstat->gen.isproc)
 		{
-			attroff(COLOR_PAIR(COLORTHR));
+			attroff(COLOR_PAIR(FGCOLORTHR));
 		}
 		else
 		{
@@ -509,10 +643,10 @@ procprt_CID_a(struct tstat *curstat, int avgval, double nsecs)
 {
         static char buf[64];
 
-	if (curstat->gen.container[0])
-        	pmsprintf(buf, sizeof buf, "%-12s", curstat->gen.container);
+	if (curstat->gen.utsname[0])
+        	pmsprintf(buf, sizeof buf, "%-15s", curstat->gen.utsname);
 	else
-        	pmsprintf(buf, sizeof buf, "%-12s", "host--------");
+        	pmsprintf(buf, sizeof buf, "%-15s", "host-----------");
 
         return buf;
 }
@@ -522,16 +656,16 @@ procprt_CID_e(struct tstat *curstat, int avgval, double nsecs)
 {
         static char buf[64];
 
-	if (curstat->gen.container[0])
-        	pmsprintf(buf, sizeof buf, "%-12s", curstat->gen.container);
+	if (curstat->gen.utsname[0])
+        	pmsprintf(buf, sizeof buf, "%-15s", curstat->gen.utsname);
 	else
-        	pmsprintf(buf, sizeof buf, "%-12s", "?");
+        	pmsprintf(buf, sizeof buf, "%-15s", "?");
 
         return buf;
 }
 
 proc_printdef procprt_CID = 
-   { "CID         ", "CID", procprt_CID_a, procprt_CID_e, 12};
+   { "CID/POD     ", "CID", procprt_CID_a, procprt_CID_e, 15};
 /***************************************************************/
 char *
 procprt_SYSCPU_ae(struct tstat *curstat, int avgval, double nsecs)
@@ -1157,6 +1291,24 @@ procprt_TSLPU_e(struct tstat *curstat, int avgval, double nsecs)
 proc_printdef procprt_TSLPU = 
    { "TSLPU", "TSLPU", procprt_TSLPU_a, procprt_TSLPU_e, 5 };
 /***************************************************************/
+char *
+procprt_TIDLE_a(struct tstat *curstat, int avgval, double nsecs)
+{
+        static char buf[15];
+
+        sprintf(buf, "%5d", curstat->gen.nthridle);
+        return buf;
+}
+
+char *
+procprt_TIDLE_e(struct tstat *curstat, int avgval, double nsecs)
+{
+        return "    0";
+}
+
+proc_printdef procprt_TIDLE = 
+   { "TIDLE", "TIDLE", procprt_TIDLE_a, procprt_TIDLE_e, 5 };
+/***************************************************************/
 #define SCHED_NORMAL	0
 #define SCHED_FIFO	1
 #define SCHED_RR	2
@@ -1486,7 +1638,7 @@ procprt_TCPRCV_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_TCPRCV_e(struct tstat *curstat, int avgval, double nsecs) 
 {      
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1517,7 +1669,7 @@ procprt_TCPRASZ_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_TCPRASZ_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1547,7 +1699,7 @@ procprt_TCPSND_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_TCPSND_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1577,7 +1729,7 @@ procprt_TCPSASZ_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_TCPSASZ_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1607,7 +1759,7 @@ procprt_UDPRCV_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_UDPRCV_e(struct tstat *curstat, int avgval, double nsecs) 
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1638,7 +1790,7 @@ procprt_UDPRASZ_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_UDPRASZ_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1669,7 +1821,7 @@ procprt_UDPSND_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_UDPSND_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1699,7 +1851,7 @@ procprt_UDPSASZ_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_UDPSASZ_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
         
@@ -1731,7 +1883,7 @@ procprt_RNET_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_RNET_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[10];
  
@@ -1760,7 +1912,7 @@ procprt_SNET_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_SNET_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
 	        static char buf[10];
         
@@ -1788,7 +1940,7 @@ procprt_BANDWI_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_BANDWI_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[16];
 		count_t     rkbps = (curstat->net.tcprsz + curstat->net.udprsz)
@@ -1817,7 +1969,7 @@ procprt_BANDWO_a(struct tstat *curstat, int avgval, double nsecs)
 char *
 procprt_BANDWO_e(struct tstat *curstat, int avgval, double nsecs)
 {
-	if (supportflags & NETATOPD)
+	if (supportflags & NETATOPD || supportflags & NETATOPBPF )
 	{
         	static char buf[16];
 		count_t     skbps = (curstat->net.tcpssz + curstat->net.udpssz)
@@ -2038,6 +2190,42 @@ proc_printdef procprt_BLKDELAY =
    { "BDELAY", "BDELAY", procprt_BLKDELAY_a, procprt_BLKDELAY_e, 6};
 /***************************************************************/
 char *
+procprt_NVCSW_a(struct tstat *curstat, int avgval, double nsecs)
+{
+	static char buf[64];
+
+        val2valstr(curstat->cpu.nvcsw, buf, sizeof buf, 6, avgval, nsecs);
+	return buf;
+}
+
+char *
+procprt_NVCSW_e(struct tstat *curstat, int avgval, double nsecs)
+{
+	return "     -";
+}
+
+proc_printdef procprt_NVCSW =
+   { " NVCSW", "NVCSW", procprt_NVCSW_a, procprt_NVCSW_e, 6 };
+/***************************************************************/
+char *
+procprt_NIVCSW_a(struct tstat *curstat, int avgval, double nsecs)
+{
+	static char buf[64];
+
+        val2valstr(curstat->cpu.nivcsw, buf, sizeof buf, 6, avgval, nsecs);
+	return buf;
+}
+
+char *
+procprt_NIVCSW_e(struct tstat *curstat, int avgval, double nsecs)
+{
+	return "     -";
+}
+
+proc_printdef procprt_NIVCSW =
+   { "NIVCSW", "NIVCSW", procprt_NIVCSW_a, procprt_NIVCSW_e, 6 };
+/***************************************************************/
+char *
 procprt_CGROUP_PATH_a(struct tstat *curstat, int avgval, double nsecs)
 {
         extern proc_printdef procprt_CGROUP_PATH;
@@ -2056,9 +2244,9 @@ procprt_CGROUP_PATH_a(struct tstat *curstat, int avgval, double nsecs)
 		return "";
 
         if (screen) 
-                sprintf(buf, "%-*.*s", curwidth, curwidth, pline+curoffset);
+                pmsprintf(buf, sizeof buf, "%-*.*s", curwidth, curwidth, pline+curoffset);
         else
-                sprintf(buf, "%.*s", CGRLEN, pline+curoffset);
+                pmsprintf(buf, sizeof buf, "%.*s", CGRLEN, pline+curoffset);
 
         return buf;
 }
