@@ -34,9 +34,6 @@ import os
 from pcp import pmapi, pmconfig
 from cpmapi import PM_CONTEXT_ARCHIVE, PM_IN_NULL, PM_DEBUG_APPL1, PM_TIME_SEC
 
-if sys.version_info[0] >= 3:
-    long = int # pylint: disable=redefined-builtin
-
 # Default config
 DEFAULT_CONFIG = ["./pcp2XXX.conf", "$HOME/.pcp2XXX.conf", "$HOME/.pcp/pcp2XXX.conf", "$PCP_SYSCONF_DIR/pcp2XXX.conf"]
 
