@@ -34,8 +34,9 @@ require DynaLoader;
 	PM_ERR_BOTCH PM_ERR_TOOSMALL PM_ERR_TOOBIG PM_ERR_FAULT
 	PM_ERR_THREAD PM_ERR_NOCONTAINER PM_ERR_BADSTORE PM_ERR_LOGHOST
 	PM_ERR_LOGCHANGETYPE PM_ERR_LOGCHANGESEM PM_ERR_LOGCHANGEINDOM
-	PM_ERR_LOGCHANGEUNITS PM_ERR_NEEDCLIENTCERT
-	PM_ERR_BADDERIVE PM_ERR_NOLABELS PM_ERR_PMDAFENCED
+	PM_ERR_LOGCHANGEUNITS PM_ERR_LOGOVERLAP PM_ERR_NEEDCLIENTCERT
+	PM_ERR_BADDERIVE PM_ERR_NOLABELS PM_ERR_PMDAFENCED PM_ERR_RECTYPE
+	PM_ERR_FEATURE PM_ERR_TLS PM_ERR_ARG
 	PM_ERR_NYI
 	PMDA_FETCH_NOVALUES PMDA_FETCH_STATIC PMDA_FETCH_DYNAMIC
 );
@@ -96,11 +97,11 @@ sub PM_ERR_GENERIC	{ -12345; }	# Generic error, already reported above
 sub PM_ERR_PMNS		{ -12346; }	# Problems parsing PMNS definitions
 sub PM_ERR_NOPMNS	{ -12347; }	# PMNS not accessible
 sub PM_ERR_DUPPMNS	{ -12348; }	# Attempt to reload the PMNS
-sub PM_ERR_TEXT		{ -12349; }	# Oneline or help text is not available
+sub PM_ERR_TEXT		{ -12349; }	# One-line or help text is not available
 sub PM_ERR_APPVERSION	{ -12350; }	# Metric not supported by this version of monitored application
 sub PM_ERR_VALUE	{ -12351; }	# Missing metric value(s)
 sub PM_ERR_TIMEOUT	{ -12353; }	# Timeout waiting for a response from PMCD
-sub PM_ERR_NODATA	{ -12354; }	# Empty archive log file
+sub PM_ERR_NODATA	{ -12354; }	# Empty archive file
 sub PM_ERR_RESET	{ -12355; }	# PMCD reset or configuration change
 sub PM_ERR_NAME		{ -12357; }	# Unknown metric name
 sub PM_ERR_PMID		{ -12358; }	# Unknown or illegal metric identifier
@@ -115,17 +116,17 @@ sub PM_ERR_PROFILE	{ -12365; }	# Explicit instance identifier(s) required
 sub PM_ERR_IPC		{ -12366; }	# IPC protocol failure
 sub PM_ERR_EOF		{ -12368; }	# IPC channel closed
 sub PM_ERR_NOTHOST	{ -12369; }	# Operation requires context with host source of metrics
-sub PM_ERR_EOL		{ -12370; }	# End of PCP archive log
+sub PM_ERR_EOL		{ -12370; }	# End of PCP archive
 sub PM_ERR_MODE		{ -12371; }	# Illegal mode specification
-sub PM_ERR_LABEL	{ -12372; }	# Illegal label record at start of a PCP archive log file
-sub PM_ERR_LOGREC	{ -12373; }	# Corrupted record in a PCP archive log
-sub PM_ERR_LOGFILE	{ -12375; }	# Missing PCP archive log file
+sub PM_ERR_LABEL	{ -12372; }	# Illegal label record at start of a PCP archive file
+sub PM_ERR_LOGREC	{ -12373; }	# Corrupted record in a PCP archive
+sub PM_ERR_LOGFILE	{ -12375; }	# Missing PCP archive file
 sub PM_ERR_NOTARCHIVE	{ -12374; }	# Operation requires context with archive source of metrics
 sub PM_ERR_NOCONTEXT	{ -12376; }	# Attempt to use an illegal context
 sub PM_ERR_PROFILESPEC	{ -12377; }	# NULL pmInDom with non-NULL instlist
-sub PM_ERR_PMID_LOG	{ -12378; }	# Metric not defined in the PCP archive log
-sub PM_ERR_INDOM_LOG	{ -12379; }	# Instance domain identifier not defined in the PCP archive log
-sub PM_ERR_INST_LOG	{ -12380; }	# Instance identifier not defined in the PCP archive log
+sub PM_ERR_PMID_LOG	{ -12378; }	# Metric not defined in the PCP archive
+sub PM_ERR_INDOM_LOG	{ -12379; }	# Instance domain identifier not defined in the PCP archive
+sub PM_ERR_INST_LOG	{ -12380; }	# Instance identifier not defined in the PCP archive
 sub PM_ERR_NOPROFILE	{ -12381; }	# Missing profile - protocol botch
 sub PM_ERR_NOAGENT	{ -12386; }	# No PMCD agent for domain of request
 sub PM_ERR_PERMISSION	{ -12387; }	# No permission to perform requested operation
@@ -154,6 +155,10 @@ sub PM_ERR_NEEDCLIENTCERT { -12408; }	# PMCD requires a client certificate
 sub PM_ERR_BADDERIVE	{ -12409; }	# Derived metric definition failed
 sub PM_ERR_NOLABELS	{ -12410; }	# No support for metric label metadata
 sub PM_ERR_PMDAFENCED	{ -12411; }	# PMDA is currently fenced and unable to respond to requests
+sub PM_ERR_RECTYPE	{ -12412; }	# Incorrect record type in an archive
+sub PM_ERR_FEATURE	{ -12413; }	# Archive feature not supported
+sub PM_ERR_TLS		{ -12414; }	# TLS protocol failure
+sub PM_ERR_ARG		{ -12415; }	# Bad value for function argument
 
 sub PM_ERR_NYI		{ -21344; }	# Functionality not yet implemented
 
