@@ -1293,14 +1293,6 @@ _install()
     echo "Terminate PMDA if already installed ..."
     __pmda_cull $iam $domain
 
-    # Rotate log files
-    #
-    if [ -f $PCP_LOG_DIR/pmcd/$iam.log ]
-    then
-	rm -f $PCP_LOG_DIR/pmcd/$iam.log.prev
-	mv -f $PCP_LOG_DIR/pmcd/$iam.log $PCP_LOG_DIR/pmcd/$iam.log.prev
-    fi
-
     # Add PMDA to pmcd's configuration file
     #
     echo "Updating the PMCD control file, and notifying PMCD ..."
@@ -1314,6 +1306,10 @@ _install()
     #
     if $do_check
     then
+	# for some PMDAs in QA, we may need to wait a little longer,
+	# or differently
+	#
+	[ -n "$PCPQA_CHECK_DELAY" ] && check_delay=$PCPQA_CHECK_DELAY
 	__delay_int=`echo $check_delay | sed -e 's/\..*//g'`
 	[ "$__delay_int" -gt 5 ] && echo "Wait $check_delay seconds for the $iam agent to initialize ..."
 	$PCP_BINADM_DIR/pmsleep $check_delay
@@ -1593,7 +1589,7 @@ do
 	    shift
 	    ;;
 
-	-Q)	# skip check for metrics going away
+	-Q)	# skip check for metrics appearing or going away
 	    do_check=false
 	    ;;
 
