@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2013 Red Hat.
+ * Copyright (c) 2012-2018 Red Hat.
  * Copyright (c) 1995-2001,2004 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,7 @@
  */
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include "pmcd.h"
 
 #define MIN_CLIENTS_ALLOC 8
@@ -24,8 +24,6 @@ __pmFdSet	clientFds;		/* for client select() */
 
 static int	clientSize;
 
-extern void	Shutdown(void);
-
 /*
  * For PMDA_INTERFACE_5 or later PMDAs, post a notification that
  * a context has been closed.
@@ -43,13 +41,11 @@ NotifyEndContext(int ctx)
 	    pmdaInterface	*dp = &agent[i].ipc.dso.dispatch;
 	    if (dp->comm.pmda_interface >= PMDA_INTERFACE_5) {
 		if (dp->version.four.ext->e_endCallBack != NULL) {
-#ifdef PCP_DEBUG
-		    if (pmDebug & DBG_TRACE_CONTEXT) {
+		    if (pmDebugOptions.context) {
 			fprintf(stderr, "NotifyEndContext: DSO PMDA %s (%d) notified of context %d close\n",
 			    agent[i].pmDomainLabel, agent[i].pmDomainId,
 			    ctx);
 		    }
-#endif
 		    (*(dp->version.four.ext->e_endCallBack))(ctx);
 		}
 	    }
@@ -64,14 +60,11 @@ NotifyEndContext(int ctx)
 	     * bother about any return status from the __pmSendError
 	     * either.
 	     */
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_CONTEXT) {
+	    if (pmDebugOptions.context) {
 		fprintf(stderr, "NotifyEndContext: daemon PMDA %s (%d) notified of context %d close\n",
 		    agent[i].pmDomainLabel, agent[i].pmDomainId, ctx);
 	    }
-#endif
-	    if (_pmcd_trace_mask)
-		pmcd_trace(TR_XMIT_PDU, agent[i].inFd, PDU_ERROR, PM_ERR_NOTCONN);
+	    pmcd_trace(TR_XMIT_PDU, agent[i].inFd, PDU_ERROR, PM_ERR_NOTCONN);
 	    __pmSendError(agent[i].inFd, ctx, PM_ERR_NOTCONN);
 	}
     }
@@ -91,18 +84,24 @@ AcceptNewClient(int reqfd)
     fd = __pmAccept(reqfd, client[i].addr, &addrlen);
     if (fd == -1) {
     	if (neterror() == EPERM) {
-	    __pmNotifyErr(LOG_NOTICE, "AcceptNewClient(%d): "
+	    pmNotifyErr(LOG_NOTICE, "AcceptNewClient(%d): "
 	 	          "Permission Denied\n", reqfd);
-	    client[i].fd = -1;
-	    DeleteClient(&client[i]);
-	    return NULL;	
+	}
+    	else if (neterror() == ECONNABORTED) {
+	    /* quietly ignore this one ... */
+	    ;
 	}
 	else {
-	    __pmNotifyErr(LOG_ERR, "AcceptNewClient(%d) __pmAccept: %s\n",
-	    reqfd, netstrerror());
-	    Shutdown();
-	    exit(1);
+	    /*
+	     * unexpected ... ignore the client (we used to kill off pmcd
+	     * but that seems way too extreme)
+	     */
+	    pmNotifyErr(LOG_ERR, "AcceptNewClient(%d): Unexpected error from __pmAccept: %d: %s\n",
+			    reqfd, neterror(), netstrerror());
 	}
+	client[i].fd = -1;
+	DeleteClient(&client[i]);
+	return NULL;	
     }
     if (fd > maxClientFd)
 	maxClientFd = fd;
@@ -115,10 +114,10 @@ AcceptNewClient(int reqfd)
 
     client[i].fd = fd;
     client[i].status.connected = 1;
+    client[i].status.attributes = 0;
     client[i].status.changes = 0;
     memset(&client[i].attrs, 0, sizeof(__pmHashCtl));
 
-#if 1	// nathans - replace these using connection attributes ... ?
     /*
      * Note seq needs to be unique, but we're using a free running counter
      * and not bothering to check here ... unless we churn through
@@ -126,14 +125,11 @@ AcceptNewClient(int reqfd)
      * we won't have a problem
      */
     client[i].seq = seq++;
-    __pmtimevalNow(&now);
+    pmtimevalNow(&now);
     client[i].start = now.tv_sec;
-#endif
 
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0)
+    if (pmDebugOptions.appl3)
 	fprintf(stderr, "AcceptNewClient(%d): client[%d] (fd %d)\n", reqfd, i, fd);
-#endif
     pmcd_trace(TR_ADD_CLIENT, i, 0, 0);
 
     return &client[i];
@@ -153,7 +149,7 @@ NewClient(void)
 	sz = sizeof(ClientInfo) * clientSize;
 	client = (ClientInfo *) realloc(client, sz);
 	if (client == NULL) {
-	    __pmNoMem("NewClient", sz, PM_RECOV_ERR);
+	    pmNoMem("NewClient", sz, PM_RECOV_ERR);
 	    Shutdown();
 	    exit(1);
 	}
@@ -162,7 +158,7 @@ NewClient(void)
     }
     client[i].addr = __pmSockAddrAlloc();
     if (client[i].addr == NULL) {
-        __pmNoMem("NewClient", __pmSockAddrSize(), PM_RECOV_ERR);
+        pmNoMem("NewClient", __pmSockAddrSize(), PM_RECOV_ERR);
 	Shutdown();
 	exit(1);
     }
@@ -171,34 +167,24 @@ NewClient(void)
     return i;
 }
 
-/*
- * Expose ClientInfo struct for client #n
- */
-ClientInfo *
-GetClient(int n)
-{
-    if (0 <= n && n < nClients && client[n].status.connected)
-	return &client[n];
-    return NULL;
-}
-
 void
 DeleteClient(ClientInfo *cp)
 {
-    int i;
+    __pmHashCtl		*hcp;
+    __pmHashNode	*hp;
+    pmProfile		*profile;
+    int			i;
 
     for (i = 0; i < nClients; i++)
 	if (cp == &client[i])
 	    break;
 
     if (i == nClients) {
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0) {
-	    __pmNotifyErr(LOG_ERR, "DeleteClient: tried to delete non-existent client\n");
+	if (pmDebugOptions.appl3) {
+	    pmNotifyErr(LOG_ERR, "DeleteClient: tried to delete non-existent client\n");
 	    Shutdown();
 	    exit(1);
 	}
-#endif
 	return;
     }
     if (cp->fd != -1) {
@@ -218,26 +204,34 @@ DeleteClient(ClientInfo *cp)
 		maxClientFd = client[i].fd;
 	}
     }
-    for (i = 0; i < cp->szProfile; i++) {
-	if (cp->profile[i] != NULL) {
-	    __pmFreeProfile(cp->profile[i]);
-	    cp->profile[i] = NULL;
+    hcp = &cp->profile;
+    for (i = 0; i < hcp->hsize; i++) {
+	for (hp = hcp->hash[i]; hp != NULL; hp = hp->next) {
+	    profile = (pmProfile *)hp->data;
+	    if (profile != NULL) {
+		__pmFreeProfile(profile);
+		hp->data = NULL;
+	    }
+
 	}
     }
+    __pmHashClear(hcp);
     __pmFreeAttrsSpec(&cp->attrs);
-    memset(&cp->attrs, 0, sizeof(cp->attrs));
+    __pmHashClear(&cp->attrs);
     __pmSockAddrFree(cp->addr);
     cp->addr = NULL;
     cp->status.connected = 0;
+    cp->status.attributes = 0;
+    cp->status.changes = 0;
     cp->fd = -1;
 
     NotifyEndContext(cp-client);
 }
 
 void
-MarkStateChanges(int changes)
+MarkStateChanges(unsigned int changes)
 {
-    int			i;
+    int i;
 
     for (i = 0; i < nClients; i++) {
 	if (client[i].status.connected == 0)
@@ -245,3 +239,28 @@ MarkStateChanges(int changes)
 	client[i].status.changes |= changes;
     }
 }
+
+int
+CheckAccountAccess(ClientInfo *cp)
+{
+    __pmHashNode *node;
+    const char *userid;
+    const char *groupid;
+
+    userid = ((node = __pmHashSearch(PCP_ATTR_USERID, &cp->attrs)) ?
+		(const char *)node->data : NULL);
+    groupid = ((node = __pmHashSearch(PCP_ATTR_GROUPID, &cp->attrs)) ?
+		(const char *)node->data : NULL);
+    if (pmDebugOptions.auth)
+	fprintf(stderr, "CheckAccountAccess: client fd=%d userid=%s groupid=%s\n", cp->fd, userid, groupid);
+    if (!userid || !groupid)
+	if (__pmServerHasFeature(PM_SERVER_FEATURE_CREDS_REQD))
+	    return PM_ERR_PERMISSION;
+    return __pmAccAddAccount(userid, groupid, &cp->denyOps);
+}
+
+int
+CheckClientAccess(ClientInfo *cp)
+{
+    return __pmAccAddClient(cp->addr, &cp->denyOps);
+}
