@@ -71,6 +71,7 @@ refresh_filesys(pmInDom filesys_indom, pmInDom tmpfs_indom,
 	    strcmp(type, "devfs") == 0 ||
 	    strcmp(type, "devpts") == 0 ||
 	    strcmp(type, "devtmpfs") == 0 ||
+	    strcmp(type, "squashfs") == 0 ||
 	    strcmp(type, "selinuxfs") == 0 ||
 	    strcmp(type, "securityfs") == 0 ||
 	    strcmp(type, "configfs") == 0 ||
@@ -84,7 +85,7 @@ refresh_filesys(pmInDom filesys_indom, pmInDom tmpfs_indom,
 	    indom = tmpfs_indom;
 	    device = path;
 	}
-	else if (strncmp(device, "/dev", 4) != 0)
+	else if (strncmp(device, "/dev", 4) != 0 && strcmp(path, "/") != 0)
 	    continue;
 
 	/* keep dm and md persistent names, RHBZ#1349932 */
