@@ -1,7 +1,7 @@
 /*
  * Linux /proc/cpuinfo metrics cluster
  *
- * Copyright (c) 2013-2017 Red Hat.
+ * Copyright (c) 2013-2017,2019-2020 Red Hat.
  * Copyright (c) 2000-2005 Silicon Graphics, Inc.  All Rights Reserved.
  * Portions Copyright (c) 2001 Gilly Ran (gilly@exanet.com) - for the
  * portions supporting the Alpha platform.  All rights reserved.
@@ -23,21 +23,22 @@
 #include "proc_stat.h"
 #include "proc_cpuinfo.h"
 
+static const char *sysfs_path = "sys/devices/system";
+
 /*
  * Refresh state of NUMA node and CPU online state for one
  * CPU or NUMA node ("node" parameter).
  */
 int
-refresh_sysfs_online(unsigned int node_num, const char *node)
+refresh_sysfs_online(char *instname, const char *node_or_cpu)
 {
-    const char *sysfs_path = "sys/devices/system";
     char path[MAXPATHLEN];
     unsigned int online;
     FILE *fp;
     int n;
 
-    pmsprintf(path, sizeof(path), "%s/%s/%s/%s%u/online",
-		linux_statspath, sysfs_path, node, node, node_num);
+    pmsprintf(path, sizeof(path), "%s/%s/%s/%s/online",
+		linux_statspath, sysfs_path, node_or_cpu, instname);
     if ((fp = fopen(path, "r")) == NULL)
 	return 1;
     n = fscanf(fp, "%u", &online);
@@ -47,6 +48,110 @@ refresh_sysfs_online(unsigned int node_num, const char *node)
     return online;
 }
 
+unsigned long
+refresh_sysfs_thermal_throttle(char *instname,
+		const char *core_or_package, const char *count_or_time)
+{
+    char path[MAXPATHLEN];
+    unsigned long value;
+    FILE *fp;
+    int n;
+
+    pmsprintf(path, sizeof(path),
+		"%s/%s/cpu/%s/thermal_throttle/%s_throttle_%s",
+		linux_statspath, sysfs_path, instname,
+		core_or_package, count_or_time);
+    if ((fp = fopen(path, "r")) == NULL)
+	return 0;
+    n = fscanf(fp, "%lu", &value);
+    fclose(fp);
+    if (n != 1)
+	return 0;
+    return value;
+}
+
+int
+refresh_sysfs_frequency_scaling(char *instname, int item, percpu_t *cpu)
+{
+    unsigned long long count, hits, total;
+    unsigned long freq, maxfreq, minfreq;
+    char path[MAXPATHLEN];
+    FILE *fp;
+
+    /* already read the value during this sample? */
+    if (cpu->freq.flags & CPUFREQ_SAMPLED)
+	return 0;
+
+    /*
+     * gather frequency scaling info from (in order of preference): 
+     *     /sys/devices/system/cpu/cpu0/cpufreq/stats/time_in_state
+     * or
+     *     /sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq
+     *     /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq
+     *     /sys/devices/system/cpu/cpu0/cpufreq/scaling_min_freq
+     */
+
+    pmsprintf(path, sizeof(path),
+		"%s/%s/cpu/%s/cpufreq/stats/time_in_state",
+		linux_statspath, sysfs_path, instname);
+    if ((fp = fopen(path, "r")) != NULL) {
+	cpu->freq.flags = CPUFREQ_COUNT|CPUFREQ_TIME|CPUFREQ_MIN|CPUFREQ_MAX;
+	maxfreq = minfreq = 0;
+	hits = total = 0;
+	while (fscanf(fp, "%lu %llu", &freq, &count) == 2) {
+	    freq /= 1000;	/* convert KHz to MHz */
+	    total += (freq * count);
+	    hits += count;
+	    if (freq > maxfreq)
+		maxfreq = freq;
+	    if (minfreq == 0 || freq < minfreq)
+		minfreq = freq;
+	}
+	fclose(fp);
+
+	cpu->freq.max = maxfreq;
+	cpu->freq.min = minfreq;
+	cpu->freq.time = hits;
+	cpu->freq.count = total;
+	cpu->freq.flags |= CPUFREQ_SAMPLED;
+	return 0;
+    }
+
+    /* governor statistics not available, try alternate files */
+    pmsprintf(path, sizeof(path),
+		"%s/%s/cpu/%s/cpufreq/cpuinfo_max_freq",
+		linux_statspath, sysfs_path, instname);
+    if ((fp = fopen(path, "r")) != NULL) {
+	if (fscanf(fp, "%lu", &maxfreq) == 1) {
+	    cpu->freq.max = maxfreq / 1000;	/* convert KHz to MHz */
+	    cpu->freq.flags |= CPUFREQ_MAX;
+	}
+	fclose(fp);
+    }
+    pmsprintf(path, sizeof(path),
+		"%s/%s/cpu/%s/cpufreq/cpuinfo_min_freq",
+		linux_statspath, sysfs_path, instname);
+    if ((fp = fopen(path, "r")) != NULL) {
+	if (fscanf(fp, "%lu", &minfreq) == 1) {
+	    cpu->freq.min = minfreq / 1000;	/* convert KHz to MHz */
+	    cpu->freq.flags |= CPUFREQ_MIN;
+	}
+	fclose(fp);
+    }
+    pmsprintf(path, sizeof(path),
+		"%s/%s/cpu/%s/cpufreq/scaling_cur_freq",
+		linux_statspath, sysfs_path, instname);
+    if ((fp = fopen(path, "r")) != NULL) {
+	if (fscanf(fp, "%lu", &freq) == 1) {
+	    cpu->freq.count = freq / 1000;	/* convert KHz to MHz */
+	    cpu->freq.flags |= CPUFREQ_COUNT;
+	}
+	fclose(fp);
+    }
+    cpu->freq.flags |= CPUFREQ_SAMPLED;
+    return 0;
+}
+
 static char *
 trim_whitespace(char *s)
 {
@@ -113,6 +218,7 @@ refresh_proc_cpuinfo(void)
 	    sts = pmdaCacheLookup(cpus, cpunum, NULL, (void **)&cp);
 	    if (sts < 0 || !cp)
 		continue;
+	    memset(&cp->freq, 0, sizeof(cp->freq));
 	    info = &cp->info;
 	}
 
