@@ -1,7 +1,7 @@
 /*
  * Linux /proc/cpuinfo metrics cluster
  *
- * Copyright (c) 2013-2015,2017,2020 Red Hat.
+ * Copyright (c) 2013-2015,2017,2020-2021 Red Hat.
  * Copyright (c) 2001 Gilly Ran (gilly@exanet.com) for the
  * portions of the code supporting the Alpha platform.
  * All rights reserved.
@@ -18,6 +18,15 @@
  * for more details.
  */
 
+enum {
+    CPUFREQ_SAMPLED	= 1<<0,
+    CPUFREQ_COUNT	= 1<<6,
+    CPUFREQ_TIME	= 1<<7,
+    CPUFREQ_MAX		= 1<<8,
+    CPUFREQ_MIN		= 1<<9,
+};
+
 extern int refresh_proc_cpuinfo(void);
 extern int refresh_sysfs_online(char *, const char *);
+extern int refresh_sysfs_frequency_scaling(char *, int, percpu_t *);
 extern unsigned long refresh_sysfs_thermal_throttle(char *, const char *, const char *);
