@@ -15,6 +15,8 @@
  */
 
 #include <sys/wait.h>
+#include <sys/sysmacros.h>
+#include <sys/vfs.h>
 #include "acct.h"
 #include "getinfo.h"
 
@@ -35,6 +37,14 @@ static uint32_t acct_open_retry_interval       = OPEN_RETRY_INTERVAL;
 static uint32_t acct_check_accounting_interval = CHECK_ACCOUNTING_INTERVAL;
 static uint64_t acct_file_size_threshold       = ACCT_FILE_SIZE_THRESHOLD;
 
+/*
+ * state of PMDA's accounting activity
+ */
+#define MYSTATE_INACTIVE	0
+#define MYSTATE_SYSTEM		1
+#define MYSTATE_PRIVATE		2
+static int32_t acct_state = MYSTATE_INACTIVE;
+
 struct timeval acct_update_interval = {
     .tv_sec = ACCT_TIMER_INTERVAL,
 };
@@ -85,7 +95,7 @@ static time_t
 get_end_time_v3(void *entry)
 {
     return ((struct acct_v3 *)entry)->ac_btime +
-	   (int)(((struct acct_v3 *)entry)->ac_etime / hz);
+	   (int)(((struct acct_v3 *)entry)->ac_etime / _pm_hertz);
 }
 
 static unsigned long long
@@ -112,7 +122,7 @@ acct_fetchCallBack_v3(int item, void *p, pmAtomValue *atom)
 	atom->ul = acctp->ac_tty;
 	break;
     case ACCT_TTYNAME:
-	atom->cp = get_ttyname_info_dev_t(acctp->ac_tty);
+	atom->cp = get_ttyname_info(acctp->ac_tty);
 	break;
     case ACCT_PID:
 	atom->ul = acctp->ac_pid;
@@ -124,13 +134,13 @@ acct_fetchCallBack_v3(int item, void *p, pmAtomValue *atom)
 	atom->ul = acctp->ac_btime;
 	break;
     case ACCT_ETIME:
-	atom->f = acctp->ac_etime / hz;
+	atom->f = acctp->ac_etime / _pm_hertz;
 	break;
     case ACCT_UTIME:
-	atom->f = decode_comp_t(acctp->ac_utime) * 1.0 / hz;
+	atom->f = decode_comp_t(acctp->ac_utime) * 1.0 / _pm_hertz;
 	break;
     case ACCT_STIME:
-	atom->f = decode_comp_t(acctp->ac_stime) * 1.0 / hz;
+	atom->f = decode_comp_t(acctp->ac_stime) * 1.0 / _pm_hertz;
 	break;
     case ACCT_MEM:
 	atom->ull = decode_comp_t(acctp->ac_mem);
@@ -190,9 +200,13 @@ static int
 set_record_size(int fd)
 {
     struct acct_header tmprec;
+    int sts;
 
-    if (read(fd, &tmprec, sizeof(tmprec)) < sizeof(tmprec))
+    if ((sts = read(fd, &tmprec, sizeof(tmprec))) < sizeof(tmprec)) {
+	if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_WARNING, "acct: bad read fd=%d len=%d (not %d), so no process accounting available\n", fd, sts, (int)sizeof(tmprec));
 	return 0;
+    }
 
     if ((tmprec.ac_version & 0x0f) == 3) {
 	acct_file.version = 3;
@@ -205,25 +219,62 @@ set_record_size(int fd)
 	return 1;
     }
 
+    if (pmDebugOptions.appl3)
+	pmNotifyErr(LOG_WARNING, "acct: fd=%d version=%d (not 3), so no process accounting available\n", fd, tmprec.ac_version & 0x0f);
     return 0;
 }
 
 static int
-check_accounting(int fd)
+check_accounting(int fd, const char *name)
 {
     struct stat before, after;
+    char	errmsg[PM_MAXERRMSGLEN];
+    int		sts;
+    static int	onetrip = 1;
 
-    if (fstat(fd, &before) < 0)
+    if (fstat(fd, &before) < 0) {
+	if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_WARNING, "acct: before fstat(fd=%d, name=%s) failed: %s\n", fd, name, pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
 	return 0;
+    }
     if (fork() == 0) {
 	is_child = 1;
-	exit(0);
+	_exit(0);
     }
     wait(0);
-    if (fstat(fd, &after) < 0)
+    if (fstat(fd, &after) < 0) {
+	if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_WARNING, "acct: after fstat(fd=%d, name=%s) failed: %s\n", fd, name, pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
 	return 0;
+    }
+    sts = after.st_size > before.st_size;
+    if (sts <= 0) {
+	if (onetrip) {
+	    pmNotifyErr(LOG_WARNING, "acct: existing pacct file did not grow as expected: system level process accounting disabled or file system full?");
+	    if (pmDebugOptions.appl3) {
+		struct timeval	now;
+		struct statfs	fstat;
+		fprintf(stderr, "acct: pacct growth test failed\n");
+		fprintf(stderr, "    name: %s\n", name);
+		fprintf(stderr, "    size: %" FMT_UINT64 "\n", (uint64_t)after.st_size);
+		fprintf(stderr, "    mtime: %s", ctime(&after.st_mtime));
+		fprintf(stderr, "    ctime: %s", ctime(&after.st_ctime));
+		gettimeofday(&now, NULL);
+		fprintf(stderr, "    nowtime: %s", ctime(&now.tv_sec));
+		fprintf(stderr, "    dev: %d/%d\n", major(after.st_dev), minor(after.st_dev));
+		fstatfs(fd, &fstat);
+		fprintf(stderr, "    filesystem (1KB blocks): size=%" FMT_UINT64 " avail=%" FMT_UINT64 " used=%d%%\n",
+			fstat.f_bsize*(uint64_t)fstat.f_blocks/1024,
+			fstat.f_bsize*(uint64_t)fstat.f_bavail/1024,
+			(int)(100*(fstat.f_blocks-fstat.f_bavail)/fstat.f_blocks));
+	    }
+	    else
+		pmNotifyErr(LOG_INFO, "acct: enable -Dappl3 for more detailed logging");
+	    onetrip = 0;
+	}
+    }
 
-    return after.st_size > before.st_size;
+    return sts;
 }
 
 static void
@@ -236,8 +287,8 @@ init_acct_file_info(void)
 static void
 close_pacct_file(void)
 {
-    if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
-	pmNotifyErr(LOG_DEBUG, "acct: close file=%s\n", acct_file.path);
+    if (pmDebugOptions.appl3)
+	pmNotifyErr(LOG_DEBUG, "acct: close file=%s fd=%d acct_enabled=%d\n", acct_file.path, acct_file.fd, acct_file.acct_enabled);
 
     if (acct_file.fd >= 0) {
 	close(acct_file.fd);
@@ -260,6 +311,7 @@ static int
 open_and_acct(const char *path, int do_acct)
 {
     struct stat file_stat;
+    char	errmsg[PM_MAXERRMSGLEN];
 
     if (acct_file.fd != -1)
 	return 0;
@@ -269,28 +321,41 @@ open_and_acct(const char *path, int do_acct)
     else
 	acct_file.fd = open(path, O_RDONLY);
 
-    if (acct_file.fd < 0)
+    if (acct_file.fd < 0) {
+	if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_DEBUG, "acct: open(\"%s\", ...) do_acct=%d failed: %s\n", path, do_acct, pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
+
 	goto err1;
+    }
 
-    if (fstat(acct_file.fd, &file_stat) < 0)
+    if (fstat(acct_file.fd, &file_stat) < 0) {
+	if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_DEBUG, "acct: fstat \"%s\" failed: %s\n", path, pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
 	goto err2;
+    }
 
-    if (do_acct && acct(path) < 0)
+    if (do_acct && acct(path) < 0) {
+	if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_DEBUG, "acct: acct(\"%s\") failed: %s\n", path, pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
 	goto err2;
+    }
 
-    if (!check_accounting(acct_file.fd))
+    if (!check_accounting(acct_file.fd, path))
 	goto err3;
 
     if (!set_record_size(acct_file.fd))
 	goto err3;
 
-    if (lseek(acct_file.fd, file_stat.st_size, SEEK_SET) < 0)
+    if (lseek(acct_file.fd, file_stat.st_size, SEEK_SET) < 0) {
+	if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_DEBUG, "acct: lseek \"%s\",%lld failed: %s\n", path, (long long)file_stat.st_size, pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
 	goto err3;
+    }
 
     acct_file.prev_size = file_stat.st_size;
     acct_file.path = path;
 
-    if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+    if (pmDebugOptions.appl3)
 	pmNotifyErr(LOG_DEBUG, "acct: open file=%s acct=%d version=%d\n", path, do_acct, acct_file.version);
     return 1;
 
@@ -312,22 +377,30 @@ open_pacct_file(void)
 {
     int ret;
 
+    if (pmDebugOptions.appl3)
+	pmNotifyErr(LOG_DEBUG, "acct: open enable_private=%d timer_id=%d\n", acct_enable_private_acct, acct_timer_id);
+
     ret = open_and_acct(pacct_system_file, 0);
     if (ret) {
 	acct_file.acct_enabled = 0;
+	acct_state = MYSTATE_SYSTEM;
 	return 1;
     }
 
-    if (!acct_enable_private_acct || acct_timer_id == -1)
+    if (!acct_enable_private_acct || acct_timer_id == -1) {
+	acct_state = MYSTATE_INACTIVE;
 	return 0;
+    }
 
     ret = open_and_acct(pacct_private_file, 1);
     if (ret) {
 	acct_file.acct_enabled = 1;
+	acct_state = MYSTATE_PRIVATE;
 	return 1;
     }
 
     acct_file.last_fail_open = time(NULL);
+    acct_state = MYSTATE_INACTIVE;
     return 0;
 }
 
@@ -426,7 +499,7 @@ exists_hash_entry(int i_inst, proc_acct_t *proc_acct)
 static void
 acct_timer(int sig, void *ptr)
 {
-    if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+    if (pmDebugOptions.appl3)
 	pmNotifyErr(LOG_DEBUG, "acct: timer called\n");
     if (acct_file.fd >= 0 && acct_file.acct_enabled && get_file_size() > acct_file_size_threshold)
 	reopen_pacct_file();
@@ -444,7 +517,7 @@ reset_acct_timer(void)
     sts = __pmAFregister(&acct_update_interval, NULL, acct_timer);
     if (sts < 0) {
 	close_pacct_file();
-	if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+	if (pmDebugOptions.appl3)
 	    pmNotifyErr(LOG_DEBUG, "acct: error registering timer: %s\n", pmErrStr(sts));
 	return;
     }
@@ -452,32 +525,67 @@ reset_acct_timer(void)
     reopen_pacct_file();
 }
 
+/*
+ * Probe for system process accounting file ... looking in:
+ * - $PCP_PACCT_SYSTEM_PATH, else
+ * - /var/log/account/pacct, else
+ * - /var/account/pacct, else
+ * - ...
+ */
+static char *pacct_path[] = {
+    "",				/* try $PCP_PACCT_SYSTEM_PATH */
+    "/var/log/account/pacct",
+    "/var/account/pacct",
+    NULL
+};
 static void
 init_pacct_system_file(void)
 {
+    char **path;
     char *tmppath;
-    if ((tmppath = pmGetOptionalConfig("PCP_PACCT_SYSTEM_PATH")) == NULL) {
-	pacct_system_file[0] = '\0';
-    } else {
-	strncpy(pacct_system_file, tmppath, sizeof(pacct_system_file)-1);
+
+    pacct_system_file[0] = '\0';
+    for (path = pacct_path; *path != NULL; path++) {
+	if ((*path)[0] == '\0') {
+	    tmppath = pmGetOptionalConfig("PCP_PACCT_SYSTEM_PATH");
+	    if (tmppath == NULL)
+		continue;
+	}
+	else
+	    tmppath = *path;
+	if (access(tmppath, F_OK) == 0) {
+	    /* file exists, pick me! */
+	    strncpy(pacct_system_file, tmppath, sizeof(pacct_system_file)-1);
+	    break;
+	}
     }
 
-    if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
-	pmNotifyErr(LOG_DEBUG, "acct: initialize pacct_system_file path to %s\n", pacct_system_file);
+    if (pmDebugOptions.appl3) {
+	if (pacct_system_file[0] == '\0')
+	    pmNotifyErr(LOG_DEBUG, "acct: no valid pacct_system_file path found\n");
+	else
+	    pmNotifyErr(LOG_DEBUG, "acct: initialize pacct_system_file path to %s\n", pacct_system_file);
+    }
 }
 
 static void
 init_pacct_private_file(void)
 {
-    char *tmpdir;
-    if ((tmpdir = pmGetOptionalConfig("PCP_TMP_DIR")) == NULL) {
+    char *pacctdir;
+
+    pacct_private_file[0] = '\0';
+    if ((pacctdir = pmGetOptionalConfig("PCP_VAR_DIR")) == NULL) {
 	pacct_private_file[0] = '\0';
     } else {
-	pmsprintf(pacct_private_file, sizeof(pacct_private_file), "%s/pmcd/pacct", tmpdir);
+	pmsprintf(pacct_private_file, sizeof(pacct_private_file), "%s/pmcd/pacct", pacctdir);
     }
 
-    if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
-	pmNotifyErr(LOG_DEBUG, "acct: initialize pacct_private_file path to %s\n", pacct_private_file);
+    if (pmDebugOptions.appl3) {
+	if (pacct_private_file[0] == '\0')
+	    pmNotifyErr(LOG_DEBUG, "acct: cannot initialize pacct_private_file path\n");
+	else
+	    pmNotifyErr(LOG_DEBUG, "acct: initialize pacct_private_file path to %s\n", pacct_private_file);
+    }
 }
 
 void
@@ -513,6 +621,8 @@ refresh_acct(proc_acct_t *proc_acct)
     if (acct_file.fd < 0) {
 	if ((proc_acct->now - acct_file.last_fail_open) > acct_open_retry_interval)
 	    open_pacct_file();
+	else if (pmDebugOptions.appl3)
+	    pmNotifyErr(LOG_DEBUG, "acct: open skipped: retry=%d < limit=%d\n", (int)(proc_acct->now - acct_file.last_fail_open), acct_open_retry_interval);
 	return;
     }
 
@@ -520,9 +630,9 @@ refresh_acct(proc_acct_t *proc_acct)
 	return;
 
     if ((proc_acct->now - acct_file.last_check_accounting) > acct_check_accounting_interval) {
-	if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+	if (pmDebugOptions.appl3)
 	    pmNotifyErr(LOG_DEBUG, "acct: check accounting\n");
-	if (!check_accounting(acct_file.fd)) {
+	if (!check_accounting(acct_file.fd, acct_file.path)) {
 	    reopen_pacct_file();
 	    return;
 	}
@@ -531,7 +641,7 @@ refresh_acct(proc_acct_t *proc_acct)
     need_update = acct_gc(&proc_acct->accthash, proc_acct->now);
 
     if (need_update) {
-	if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+	if (pmDebugOptions.appl3)
 	    pmNotifyErr(LOG_DEBUG, "acct: acct_gc n=%d\n", need_update);
     }
 
@@ -572,7 +682,7 @@ refresh_acct(proc_acct_t *proc_acct)
 	ringbuf_entry.instid.i_inst = i_inst;
 	ringbuf_entry.instid.i_name = acct_ops.get_comm(acctp);
 
-	if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+	if (pmDebugOptions.appl3)
 	    pmNotifyErr(LOG_DEBUG, "acct: hash add pid=%d comm=%s\n", i_inst, acct_ops.get_comm(acctp));
 
 	acct_ringbuf_add(&proc_acct->accthash, &ringbuf_entry);
@@ -582,7 +692,7 @@ refresh_acct(proc_acct_t *proc_acct)
 
     if (need_update) {
 	copy_ringbuf_to_indom(proc_acct->indom, proc_acct->now);
-	if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+	if (pmDebugOptions.appl3)
 	    pmNotifyErr(LOG_DEBUG, "acct: update indom it_numinst=%d\n", proc_acct->indom->it_numinst);
     }
     acct_file.prev_size = acct_file_size;
@@ -612,6 +722,9 @@ acct_fetchCallBack(int i_inst, int item, proc_acct_t *proc_acct, pmAtomValue *at
     case CONTROL_ACCT_ENABLE:
 	atom->ul = acct_enable_private_acct;
 	return 1;
+    case CONTROL_ACCT_STATE:
+	atom->l = acct_state;
+	return 1;
     }
 
     if (acct_file.fd < 0)
@@ -672,7 +785,7 @@ acct_store(pmResult *result, pmdaExt *pmda, pmValueSet *vsp)
 	if ((sts = pmExtractValue(vsp->valfmt, &vsp->vlist[0],
 			PM_TYPE_U32, &av, PM_TYPE_U32)) >= 0) {
 	    int state_changed = !acct_enable_private_acct != !av.ul;
-	    if (pmDebugOptions.libpmda && pmDebugOptions.desperate)
+	    if (pmDebugOptions.appl3)
 		pmNotifyErr(LOG_DEBUG, "acct: store enable_acct old=%d new=%d\n", acct_enable_private_acct, av.ul);
 	    acct_enable_private_acct = av.ul;
 	    if (state_changed)
