@@ -23,6 +23,7 @@
 #include <sys/types.h>
 #include <sys/sysctl.h>
 #include <sys/resource.h>
+#include <sys/sched.h>
 #include <sys/time.h>
 #include <uvm/uvm_param.h>
 #include <sys/utsname.h>
@@ -80,7 +81,7 @@ static pmdaMetric metrictab[] = {
       { PMDA_PMID(CL_SYSCTL,0), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
 	PMDA_PMUNITS(0,0,0,0,0,0) } },
     { (void *)"hinv.physmem",
-      { PMDA_PMID(CL_SYSCTL,1), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_DISCRETE,
+      { PMDA_PMID(CL_SYSCTL,1), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_MBYTE,0,0) } },
     { NULL,	/* kernel.all.load */
       { PMDA_PMID(CL_SPECIAL,2), PM_TYPE_FLOAT, LOADAV_INDOM, PM_SEM_INSTANT,
@@ -220,13 +221,13 @@ static pmdaMetric metrictab[] = {
 	PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
     { NULL,	/* disk.dev.read_bytes */
       { PMDA_PMID(CL_DISK,3), PM_TYPE_U64, DISK_INDOM, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.dev.write_bytes */
       { PMDA_PMID(CL_DISK,4), PM_TYPE_U64, DISK_INDOM, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.dev.total_bytes */
       { PMDA_PMID(CL_DISK,5), PM_TYPE_U64, DISK_INDOM, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.all.read */
       { PMDA_PMID(CL_DISK,6), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
 	PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
@@ -238,13 +239,13 @@ static pmdaMetric metrictab[] = {
 	PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
     { NULL,	/* disk.all.read_bytes */
       { PMDA_PMID(CL_DISK,9), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.all.write_bytes */
       { PMDA_PMID(CL_DISK,10), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.all.total_bytes */
       { PMDA_PMID(CL_DISK,11), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
 
     { NULL,	/* network.interface.mtu */
       { PMDA_PMID(CL_NETIF,0), PM_TYPE_U64, NETIF_INDOM, PM_SEM_INSTANT,
@@ -353,6 +354,9 @@ static pmdaMetric metrictab[] = {
     { NULL,	/* kernel.uname.nodename */
       { PMDA_PMID(CL_SPECIAL,18), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE, 
       PMDA_PMUNITS(0,0,0,0,0,0) } },
+    { NULL,	/* kernel.all.uptime */
+      { PMDA_PMID(CL_SPECIAL,19), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT, 
+      PMDA_PMUNITS(0,1,0,0,PM_TIME_SEC,0) } },
     { NULL,	/* pmda.uname */
       { PMDA_PMID(CL_SPECIAL,20), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE, 
       PMDA_PMUNITS(0,0,0,0,0,0) } },
@@ -404,7 +408,7 @@ int	ncpu;			/* number of cpus in kern.cp_times.* data */
  * Fetch values from sysctl()
  *
  * Expect the result to be xpect bytes to match the PCP data size or
- * anticipated structure size, unless xpect is ==0 in which case the
+ * anticipated structure size, unless xpect is == 0 in which case the
  * size test is skipped.
  */
 static int
@@ -437,55 +441,33 @@ do_sysctl(mib_t *mp, size_t xpect)
     }
     if (xpect > 0 && mp->m_datalen != xpect) {
 	fprintf(stderr, "Error: %s: sysctl(%s) datalen=%d not %d!\n", mp->m_pcpname, mp->m_name, (int)mp->m_datalen, (int)xpect);
-	return 0;
+	return -1;
     }
     return mp->m_datalen;
 }
 
-/*
- * kernel memory reader setup
- */
-struct nlist	symbols[] = {
-	{ .n_name = "_ifnet" },
-	{ .n_name = NULL }
-};
-kvm_t	*kvmp;
-
-static void
-kmemread_init(void)
+static unsigned long
+uptime(void)
 {
-    int		sts;
-    int		i;
-    char	errmsg[_POSIX2_LINE_MAX];
+    static struct timespec	boottime = { 0, 0};
+    int				mib[2];
+    size_t			len;
 
-    /*
-     * If we're running as a daemon PMDA, assume we're setgid kmem,
-     * so we can open /dev/kmem, and downgrade privileges after the
-     * kvm_open().
-     * For a DSO PMDA, we have to assume pmcd has the required
-     * privileges and don't dink with them.
-     */
-    kvmp = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errmsg);
-    if (!isDSO)
-	setgid(getgid());
-    if (kvmp == NULL) {
-	fprintf(stderr, "kmemread_init: kvm_openfiles failed: %s\n", errmsg);
-	return;
-    }
-
-    sts = kvm_nlist(kvmp, symbols);
-    if (sts < 0) {
-	fprintf(stderr, "kmemread_init: kvm_nlist failed: %s\n", pmErrStr(-errno));
-	for (i = 0; i < sizeof(symbols)/sizeof(symbols[0])-1; i++)
-	    symbols[i].n_value = 0;
-	return;
-    }
-    if (pmDebugOptions.appl0) {
-	for (i = 0; i < sizeof(symbols)/sizeof(symbols[0])-1; i++) {
-	    fprintf(stderr, "Info: kernel symbol %s found at 0x%08lx\n", symbols[i].n_name, symbols[i].n_value);
+    if (boottime.tv_sec == 0 && boottime.tv_nsec == 0) {
+	/* one trip */
+	mib[0] = CTL_KERN;
+	mib[1] = KERN_BOOTTIME;
+	len = sizeof(boottime);
+	if (sysctl(mib, 2, &boottime, &len, NULL, 0) == -1) {
+	    /* failed ... not expected */
+	    fprintf(stderr, "Warning: uptime: sysctl failed: %s\n", pmErrStr(-errno));
+	    boottime.tv_nsec = 1;
 	}
     }
-
+    if (boottime.tv_sec == 0 && boottime.tv_nsec == 1)
+	return 0;
+    else
+	return (unsigned long)(time(NULL) - boottime.tv_sec);
 }
 
 /*
@@ -514,14 +496,26 @@ netbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		}
 		break;
 
-	    /* 64-bit integer values */
+	    /* 32-bit or 64-bit integer values, depends on kernel */
 	    case 1:		/* hinv.physmem */
-		sts = do_sysctl(mp, sizeof(atom->ull));
-		if (sts > 0) {
-		    atom->ull = *((__uint64_t *)mp->m_data);
-		    atom->ull /= 1024*1024;
+		sts = do_sysctl(mp, 0);
+		/* sysctl returns bytes, convert to Mbytes */
+		if (sts == sizeof(__uint64_t)) {
+		    /* 64-bit values from sysctl() */
+		    atom->ull = (__uint64_t)((__uint64_t *)mp->m_data)[0]/(1024*1024);
 		    sts = 1;
 		}
+		else if (sts == CPUSTATES*sizeof(__uint32_t)) {
+		    /* 32-bit values from sysctl() */
+		    atom->ull = (__uint64_t)((__uint32_t *)mp->m_data)[0]/(1024*1024);
+		    sts = 1;
+		}
+		else {
+		    fprintf(stderr, "Error: %s: sysctl(%s) datalen=%d not %d (long) or %d (int)!\n",
+			mp->m_pcpname, mp->m_name, sts, (int)sizeof(__uint64_t), 
+			(int)sizeof(__uint32_t)); 
+		    sts = 0;
+		}
 		break;
 
 	    /* string values */
@@ -541,18 +535,30 @@ netbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	    case 5:		/* kernel.all.cpu.sys */
 	    case 6:		/* kernel.all.cpu.intr */
 	    case 7:		/* kernel.all.cpu.idle */
-		sts = do_sysctl(mp, CPUSTATES*sizeof(atom->ull));
-		if (sts > 0) {
-		    /*
-		     * PMID assignment is important in the "-3" below so
-		     * that metrics map to consecutive elements of the
-		     * returned value in the order defined for CPUSTATES,
-		     * i.e. CP_USER, CP_NICE, CP_SYS, CP_INTR and
-		     * CP_IDLE
-		     */
-		    atom->ull = 1000*((__uint64_t *)mp->m_data)[item-3]/cpuhz;
+		/*
+		 * PMID assignment is important in the "-3" below so
+		 * that metrics map to consecutive elements of the
+		 * returned value in the order defined for CPUSTATES,
+		 * i.e. CP_USER, CP_NICE, CP_SYS, CP_INTR and
+		 * CP_IDLE
+		 */
+		sts = do_sysctl(mp, 0);
+		if (sts == CPUSTATES*sizeof(__uint64_t)) {
+		    /* 64-bit values from sysctl() */
+		    atom->ull = 1000*((__uint64_t)((__uint64_t *)mp->m_data)[item-3])/cpuhz;
 		    sts = 1;
 		}
+		else if (sts == CPUSTATES*sizeof(__uint32_t)) {
+		    /* 32-bit values from sysctl() */
+		    atom->ull = 1000*((__uint64_t)((__uint32_t *)mp->m_data)[item-3])/cpuhz;
+		    sts = 1;
+		}
+		else {
+		    fprintf(stderr, "Error: %s: sysctl(%s) datalen=%d not %d (long) or %d (int)!\n",
+			mp->m_pcpname, mp->m_name, sts, (int)(CPUSTATES*sizeof(__uint64_t)), 
+			(int)(CPUSTATES*sizeof(__uint32_t))); 
+		    sts = 0;
+		}
 		break;
 
 	    case 13:		/* kernel.all.hz */
@@ -568,8 +574,8 @@ netbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     }
     else if (cluster == CL_SPECIAL) {
 	/* special cases */
-	double	loadavg[3];
-	char 	uname_string[sizeof(kernel_uname)];
+	double		loadavg[3];
+	static char 	uname_string[sizeof(kernel_uname)];
 
 	switch (item) {
 	    case 0:	/* hinv.ndisk */
@@ -626,6 +632,11 @@ netbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		sts = 1;
 		break;
 
+	    case 19:	/* kernel.all.uptime */
+		atom->ul = uptime();
+		sts = 1;
+		break;
+
 	    case 20: /* pmda.uname */
 		pmsprintf(uname_string, sizeof(uname_string), "%s %s %s %s %s",
 		    kernel_uname.sysname, 
@@ -855,12 +866,12 @@ netbsd_init(pmdaInterface *dp)
 		}
 		if (pmDebugOptions.appl0) {
 		    int	p;
-		    fprintf(stderr, "Info: %s (%s): sysctl metric \"%s\" -> ", (char *)metrictab[m].m_user, pmIDStr(metrictab[m].m_desc.pmid), map[i].m_name);
+		    fprintf(stderr, "Info: %s (%s): sysctl metric \"%s\" -> mib ", (char *)metrictab[m].m_user, pmIDStr(metrictab[m].m_desc.pmid), map[i].m_name);
 		    for (p = 0; p < map[i].m_miblen; p++) {
 			if (p > 0) fputc('.', stderr);
 			fprintf(stderr, "%d", map[i].m_mib[p]);
 		    }
-		    fputc('\n', stderr);
+		    fprintf(stderr, " (len=%d)\n", (int)map[i].m_miblen);
 		}
 		metrictab[m].m_user = (void *)&map[i];
 		break;
@@ -921,8 +932,6 @@ netbsd_init(pmdaInterface *dp)
 	    /*NOTREACHED*/
 	}
     }
-
-    kmemread_init();
 }
 
 static void
