@@ -180,8 +180,8 @@ use vars qw( $pmda %cfg %id2metrics %cur_data %var_metrics);
         # Clients
         connected_clients => { type => PM_TYPE_U32,
                                semantics => PM_SEM_INSTANT,
-                               help => "Number of client connections (excluding connections from slaves)",
-                               longhelp => "Number of client connections (excluding connections from slaves)",
+                               help => "Number of client connections (excluding connections from replicas)",
+                               longhelp => "Number of client connections (excluding connections from replicas)",
                                id => 17,
                            },
         client_longest_output_list => { type => PM_TYPE_U32,
@@ -428,14 +428,14 @@ use vars qw( $pmda %cfg %id2metrics %cur_data %var_metrics);
         # Replication
         role => { type => PM_TYPE_STRING,
                   semantics => PM_SEM_INSTANT,
-                  help => "Replication \"master\" or \"slave\" role",
-                  longhelp => "Value is \"master\" if the instance is slave of no one, or \"slave\" if the\ninstance is enslaved to a master. Note that a slave can be master of\nanother slave (daisy chaining).",
+                  help => "Replication \"master\" or \"replica\" role",
+                  longhelp => "Value is \"master\" if the instance is replica of no one, or \"replica\" if the\ninstance is replicating a master. Note that a replica can be master of\nanother replica (daisy chaining).",
                   id => 57,
               },
         connected_slaves => { type => PM_TYPE_U32,
                               semantics => PM_SEM_INSTANT,
-                              help => "Number of connected slaves",
-                              longhelp => "Number of connected slaves",
+                              help => "Number of connected replicas",
+                              longhelp => "Number of connected replicas",
                               id => 58,
                           },
         master_repl_offset => { type => PM_TYPE_32,
@@ -827,7 +827,7 @@ sub redis_fetch_callback {
         ($refh_redis_info,$refh_inst_keys) = get_redis_data($host,$port);
 
         unless (defined $refh_redis_info) {
-            $pmda->err("Reading from socket ($host:$port) timeouted after $cfg{recv_wait_sec} seconds");
+            $pmda->err("Reading from socket ($host:$port) timed out after $cfg{recv_wait_sec} seconds");
 
             return (PM_ERR_AGAIN, 0)
         }
@@ -878,7 +878,7 @@ sub redis_fetch_callback {
         # Return error if the atom value was not succesfully retrieved
         mydebug("Required metric '$searched_key' was not found in redis INFO or was undefined");
 
-        return (PM_ERR_AGAIN, 0)
+        return (PM_ERR_APPVERSION, 0)
     } else {
         # Success - return (<value>,<success code>)
         mydebug("Returning success");
