@@ -22,9 +22,7 @@
 int
 root_agent_wait(int *status)
 {
-#if defined(HAVE_WAIT3)
-    return wait3(status, WNOHANG, NULL);
-#elif defined(HAVE_WAITPID)
+#if defined(HAVE_WAITPID)
     return waitpid((pid_t)-1, status, WNOHANG);
 #else
     status = 0;
