@@ -343,12 +343,17 @@ metricdesc_t metricdesc[] = {
 
     { "hinv.ncpu",
       { PMDA_PMID(SCLR_SYSINFO,56), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
-	PMDA_PMUNITS(0, 0, 1, 0, 0, PM_COUNT_ONE)
+	PMDA_PMUNITS(0, 0, 0, 0, 0, 0)
       }, -1},
 
     { "hinv.ndisk",
       { PMDA_PMID(SCLR_DISK,20), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
-	PMDA_PMUNITS(0, 0, 1, 0, 0, PM_COUNT_ONE)
+	PMDA_PMUNITS(0, 0, 0, 0, 0, 0)
+      }, -1},
+
+    { "hinv.nfilesys",
+      { PMDA_PMID(SCLR_FILESYS,1023), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
+	PMDA_PMUNITS(0, 0, 0, 0, 0, 0)
       }, -1},
 
     { "pmda.uname",
@@ -1400,7 +1405,6 @@ init_data(int domain)
 {
     int			i;
     int			serial;
-    __pmID_int		*ip;
 
     /*
      * set up kstat() handle ... failure is fatal
@@ -1425,8 +1429,7 @@ init_data(int domain)
     for (i = 0; i < metrictab_sz; i++) {
 	metrictab[i].m_user = &metricdesc[i];
 	metrictab[i].m_desc = metricdesc[i].md_desc;
-	ip = (__pmID_int *)&metricdesc[i].md_desc.pmid;
-	ip->domain = domain;
+	metricdesc[i].md_desc.pmid = pmID_build(domain, pmID_cluster(metricdesc[i].md_desc.pmid), pmID_item(metricdesc[i].md_desc.pmid));
 
 	if (metricdesc[i].md_desc.indom != PM_INDOM_NULL) {
 	    serial = metricdesc[i].md_desc.indom;
@@ -1440,7 +1443,7 @@ init_data(int domain)
      * us but we need properly setup indoms for pmdaCache which means that
      * we have to do it ourselves */
     for (i = 0; i < indomtab_sz; i++) {
-	__pmindom_int(&indomtab[i].it_indom)->domain = domain;
+	indomtab[i].it_indom = pmInDom_build(domain, pmInDom_serial(indomtab[i].it_indom));
     }
 
     /*
