@@ -1,11 +1,12 @@
 /*
  * Copyright (c) 1995,2003 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ * Copyright (c) 2022 Red Hat.
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -59,7 +60,7 @@ service_client(__pmPDU *pb)
     int		j;
     pmDesc	desc;
     pmDesc	foundDesc;
-    pmResult	*resp;
+    __pmResult	*resp;
     pmValueSet	*vsp;
     __pmPDUHdr   *ph = (__pmPDUHdr *)pb;
 
@@ -145,7 +146,7 @@ service_client(__pmPDU *pb)
 
 	}
 
-	pmFreeResult(resp);
+	__pmFreeResult(resp);
 	break;
 	    
     case PDU_ERROR:
