@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include "pmapi.h"
@@ -23,31 +19,32 @@
 int
 main(int argc, char **argv)
 {
-    int		verbose = 0;
+    int		fd, verbose = 0;
 
-    if (argc > 1 && strcmp(argv[1], "-v") == 0) {
+    if (argc > 1 &&
+	(strcmp(argv[1], "-v") == 0 || strcmp(argv[1], "--verbose") == 0)) {
 	verbose = 1;
 	argc--;
 	argv++;
     }
     if (argc != 2 || (argc == 2 && strcmp(argv[1], "-?") == 0)) {
-	fprintf(stderr, "Usage: pmlock [-v] file\n");
+	fprintf(stderr, "Usage: pmlock [-v,--verbose] file\n");
 	exit(1);
     }
-    if (open(argv[1], O_CREAT|O_EXCL|O_RDONLY, 0) < 0) {
+    if ((fd = open(argv[1], O_CREAT|O_EXCL|O_RDONLY, 0)) < 0) {
 	if (verbose) {
-	    if (errno == EACCES) {
+	    if (oserror() == EACCES) {
 		char	*p = dirname(argv[1]);
 		if (access(p, W_OK) == -1)
-		    printf("%s: Directory not writeable\n", p);
+		    printf("%s: Directory not writable\n", p);
 		else
 		    printf("%s: %s\n", argv[1], strerror(EACCES));
 	    }
 	    else
-		printf("%s: %s\n", argv[1], strerror(errno));
+		printf("%s: %s\n", argv[1], osstrerror());
 	}
 	exit(1);
     }
-	
+    close(fd);
     exit(0);
 }
