@@ -117,14 +117,17 @@ pass0(char *fname)
 {
     int		len;
     int		check;
+    int		type;
     int		i;
     int		sts;
     int		nrec = 0;
     int		is = IS_UNKNOWN;
+    int		eol;
     char	*p;
     __pmFILE	*f = NULL;
     int		label_ok = STS_OK;
     char	logBase[MAXPATHLEN];
+    long	offset = 0;
 
     if ((f = __pmFopen(fname, "r")) == NULL) {
 	fprintf(stderr, "%s: cannot open file: %s\n", fname, osstrerror());
@@ -154,18 +157,38 @@ pass0(char *fname)
 	exit(1);
     }
 
-    if (vflag)
+    if (vflag) {
 	fprintf(stderr, "%s: start pass0 ... ", fname);
+	eol = 0;
+    }
 
+    type = 0;
     while ((sts = __pmFread(&len, 1, sizeof(len), f)) == sizeof(len)) {
 	len = ntohl(len);
+	if (len < 2 * sizeof(len)) {
+	    if (vflag && !eol) {
+		fputc('\n', stderr);
+		eol = 1;
+	    }
+	    if (nrec == 0)
+		fprintf(stderr, "%s: illegal header record length (%d) in label record\n", fname, len);
+	    else
+		fprintf(stderr, "%s[record %d]: illegal header record length (%d)\n", fname, nrec, len);
+	    sts = STS_FATAL;
+	    goto done;
+	}
 	len -= 2 * sizeof(len);
-	/* gobble stuff between header and trailer without looking at it */
+	/*
+	 * gobble stuff between header and trailer without looking at it
+	 * ... except for the record type in the case of metadata records
+	 */
 	for (i = 0; i < len; i++) {
 	    check = __pmFgetc(f);
 	    if (check == EOF) {
-		if (vflag)
+		if (vflag && !eol) {
 		    fputc('\n', stderr);
+		    eol = 1;
+		}
 		if (nrec == 0)
 		    fprintf(stderr, "%s: unexpected EOF in label record body, wanted %d, got %d bytes\n", fname, len, i);
 		else
@@ -173,10 +196,18 @@ pass0(char *fname)
 		sts = STS_FATAL;
 		goto done;
 	    }
+	    if (is == IS_META && i <= 3 && nrec > 0) {
+		/*
+		 * first word (after len) for metadata record, save type
+		 */
+		type = (type << 8) | check;
+	    }
 	}
 	if ((sts = __pmFread(&check, 1, sizeof(check), f)) != sizeof(check)) {
-	    if (vflag)
+	    if (vflag && !eol) {
 		fputc('\n', stderr);
+		eol = 1;
+	    }
 	    if (nrec == 0)
 		fprintf(stderr, "%s: unexpected EOF in label record trailer, wanted %d, got %d bytes\n", fname, (int)sizeof(check), sts);
 	    else
@@ -185,10 +216,24 @@ pass0(char *fname)
 	    goto done;
 	}
 	check = ntohl(check);
+	if (check < 2 * sizeof(len)) {
+	    if (vflag && !eol) {
+		fputc('\n', stderr);
+		eol = 1;
+	    }
+	    if (nrec == 0)
+		fprintf(stderr, "%s: illegal trailer record length (%d) in label record\n", fname, check);
+	    else
+		fprintf(stderr, "%s[record %d]: illegal trailer record length (%d)\n", fname, nrec, check);
+	    sts = STS_FATAL;
+	    goto done;
+	}
 	len += 2 * sizeof(len);
 	if (check != len) {
-	    if (vflag)
+	    if (vflag && !eol) {
 		fputc('\n', stderr);
+		eol = 1;
+	    }
 	    if (nrec == 0)
 		fprintf(stderr, "%s: label record length mismatch: header %d != trailer %d\n", fname, len, check);
 	    else
@@ -205,7 +250,6 @@ pass0(char *fname)
 		label_ok = xsts;
 	}
 
-	nrec++;
 	if (is == IS_INDEX) {
 	    /* for index files, done label record, now eat index records */
 	    size_t	record_size;
@@ -220,12 +264,16 @@ pass0(char *fname)
 		/* NOTREACHED */
 	    }
 
+	    nrec++;
 	    while ((sts = __pmFread(buffer, 1, record_size, f)) == record_size) { 
 		nrec++;
 	    }
+	    free(buffer);
 	    if (sts != 0) {
-		if (vflag)
+		if (vflag && !eol) {
 		    fputc('\n', stderr);
+		    eol = 1;
+		}
 		fprintf(stderr, "%s[record %d]: unexpected EOF in index entry, wanted %zd, got %d bytes\n", fname, nrec, record_size, sts);
 		index_state = STATE_BAD;
 		sts = STS_FATAL;
@@ -233,17 +281,58 @@ pass0(char *fname)
 	    }
 	    goto empty_check;
 	}
+	else if (is == IS_META && nrec > 0) {
+	    switch (type) {
+		case TYPE_DESC:
+		case TYPE_TEXT:
+		    /* good for all versions */
+		    break;
+
+		case TYPE_INDOM:
+		case TYPE_INDOM_DELTA:
+		case TYPE_LABEL:
+		    /* not good for V2 */
+		    if ((goldenmagic & 0xff) == PM_LOG_VERS02) {
+			if (vflag && !eol) {
+			    fputc('\n', stderr);
+			    eol = 1;
+			}
+			fprintf(stderr, "%s[record %d]: unexpected record type %s (%d) for V2 archive\n", fname, nrec, __pmLogMetaTypeStr(type), type);
+			sts = STS_FATAL;
+		    }
+		    break;
+
+		case TYPE_INDOM_V2:
+		case TYPE_LABEL_V2:
+		    /* not good for V3 */
+		    if ((goldenmagic & 0xff) == PM_LOG_VERS03) {
+			if (vflag && !eol) {
+			    fputc('\n', stderr);
+			    eol = 1;
+			}
+			fprintf(stderr, "%s[record %d]: unexpected record type %s (%d) for V3 archive\n", fname, nrec, __pmLogMetaTypeStr(type), type);
+			sts = STS_FATAL;
+		    }
+		    break;
+	    }
+	}
+	nrec++;
+	offset = __pmFtell(f);
     }
     if (sts != 0) {
-	if (vflag)
+	if (vflag && !eol) {
 	    fputc('\n', stderr);
+	    eol = 1;
+	}
 	fprintf(stderr, "%s[record %d]: unexpected EOF in record header, wanted %d, got %d bytes\n", fname, nrec, (int)sizeof(len), sts);
 	sts = STS_FATAL;
     }
 empty_check:
     if (sts != STS_FATAL && nrec < 2) {
-	if (vflag)
+	if (vflag && !eol) {
 	    fputc('\n', stderr);
+	    eol = 1;
+	}
 	fprintf(stderr, "%s: contains no PCP data\n", fname);
 	sts = STS_WARNING;
     }
@@ -251,6 +340,9 @@ empty_check:
      * sts == 0 (from __pmFread) => STS_OK
      */
 done:
+    if (sts == STS_FATAL && offset > 0) {
+	fprintf(stderr, "%s: last valid record ends at offset %ld\n", fname, offset);
+    }
     if (is == IS_INDEX) {
 	if (sts == STS_OK)
 	    index_state = STATE_OK;
