@@ -240,7 +240,7 @@ else
     # Salt away previous log, if any ...
     #
     PROGLOGDIR=`dirname "$PROGLOG"`
-    [ -d "$PROGLOGDIR" ] || mkdir_and_chown "$PROGLOGDIR" 755 $PCP_USER:$PCP_GROUP 2>/dev/null
+    [ -d "$PROGLOGDIR" ] || mkdir_and_chown "$PROGLOGDIR" 775 $PCP_USER:$PCP_GROUP 2>/dev/null
     _save_prev_file "$PROGLOG"
     # After argument checking, everything must be logged to ensure no mail is
     # accidentally sent from cron.  Close stdout and stderr, then open stdout
@@ -417,6 +417,8 @@ _configure_pmlogger()
 			$VERBOSE && echo "Reconfigured: \"$configfile\" (pmlogconf)"
 			chown $PCP_USER:$PCP_GROUP "$tmpconfig" >/dev/null 2>&1
 			eval $MV "$tmpconfig" "$configfile"
+			echo "=== pmlogconf changes @ `date` ==="
+			cat $tmp/diag
 		    else
 			_warning "no write access to pmlogconf file \"$configfile\", skip reconfiguration"
 			ls -l "$configfile"
@@ -516,7 +518,7 @@ _wait_for_pmcd()
 	    _dead=false
 	    break
 	fi
-	pmsleep 0.1
+	pmsleep -w 'waiting for pmcd start' 0.1
 	_i=`expr $_i + 1`
     done
     if $_dead
@@ -626,7 +628,7 @@ _check_logger()
 		return 1
 	    fi
 	fi
-	pmsleep 0.1
+	pmsleep -w 'waiting for pmlogger start' 0.1
 	delay=`expr $delay - 1`
 	$VERBOSE && [ `expr $delay % 10` -eq 0 ] && \
 			$PCP_ECHO_PROG $PCP_ECHO_N ".""$PCP_ECHO_C"
@@ -812,7 +814,7 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	#
 	if [ ! -d "$dir" ]
 	then
-	    mkdir_and_chown "$dir" 755 $PCP_USER:$PCP_GROUP >$tmp/tmp 2>&1
+	    mkdir_and_chown "$dir" 775 $PCP_USER:$PCP_GROUP >$tmp/tmp 2>&1
 	    if [ ! -d "$dir" ]
 	    then
 		cat $tmp/tmp
@@ -876,7 +878,7 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 			fi
 		    fi
 		fi
-		pmsleep 0.1
+		pmsleep -w 'waiting for lock' 0.1
 		delay=`expr $delay - 1`
 	    done
 
@@ -1070,11 +1072,6 @@ END				{ print m }'`
             fi
 
 	    _get_logfile
-	    if [ -f $logfile ]
-	    then
-		$VERBOSE && $SHOWME && echo
-		eval $MV -f $logfile $logfile.prior
-	    fi
 
 	    # Notify service manager (if any) for the primary logger ONLY.
 	    [ "$primary" = y ] && args="-N $args"
@@ -1161,7 +1158,7 @@ then
         do
             if [ $delay -gt 0 ]
             then
-                pmsleep 0.1
+                pmsleep -w 'waiting for pmlogger exit' 0.1
                 delay=`expr $delay - 1`
                 continue
             fi
