@@ -21,6 +21,7 @@
  * appl2	pmResult changes
  * appl3	-q and reason for not taking quick exit
  * appl4	config parser
+ * appl5	regexp matching for metric value changes and iname changes
  */
 
 #include <math.h>
@@ -57,7 +58,7 @@ static pmLongOptions longopts[] = {
     { "warnings", 0, 'w', 0, "emit warnings [default is silence]" },
     PMOPT_HELP,
     PMAPI_OPTIONS_TEXT(""),
-    PMAPI_OPTIONS_TEXT("output-archive is required unless -i is specified"),
+    PMAPI_OPTIONS_TEXT("output-archive is required unless -C or -i is specified"),
     PMAPI_OPTIONS_END
 };
 
@@ -212,7 +213,7 @@ writelabel(int do_rewind)
 	assert(old_offset >= 0);
 	__pmRewind(outarch.archctl.ac_mfp);
     }
-    outarch.logctl.label.vol = 0;
+    outarch.logctl.label.vol = outarch.archctl.ac_curvol;
     __pmLogWriteLabel(outarch.archctl.ac_mfp, &outarch.logctl.label);
     if (do_rewind)
 	__pmFseek(outarch.archctl.ac_mfp, (long)old_offset, SEEK_SET);
@@ -406,8 +407,24 @@ parseargs(int argc, char *argv[])
     }
 
     if (opts.errors == 0) {
-	if ((iflag == 0 && opts.optind != argc-2) ||
-	    (iflag == 1 && opts.optind != argc-1))
+	if (iflag) {
+	    if (opts.optind == argc-1)
+		inarch.name = argv[argc-1];
+	    else
+		opts.errors++;
+	}
+	else if (Cflag) {
+	    /* output-archive is sort of optional for -C */
+	    if (opts.optind == argc-2)
+		inarch.name = argv[argc-2];
+	    else if (opts.optind == argc-1)
+		inarch.name = argv[argc-1];
+	    else
+		opts.errors++;
+	}
+	else if (opts.optind == argc-2)
+	    inarch.name = argv[argc-2];
+	else
 	    opts.errors++;
     }
 
@@ -510,7 +527,7 @@ reportconfig(void)
     int			change = 0;
     char		buf[64];
 
-    printf("PCP Archive Log Rewrite Specifications Summary\n");
+    printf("PCP Archive Rewrite Specifications Summary\n");
     change |= (global.flags != 0);
     // TODO WARN about no-ops for changes to V3 label fields in V2 output?
     if (global.flags & GLOBAL_CHANGE_HOSTNAME)
@@ -570,9 +587,36 @@ reportconfig(void)
 	}
     }
     for (mp = metric_root; mp != NULL; mp = mp->m_next) {
-	if (mp->flags != 0 || mp->ip != NULL) {
+	if (mp->flags != 0 || mp->ip != NULL || mp->nvc > 0) {
+	    char	**names;
+	    int		sts;
+
+	    sts = pmNameAll(mp->old_desc.pmid, &names);
+	    if (sts < 0) {
+		printf("Warning: cannot get all names for PMID %s\n", pmIDStr(mp->old_desc.pmid));
+		printf("\nMetric: %s (%s)\n", mp->old_name, pmIDStr(mp->old_desc.pmid));
+	    }
+	    else {
+		printf("\nMetric");
+		if (sts > 1)
+		    putchar('s');
+		putchar(':');
+		/*
+		 * Names are likely to be dups first, primary name last
+		 */
+		for (i = sts-1; i >= 0; i--) {
+		    if (i == sts-2)
+			printf(" [");
+		    else
+			putchar(' ');
+		    printf("%s", names[i]);
+		}
+		if (sts > 1)
+		    putchar(']');
+		printf(" (%s)\n", pmIDStr(mp->old_desc.pmid));
+		free(names);
+	    }
 	    change |= 1;
-	    printf("\nMetric: %s (%s)\n", mp->old_name, pmIDStr(mp->old_desc.pmid));
 	}
 	if (mp->flags & METRIC_CHANGE_PMID) {
 	    printf("pmID:\t\t%s ->", pmIDStr(mp->old_desc.pmid));
@@ -658,6 +702,11 @@ reportconfig(void)
 		printf(" (rescale)");
 	    putchar('\n');
 	}
+	if (mp->flags & METRIC_CHANGE_VALUE) {
+	    for (i = 0; i < mp->nvc; i++) {
+		printf("Value:\t\t/%s/ -> \"%s\"\n", mp->vc[i].pat, mp->vc[i].replace);
+	    }
+	}
 	if (mp->flags & METRIC_DELETE)
 	    printf("DELETE\n");
     }
@@ -1525,47 +1574,26 @@ do_newlabelsets(void)
     }
 }
 
-int
-main(int argc, char **argv)
+void
+open_input(int flags)
 {
     int		sts;
-    int		stslog;			/* sts from nextlog() */
-    int		stsmeta = 0;		/* sts from nextmeta() */
-    int		i;
-    int		ti_idx;			/* next slot for input temporal index */
-    int		dir_fd = -1;		/* poinless initialization to humour gcc */
-    int		doneti = 0;
-    __pmTimestamp	tstamp = { 0, 0 };	/* for last log record */
-    off_t	old_log_offset = 0;	/* log offset before last log record */
-    off_t	old_meta_offset;
-    int		seen_event = 0;
-    metricspec_t	*mp;
 
-    /* process cmd line args */
-    if (parseargs(argc, argv) < 0) {
-	pmUsageMessage(&opts);
-	exit(1);
-    }
-
-    /* input archive */
-    if (iflag == 0)
-	inarch.name = argv[argc-2];
-    else
-	inarch.name = argv[argc-1];
     inarch.logrec = inarch.metarec = NULL;
     inarch.mark = 0;
     inarch.rp = NULL;
 
-    if ((inarch.ctx = pmNewContext(PM_CONTEXT_ARCHIVE, inarch.name)) < 0) {
+    if ((inarch.ctx = pmNewContext(PM_CONTEXT_ARCHIVE | flags, inarch.name)) < 0) {
 	if (inarch.ctx == PM_ERR_NODATA) {
 	    fprintf(stderr, "%s: Warning: empty archive \"%s\" will be skipped\n",
 		    pmGetProgname(), inarch.name);
 	    exit(0);
 	}
 	if (inarch.ctx == PM_ERR_FEATURE) {
+	    /* try w/out feature bits checking */
 	    fprintf(stderr, "%s: Warning: archive \"%s\": unsupported feature bits, other errors may follow ...\n",
 		    pmGetProgname(), inarch.name);
-	    inarch.ctx = pmNewContext(PM_CONTEXT_ARCHIVE | PM_CTXFLAG_NO_FEATURE_CHECK, inarch.name);
+	    inarch.ctx = pmNewContext(PM_CONTEXT_ARCHIVE | flags | PM_CTXFLAG_NO_FEATURE_CHECK, inarch.name);
 	}
 	if (inarch.ctx < 0) {
 	    fprintf(stderr, "%s: Error: cannot open archive \"%s\": %s\n",
@@ -1596,6 +1624,39 @@ main(int argc, char **argv)
 		pmGetProgname(), inarch.version, inarch.name);
 	exit(1);
     }
+}
+
+int
+main(int argc, char **argv)
+{
+    int		sts;
+    int		stslog;			/* sts from nextlog() */
+    int		stsmeta = 0;		/* sts from nextmeta() */
+    int		i;
+    int		ti_idx;			/* next slot for input temporal index */
+    int		dir_fd = -1;		/* poinless initialization to humour gcc */
+    int		doneti = 0;
+    int		in_vol_missing = 0;	/* == 1 if one or more input data volumes missing */
+    __pmTimestamp	tstamp = { 0, 0 };	/* for last log record */
+    off_t	old_log_offset = 0;	/* log offset before last log record */
+    off_t	old_meta_offset;
+    int		old_in_vol;		/* previous input data volume */
+    int		seen_event = 0;
+    metricspec_t	*mp;
+
+    /* process cmd line args */
+    if (parseargs(argc, argv) < 0) {
+	pmUsageMessage(&opts);
+	exit(1);
+    }
+
+    /* input archive ... inarch.name set in parseargs() */
+    if (qflag)
+	/* -q => "peek" only at metadata initially */
+	open_input(PM_CTXFLAG_METADATA_ONLY);
+    else
+	/* no -q => fully functional context needed */
+	open_input(0);
 
     if (outarch.version == 0)
 	outarch.version = inarch.version;
@@ -1718,6 +1779,9 @@ main(int argc, char **argv)
      * process config file(s)
      */
     for (i = 0; i < nconf; i++) {
+	if (pmDebugOptions.appl4) {
+	    fprintf(stderr, "Start config: %s\n", conf[i]);
+	}
 	parseconfig(conf[i]);
     }
 
@@ -1742,9 +1806,22 @@ main(int argc, char **argv)
 	exit(0);
     }
 
-    /* create output log - must be done before writing label */
+    if (qflag) {
+	/*
+	 * rewriting is in the wind, so need a fully functional
+	 * context now
+	 */
+	pmDestroyContext(inarch.ctx);
+	open_input(0);
+    }
+
+    /*
+     * create output log - must be done before writing label
+     * ... and start at the same initial volume (volume 0, 1, ...
+     * may be missing)
+     */
     outarch.archctl.ac_log = &outarch.logctl;
-    if ((sts = __pmLogCreate("", outarch.name, outarch.version, &outarch.archctl)) < 0) {
+    if ((sts = __pmLogCreate("", outarch.name, outarch.version, &outarch.archctl, inarch.ctxp->c_archctl->ac_curvol)) < 0) {
 	fprintf(stderr, "%s: Error: __pmLogCreate(%s,v%d): %s\n",
 		pmGetProgname(), outarch.name, outarch.version, pmErrStr(sts));
 	/*
@@ -1754,6 +1831,11 @@ main(int argc, char **argv)
 	exit(1);
 	/*NOTREACHED*/
     }
+    outarch.archctl.ac_curvol = inarch.ctxp->c_archctl->ac_curvol;
+    if (outarch.archctl.ac_curvol != 0) {
+	/* volume 0 is missing */
+	in_vol_missing =  1;
+    }
 
     /* initialize and write label records */
     newlabel();
@@ -1908,6 +1990,7 @@ main(int argc, char **argv)
 	__pmFflush(outarch.logctl.mdfp);
 	old_meta_offset = __pmFtell(outarch.logctl.mdfp);
 	assert(old_meta_offset >= 0);
+	old_in_vol = inarch.ctxp->c_archctl->ac_curvol;
 
 	in_offset = __pmFtell(inarch.ctxp->c_archctl->ac_mfp);
 	stslog = nextlog();
@@ -1918,6 +2001,8 @@ main(int argc, char **argv)
 	}
 	if (stslog == 1) {
 	    /* volume change */
+	    if (inarch.ctxp->c_archctl->ac_curvol != old_in_vol + 1)
+		in_vol_missing = 1;
 	    if (inarch.ctxp->c_archctl->ac_curvol >= outarch.archctl.ac_curvol+1)
 		/* track input volume numbering */
 		newvolume(inarch.ctxp->c_archctl->ac_curvol);
@@ -1927,6 +2012,7 @@ main(int argc, char **argv)
 		 * rewriting has forced an earlier volume change
 		 */
 		newvolume(outarch.archctl.ac_curvol+1);
+	    needti = 1;
 	}
 	if (pmDebugOptions.appl0) {
 	    fprintf(stderr, "Log: read ");
@@ -2021,9 +2107,15 @@ main(int argc, char **argv)
 		 * pmDesc metadata until it is exhausted, or we find
 		 * a pmInDom metadata record with a timestamp after the
 		 * current pmResult.
+		 *
+		 * The other bad news case is when an input volume is
+		 * missing, and then we may have pmDesc metadata but no
+		 * corresponding pmResult, so if this happens all we
+		 * can do is push on until the timestamp in an indom
+		 * record stops us.
 		 */
 
-		if (!seen_event) {
+		if (!seen_event && !in_vol_missing) {
 		    if (type == PM_TYPE_EVENT || type == PM_TYPE_HIGHRES_EVENT)
 			seen_event = 1;
 		    else {
