@@ -17,7 +17,9 @@
 #include "pmproxy.h"
 #include "pmwebapi.h"
 #include <strings.h>
+#if defined(HAVE_SYS_RESOURCE_H)
 #include <sys/resource.h>
+#endif
 #include <sys/stat.h>
 
 #define MAXPENDING	128	/* maximum number of pending connections */
@@ -36,9 +38,6 @@ static char	*logfile = "pmproxy.log";	/* log file name */
 static int	run_mode = RUN_DAEMON;	/* style of execution, see -f and -F */
 static char	*fatalfile = "/dev/tty";/* fatal messages at startup go here */
 static char	*username;
-static char	*certdb;		/* certificate DB path (NSS) */
-static char	*dbpassfile;		/* certificate DB password file */
-static char     *cert_nickname;         /* alternate nickname for server certificate */
 static char	sockpath[MAXPATHLEN];	/* local unix domain socket path */
 
 static void
@@ -79,9 +78,6 @@ static pmLongOptions longopts[] = {
     { "username", 1, 'U', "USER", "in daemon mode, run as named user [default pcp]" },
     PMAPI_OPTIONS_HEADER("Configuration options"),
     { "config", 1, 'c', "PATH", "path to configuration file (implies --timeseries)"},
-    { "certdb", 1, 'C', "PATH", "path to NSS certificate database (implies --deprecated)" },
-    { "passfile", 1, 'P', "PATH", "password file for certificate database access (implies --deprecated)" },
-    { "certname", 1, 'M', "NAME", "certificate name to use (implies --deprecated)" },
     { "", 0, 'L', 0, "maximum size for PDUs from clients [default 65536]" },
     PMAPI_OPTIONS_HEADER("Connection options"),
     { "interface", 1, 'i', "ADDR", "accept connections on this IP address" },
@@ -90,13 +86,14 @@ static pmLongOptions longopts[] = {
     { "redisport", 1, 'r', "PORT", "Connect to Redis instance on this TCP/IP port (implies --timeseries)" },
     { "redishost", 1, 'h', "HOST", "Connect to Redis instance on this host name (implies --timeseries)" },
     PMAPI_OPTIONS_HEADER("Diagnostic options"),
+    { "", 1, 'T', "TIME", "terminate after an elapsed time interval" },
     { "log", 1, 'l', "PATH", "redirect diagnostics and trace output" },
     { "", 1, 'x', "PATH", "fatal messages at startup sent to file [default /dev/tty]" },
     PMAPI_OPTIONS_END
 };
 
 static pmOptions opts = {
-    .short_options = "Ac:C:dD:Ffh:i:l:L:M:p:P:r:s:tU:x:?",
+    .short_options = "Ac:dD:Ffh:i:l:L:p:r:s:tT:U:x:?",
     .long_options = longopts,
 };
 
@@ -109,6 +106,7 @@ ParseOptions(int argc, char *argv[], int *nports, int *maxpending)
     int		timeseries = 1;
     int		redis_port = 6379;
     char	*redis_host = NULL;
+    char	*endnum;
     const char	*inifile = NULL;
     sds		option;
 
@@ -124,10 +122,6 @@ ParseOptions(int argc, char *argv[], int *nports, int *maxpending)
 	    timeseries = 1;
 	    break;
 
-	case 'C':	/* path to NSS certificate database */
-	    certdb = opts.optarg;
-	    break;
-
 	case 'd':	/* run in deprecated (libpcp, select) mode */
 	    timeseries = 0;
 	    break;
@@ -174,11 +168,6 @@ ParseOptions(int argc, char *argv[], int *nports, int *maxpending)
 	    logfile = opts.optarg;
 	    break;
 
-        case 'M':	/* nickname for server cert, use to query the nssdb */
-            cert_nickname = opts.optarg;
-	    timeseries = 0;
-            break;
-
 	case 'L':	/* maximum size for PDUs from clients */
 	    sts = (int)strtol(opts.optarg, NULL, 0);
 	    if (sts <= 0) {
@@ -199,11 +188,6 @@ ParseOptions(int argc, char *argv[], int *nports, int *maxpending)
 	    }
 	    break;
 
-	case 'P':	/* password file for certificate database access */
-	    dbpassfile = opts.optarg;
-	    timeseries = 0;
-	    break;
-
 	case 'Q':	/* require clients to provide a trusted cert */
 	    __pmServerSetFeature(PM_SERVER_FEATURE_CERT_REQD);
 	    break;
@@ -229,6 +213,17 @@ ParseOptions(int argc, char *argv[], int *nports, int *maxpending)
 	    timeseries = 1;
 	    break;
 
+	case 'T':	/* terminate after given time has elapsed */
+	    sts = pmParseInterval(opts.optarg, &opts.finish, &endnum);
+	    if (sts < 0) {
+		pmprintf("%s: bad -T interval format:\n", pmGetProgname());
+		opts.errors++;
+		free(endnum);
+	    } else {
+	        opts.finish_optarg = opts.optarg;
+	    }
+	    break;
+
 	case 'U':	/* run as user username */
 	    username = opts.optarg;
 	    break;
@@ -314,7 +309,7 @@ void
 Shutdown(void)
 {
     server->shutdown(info);
-    __pmSecureServerShutdown();
+    __pmSecureServerShutdown(NULL, NULL);
     pmNotifyErr(LOG_INFO, "pmproxy Shutdown\n");
     fflush(stderr);
 }
@@ -328,6 +323,7 @@ GetServerInfo(void)
 static void
 set_rlimit_maxfiles(void)
 {
+#ifndef IS_MINGW
     struct rlimit limit;
 
     if (getrlimit(RLIMIT_NOFILE, &limit) != 0)
@@ -337,6 +333,7 @@ set_rlimit_maxfiles(void)
 	if (setrlimit(RLIMIT_NOFILE, &limit) != 0)
 	    pmNotifyErr(LOG_ERR, "Cannot adjust open file limits\n");
     }
+#endif
 }
 
 #define ENV_WARN_PORT		1
@@ -459,10 +456,6 @@ main(int argc, char *argv[])
 	    DontStart();
     }
 
-    if (!timeseries &&
-        __pmSecureServerCertificateSetup(certdb, dbpassfile, cert_nickname) < 0)
-	DontStart();
-
     fprintf(stderr, "pmproxy: PID = %" FMT_PID, mainpid);
     fprintf(stderr, ", PDU version = %u", PDU_VERSION);
 #ifdef HAVE_GETUID
@@ -472,7 +465,7 @@ main(int argc, char *argv[])
     fflush(stderr);
 
     /* Loop processing client connections and server responses */
-    server->loop(info);
+    server->loop(info, opts.finish_optarg? &opts.finish : NULL);
 
     /* inform service manager and shutdown cleanly */
     __pmServerNotifyServiceManagerStopping(mainpid);
