@@ -82,18 +82,18 @@ proxymetrics(struct proxy *proxy, enum proxy_registry prid)
 	return proxy->metrics[prid];
 
     pmsprintf(path, sizeof(path), "%s%cpmproxy%c%s",
-	    pmGetConfig("PCP_TMP_DIR"), sep, sep, server_metrics[prid].group);
+	    pmGetConfig("PCP_TMP_DIR"), sep, sep, server_metrics[prid].group); //creates a new file
     if ((file = strdup(path)) == NULL)
 	return NULL;
 
     if (prid == METRICS_SERVER)
 	flags |= MMV_FLAG_NOPREFIX;
-    if ((registry = mmv_stats_registry(file, prid, flags)) != NULL)
+    if ((registry = mmv_stats_registry(file, prid, flags)) != NULL) 
 	server_metrics[prid].path = file;
     else
 	free(file);
     proxy->metrics[prid] = registry;
-    return registry;
+    return registry; //once you have a registry you can start adding metrics to it 
 }
 
 void
@@ -218,7 +218,7 @@ on_client_close(uv_handle_t *handle)
 {
     struct client	*client = (struct client *)handle;
 
-    if (pmDebugOptions.context | pmDebugOptions.desperate)
+    if (pmDebugOptions.context || pmDebugOptions.desperate)
 	fprintf(stderr, "%s: client %p connection closed\n",
 			"on_client_close", client);
 
@@ -453,7 +453,9 @@ on_protocol_read(uv_stream_t *stream, ssize_t nread, const uv_buf_t *buf)
 static void
 on_client_read(uv_stream_t *stream, ssize_t nread, const uv_buf_t *buf)
 {
+#ifdef HAVE_OPENSSL
     struct proxy	*proxy = (struct proxy *)stream->data;
+#endif
     struct client	*client = (struct client *)stream;
 
     if (nread > 0) {
@@ -499,7 +501,7 @@ on_client_connection(uv_stream_t *stream, int status)
 			"ENOMEM", strerror(ENOMEM));
 	return;
     }
-    if (pmDebugOptions.context | pmDebugOptions.af)
+    if (pmDebugOptions.context || pmDebugOptions.af)
 	fprintf(stderr, "%s: accept new client %p\n",
 			"on_client_connection", client);
 
@@ -567,8 +569,8 @@ open_request_port(struct proxy *proxy, struct server *server,
 
     sts = uv_listen((uv_stream_t *)&stream->u.tcp, maxpending, on_client_connection);
     if (sts != 0) {
-	pmNotifyErr(LOG_ERR, "%s: %s - uv_listen failed [%s]: %s\n",
-			pmGetProgname(), "open_request_port",
+	pmNotifyErr(LOG_ERR, "%s: %s - uv_listen failed port=%d [%s]: %s\n",
+			pmGetProgname(), "open_request_port", port,
 			uv_err_name(sts), uv_strerror(sts));
 	uv_close(handle, NULL);
 	return -ENOTCONN;
